/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.audiocd;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.console.forms.OnlineSearchForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.onlinesearch.AsinSearchMode;
import net.sf.dc.core.objects.onlinesearch.BarcodeSearchMode;
import net.sf.dc.core.objects.onlinesearch.DcAmazonServer;
import net.sf.dc.core.objects.onlinesearch.DcFreeDbServer;
import net.sf.dc.core.objects.onlinesearch.DcMusicBrainzServer;
import net.sf.dc.core.objects.onlinesearch.DcRegion;
import net.sf.dc.core.objects.onlinesearch.DcServer;
import net.sf.dc.core.objects.onlinesearch.KeywordSearchMode;
import net.sf.dc.core.objects.onlinesearch.MbAlbumTitleSearch;
import net.sf.dc.core.objects.onlinesearch.MbArtistSearch;
import net.sf.dc.freedb.Freedb;
import net.sf.dc.processes.http.AmazonAudioCdUpdate;
import net.sf.dc.processes.http.FreeDBAudioCDUpdate;
import net.sf.dc.processes.http.MusicBrainzUpdate;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class OnlineAudioCdSearchForm extends OnlineSearchForm {
	
    public OnlineAudioCdSearchForm(ItemForm itemForm, boolean advanced) {
        super(itemForm, null, itemForm, advanced);
    }
    
    public OnlineAudioCdSearchForm(boolean advanced) {
        super(DataCrow.mainFrame, null, MainFrame._INSERTTAB, advanced);
    }
    
    public String getDefaultServer() {
        return DcSettings.getValueAsString(Repository.settings.stAudioCdOnlineUpdateServer);
    }
    
    protected Collection getLanguages() {
        return null;
    }
    
    protected Collection getSearchModes() {
        return null;
    }
    
    public Collection getServerDefinitions() {
        Collection servers = new ArrayList();
        DcMusicBrainzServer musicBrainzServer = new DcMusicBrainzServer("MusicBrainz", "http://musicbrainz.org/ws/1/", true);
        musicBrainzServer.addRegion(new DcRegion("uk", "The main server", "http://musicbrainz.org/ws/1"));
        musicBrainzServer.addMode(new MbAlbumTitleSearch());
        musicBrainzServer.addMode(new MbArtistSearch());
        
        DcFreeDbServer freeDbServer = new DcFreeDbServer("FreeDB Servers", "http://freedb.freedb.org", false);
        boolean freedbReachable = false;
        try {
            String[] c = new Freedb("freedb.org:80", this).getAvailableServers();
            for (int i = 0; i < c.length; i++) {
                freeDbServer.addRegion(new DcRegion(c[i], c[i], c[i]));
            }
            freedbReachable = true;
        } catch (Exception exp) {}
        
        
        DcServer amazonServer = new DcAmazonServer();
        amazonServer.addMode(new KeywordSearchMode("Music", "Keyword"));
        amazonServer.addMode(new BarcodeSearchMode("Music", "Barcode"));
        amazonServer.addMode(new AsinSearchMode("Asin"));        
        
        servers.add(musicBrainzServer);
        servers.add(amazonServer);
        if (freedbReachable)
            servers.add(freeDbServer);
        
        return servers;        
    }
    
    public JButton getSearchButton() {
        DcServer server = getServer();
        
        JButton button;
        if (server.isKeywordSearchAllowed()) {
            button = ComponentFactory.getButton(DcResources.getText("lblFind"));
            button.addActionListener(new SearchAction());
        } else {
            button = ComponentFactory.getButton(DcResources.getText("lblCDSearch"));
            button.addActionListener(new SearchDiscIDAction());
            button.setMnemonic('F');
        }
        return button;
    }
    
    public void saveSettings() {
        super.saveSettings();
        DcServer server = getServer();
        DcSettings.setValue(Repository.settings.stMovieOnlineUpdateServer, server.getName() + "," + getServerAddress());         
    }
    
    public void startSearch() {
        saveSettings();
        setCancelMode();

        if (getServer() instanceof DcFreeDbServer)
            process = new FreeDBAudioCDUpdate(this, getRegion());
        else if (getServer() instanceof DcAmazonServer)
            process = new AmazonAudioCdUpdate(this, getSearchString(), getMode(), getRegion());
        else if (getServer() instanceof DcMusicBrainzServer)
            process = new MusicBrainzUpdate(this, getSearchString(), getMode(), getRegion());
        
        process.setPriority(Thread.MAX_PRIORITY);
        process.start();
    }    
    
    private class SearchDiscIDAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            startSearch();
        }
    }
}
