/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.BrowserDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

public class DcFileField extends JComponent implements ISettingsComponent {

    private final JTextField text;
    private final JButton button;

    private ActionListener al;
    private FileFilter filter;

    private String title = DcResources.getText("lblFileBrowser");
    private File file = null;

    private final ShowSaveDialogAction showSaveDialogAction = new ShowSaveDialogAction();
    private final ShowSelectDirectoryDialogAction showSelectDirectoryDialogAction = new ShowSelectDirectoryDialogAction();
    private final ShowSelectDialogAction showSelectFilesDialogAction = new ShowSelectDialogAction();

    public DcFileField(FileFilter filter) {
        this.filter = filter;
        text = ComponentFactory.getTextFieldDisabled();

        button = ComponentFactory.getButton(IconLibrary._icoOpen);
        button.setPreferredSize(new Dimension(25, 22));
        button.setMaximumSize(new Dimension(25, 22));
        button.setMinimumSize(new Dimension(25, 22));
        
        this.setLayout(Layout.getGBL());

        add( text,   Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                    ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                     new Insets(0, 0, 0, 0), 0, 0));
        button.setPreferredSize(new Dimension(25, 22));
        add( button, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                     new Insets(0, 0, 0, 0), 0, 0));
    }

    public void setModus(boolean save, boolean dirsOnly) {
        button.removeActionListener(showSaveDialogAction);
        button.removeActionListener(showSelectDirectoryDialogAction);
        button.removeActionListener(showSelectFilesDialogAction);

        if (save) {
            this.title = DcResources.getText("lblCreateNewFile");
            button.addActionListener(showSaveDialogAction);
        } else {
            if (dirsOnly) {
                this.title = DcResources.getText("lblSelectDirectory");
                button.addActionListener(showSelectDirectoryDialogAction);
            } else {
                this.title = DcResources.getText("lblSelectFile");
                button.addActionListener(showSelectFilesDialogAction);
            }
        }
    }


    public void setFileFilter(FileFilter filter) {
        this.filter = filter;
    }

    public File getFile() {
        return file;
    }

    public String getFileAsString() {
    	return file == null ? "" : file.toString();
    }

    public void setFile(File file) {
        this.file = file;
        text.setText(file == null ? "" : file.toString());
    }

    protected void showFileSaveDialog() {
        BrowserDialog dialog = new BrowserDialog(title, filter);
        file = dialog.showCreateFileDialog(this, file);
        text.setText(getFileAsString());

        if (file != null) {
            try {
                RandomAccessFile random = new RandomAccessFile(file, "rw");
                random.close();
            } catch (Exception exp) {
                new MessageBox( DcResources.getText("msgFileCouldNotbeCreated"), MessageBox._ERROR);
                Messages.add(
                    DcResources.getText("msgFileCouldNotbeCreated"),
                    Messages._ERROR);
                Messages.add(exp);
            }
        }
        dialog.dispose();
        dialog = null;
    }

    protected void showFileOpenDialog() {
        BrowserDialog dialog = new BrowserDialog(title, filter);

        file = dialog.showOpenFileDialog(this, null);
        text.setText(getFileAsString());

        if (al != null) {
        	al.actionPerformed(new ActionEvent(this.getParent(), 1, ""));
        }

        dialog.dispose();
        dialog = null;
    }

    protected void showDirectoryOpenDialog() {
        BrowserDialog dialog = new BrowserDialog(title, filter);

        file = dialog.showSelectDirectoryDialog(this, null);
        text.setText(getFileAsString());

        dialog.dispose();
        dialog = null;
    }

    public void addListener(ActionListener al) {
        this.al = al;
    }

    private class ShowSelectDirectoryDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showDirectoryOpenDialog();
        }
    }

    private class ShowSelectDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileOpenDialog();
        }
    }

    private class ShowSaveDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileSaveDialog();
        }
    }

	public void setValue(Object value) {
        if (value != null && value.toString().length() > 0) {
        	File file = new File(value.toString());
        	setFile(file);
        } else {
        	setFile(null);
        }
	}

	public Object getValue() {
		return file == null ? "" : file.toString();
	}
}
