/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;

public class DcList extends JList {

    public DcList(Collection data) {
        this();
        this.setListData(data.toArray());
    }
	
    public DcList() {
        setCellRenderer(new CustomCellRenderer());   
    }
    
    public Collection getData() {
    	Collection data = new ArrayList();
    	for (int i = 0; i < getModel().getSize(); i++) {
    		data.add(getModel().getElementAt(i));
    	}
    	return data;
    }
    
    public void setFont(Font font) {
        super.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));  
        for (Iterator iter = getData().iterator(); iter.hasNext(); ) {
            JPanel panel = (JPanel) iter.next();
            Component[] components = panel.getComponents();
            for (int j = 0; j < components.length; j++) {
                components[j].setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));
            }
        }
    }     
    
    public class CustomCellRenderer implements ListCellRenderer {
        public Component getListCellRendererComponent(
                JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            
            Component component = (Component) value;
            Color selectionColor = DcSettings.getValueAsColor(Repository.settings.stRowSelectionColor);
            component.setBackground(isSelected ? selectionColor : Color.WHITE);
            return component;
        }
    }    
}
