/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

public class DcNumberField extends JTextField implements ISettingsComponent {

    private final NumberFormat integerFormatter;

    public DcNumberField() {
        super();
        integerFormatter = NumberFormat.getNumberInstance(Locale.US);
        integerFormatter.setParseIntegerOnly(true);
    }

    protected Document createDefaultModel() {
        return new WholeNumberDocument();
    }

    /**
     * Returns -1 (as an Integer) when no value has been set,
     * else the Integer value of this field is returned 
     */
    public Object getValue() {
        Object o = null;
        try {
            o = new Integer(integerFormatter.parse(getText()).intValue());
        } catch(ParseException _ex) {}
        return o;
    }

    public void setValue(int i) {
        setText(integerFormatter.format(i));
    }

    /**
     * Sets the value. The value is first transformed to a String and
     * then parsed back to an int. If this is not successfull, 0 is set
     */
    public void setValue(Object o) {
        int i = 0;
        try {
            i = integerFormatter.parse(o.toString()).intValue();
        } catch (Exception ignore) {}
        
        o = new Integer(i);

        setText(integerFormatter.format(((Integer) o).intValue()));
    }

    /**
     * Document implementation. This document makes sure that only 
     * digits can be entered by the user. 
     */
    protected class WholeNumberDocument extends PlainDocument {

        protected WholeNumberDocument() {
        }

        public void insertString(int i, String s, AttributeSet attributeset)
        throws BadLocationException {

            char ac[] = s.toCharArray();
            char ac1[] = new char[ac.length];
            int j = 0;
            for(int k = 0; k < ac1.length; k++)
                if(Character.isDigit(ac[k]))
                    ac1[j++] = ac[k];

            super.insertString(i, new String(ac1, 0, j), attributeset);
        }
    } 
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent( g2d );
    }    
}
