/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcPanel extends JPanel {

	private String helpIndex = "";

    private final TitledBorder border = ComponentFactory.getTitleBorder(DcResources.getText("lblStatus"));
    private final JLabel labelStatus = new JLabel();
    private final DcProgressBar progress = new DcProgressBar();
    private final String title;
    private final ImageIcon icon;

    private final JPanel panelStatus = new JPanel();

    public DcPanel(String title, ImageIcon icon) {
        this.title = title;
        this.icon = icon;
        createStatusPanel();
    }

    public String getTitle() {
    	return title;
    }

    public ImageIcon getIcon() {
    	return icon;
    }

    public void setFont(Font font) {
        super.setFont(font);

        if (labelStatus != null) {
            Font system = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
            labelStatus.setFont(system);
            border.setTitleFont(system);
        }
    }

    public void setHelpIndex(String helpIndex) {
        this.helpIndex = helpIndex;
    }

    public String getHelpIndex() {
    	return helpIndex;
    }

    public void setStatus(String text) {
        if (text != null)
            labelStatus.setText(text);
    }

    public void initProgressBar(int maxValue) {
        progress.setValue(0);
        progress.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progress.setValue(value);
    }

    public JPanel getStatusPanel() {
    	return panelStatus;
    }

    private void createStatusPanel() {
        panelStatus.setLayout(Layout.getGBL());

        labelStatus.setBorder(border);
        labelStatus.setText(Repository.system.version);
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progress, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 0, 0, 0), 0, 0));

        panelStatus.add(labelStatus, Layout.getGBC(0, 0, 1, 1, 20.0, 20.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 0, 0, 0), 0, 0));
        panelStatus.add(panelProgress, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(2, 0, 2, 0), 0, 0));
    }
}
