/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JLabel;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.objects.book.BookCategory;

public class DcBookListHwElement extends DcObjectListHwElement {

    private JTextArea descriptionField;
    
    public DcBookListHwElement(DcObject dco) {
        super(dco);
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        if (descriptionField != null)
            descriptionField.setBackground(new Color(color.getRed() - 20, color.getGreen() - 20, color.getBlue() - 20));
    }     
    
    public Collection getPictures() {
        Collection pictures = new ArrayList();
        pictures.add(dco.getValue(Book._K_PICTUREFRONT));
        return pictures;
    }

    public void build() {
        setLayout(Layout.getGBL());
        
        JLabel titleLabel = getLabel(Book._A_TITLE, true, label1Length);
        if (DcModules.getCurrentModule().getIndex() == DcModules._GENERIC)
            titleLabel.setIcon(dco.getModule().getIconSmall());

        addComponent(titleLabel, 0, 0);
        addComponent(getLabel(Book._A_TITLE, false, field1Length), 1, 0);
        addComponent(getLabel(Book._I_CATEGORY, true, label2Length), 2, 0);
        
        JLabel label = getLabel(Book._I_CATEGORY, false, field2Length);
        try {
            BookCategory category = (BookCategory) dco.getValue(Book._I_CATEGORY);
            if (category != null)
                label.setIcon(category.getIcon());
        } catch (Exception exp) {}
        addComponent(label, 3, 0);
        
        addComponent(getLabel(Book._G_AUTHOR, true, label1Length), 0, 1);
        addComponent(getLabel(Book._G_AUTHOR, false, field1Length), 1, 1);
        addComponent(getLabel(DcMediaObject._E_RATING, true, label2Length), 2, 1);
        addComponent(getRatingValueLabel(), 3, 1);        
        
        descriptionField = ComponentFactory.getTextArea();
        descriptionField.setPreferredSize(new Dimension(600, 120));
        descriptionField.setText(getShortDescription((String) dco.getValue(Book._B_DESCRIPTION)));
        add(descriptionField, Layout.getGBC( 0, 2, 4, 1, 10.0, 10.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(0, 5, 5, 0), 0, 0));
        add(getPicturePanel(getPictures()), Layout.getGBC( 4, 0, 1, 3, 10.0, 10.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(1, 1, 1, 1), 0, 0));           
    }  
}
