/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JList;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.renderers.DcObjectListRenderer;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.contactperson.ContactPersonModule;
import net.sf.dc.core.objects.DcCategory;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcState;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.template.ITemplate;
import net.sf.dc.resources.DcResources;

public class DcObjectList extends JList {

    private boolean hwElements;
    
    public DcObjectList(DcModule module, boolean hwElements, boolean wrap) {
        super(new DcObjectListModel());
        
        this.hwElements = hwElements;
        
        setCellRenderer(new DcObjectListRenderer());
        setBackground(ComponentFactory.getDisabledColor());
        
        if ((module instanceof DcMediaModule|| module instanceof ContactPersonModule) && wrap)
            setLayoutOrientation(JList.HORIZONTAL_WRAP);
    }

    public void setSelectionMode(int selectionMode) {
        getSelectionModel().setSelectionMode(selectionMode);        
    }
    
    public DcObjectListModel getDcModel() {
        if (!(getModel() instanceof DcObjectListModel))
            setModel(new DcObjectListModel());
            
        return (DcObjectListModel) getModel();
    }
    
    public int[] getChangedIndices() {
        Collection rows = new ArrayList();
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (element.getDcObject() != null && element.getDcObject().isChanged()) 
                rows.add(new Integer(i));
        }
        
        int[] indices = new int[rows.size()];
        int counter = 0;
        for (Iterator iter = rows.iterator(); iter.hasNext(); counter++) {
            indices[counter] = ((Integer) iter.next()).intValue();
        }
        return indices;        
    }
    
    public void markAsUnchanged() {
        int[] rows = getChangedIndices();
        for (int i = 0; i < rows.length; i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            element.getDcObject().markAsUnchanged(); 
        }
    }    
    
    public DcObject getObjectForID(String ID) {
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (element.getDcObject() != null && element.getDcObject().getID().equals(ID)) 
                return element.getDcObject();
        }
        return null;
    }    
    
    public Collection getObjects() {
        Collection objects = new ArrayList();
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (element.getDcObject() != null) 
                objects.add(element.getDcObject());
        }
        return objects;
    }
    
    public Collection getSelectedObjects() {
        int[] indices = getSelectedIndices();
        Collection objects = new ArrayList();

        if (indices == null || indices.length == 0) {
            new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
        } else {
            for (int i = 0; i < indices.length; i++) {
                DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(indices[i]);
                objects.add(element.getDcObject());
            }
        }
        return objects;
    }
    
    public void update() {
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            element.update();
        }
    }
    
    public void remove() {
        getDcModel().remove(getSelectedIndex());
    }

    public void remove(int[] indices) {
        for (int i = 0; i < indices.length; i++) {
            getDcModel().removeElementAt(indices[i]);
        }
    }    
    
    public void remove(String[] ids) {
        for (int i = 0; i < ids.length; i++) {
            getDcModel().removeElement(getElementForObjectID(ids[i]));
        }
    }

    public void updateElement(int index, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(index);
        element.update(dco, overwrite, allowDeletes, mark);
        
        try {
            setSelectedValue(getDcModel().getElementAt(index), true);
        } catch (Exception ignore) {}
    }
    
    public void updateElement(String objectID, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        DcObjectListElement element = getElementForObjectID(objectID);
        if (element != null)
            element.update(dco, overwrite, allowDeletes, mark);
        
        try {
            setSelectedValue(getElementForObjectID(objectID), true);
        } catch (Exception ignore) {}
    }
    
    public void clear() {
        try {
           for (int i = 0 ; i < getDcModel().getSize(); i++) {
              DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
              element.clear();
              remove(element);
           }
        } catch (Exception ignore) {}
        
        setModel(new DcObjectListModel());
    }
    
    private DcObjectListElement getElementForObjectID(String objectID) {
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (objectID.equals(element.getDcObject().getID()))
                return element;
        }
        return null;
    }    
    
    public DcObject getSelectedObject() {
        DcObjectListElement element = (DcObjectListElement) getSelectedValue();
        
        DcObject dco = null;
        if (element != null) 
            dco = element.getDcObject(); 
        
        return dco;
    }
    
    public void listenersEnabled(boolean b) {
        getDcModel().setListenersEnabled(b);        
    }
    
    public void add(DcObject dco) {
        if (dco.getID() == null || dco.getID().equals("")) 
            dco.setIDs();        
        
        dco.markAsUnchanged();
        getDcModel().addElement(getDisplayElement(dco));
    }
    
    public void add(DcObject[] objects) {
        listenersEnabled(false);
        
        for (int i = 0; i < objects.length; i++) {
            DcObject dco = objects[i];
            if (dco.getID() == null || dco.getID().equals("")) 
                dco.setIDs(); 
            
            add(dco);
        }
        
        listenersEnabled(true);
        
        getDcModel().fireIntervalAdded(getDcModel(), 0, objects.length - 1);
    }    
    
    public void add(Collection objects) {
        listenersEnabled(false);
        
        for (Iterator iter = objects.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
            
            if (dco.getID() == null || dco.getID().equals("")) 
                dco.setIDs(); 
            
            add(dco);
        }
        
        listenersEnabled(true);
        
        getDcModel().fireIntervalAdded(getDcModel(), 0, objects.size() - 1);
    }
    
    public DcObjectListElement getDisplayElement(DcObject dco) {
        DcObjectListElement element;
        DcModule module = dco.getModule();
        
        if (module.getIndex() == DcModules._AUDIOCD) { 
            if (hwElements)
                element = new DcAudioCDListHwElement(dco);
            else 
                element = new DcAudioCDListElement(dco);
        } else if (module.getIndex() == DcModules._MUSICALBUM) {
            element = new DcMusicAlbumListElement(dco);
        } else if (module.getIndex() == DcModules._MUSICTRACK) {
            element = new DcMusicTrackListElement(dco);
        } else if (module.getIndex() == DcModules._SOFTWARE) {
            if (hwElements)
                element = new DcSoftwareListHwElement(dco);
            else 
                element = new DcSoftwareListElement(dco);
        } else if (dco instanceof DcCategory) {
            element = new DcCategoryListElement(dco);
        } else if (dco instanceof DcState) {
            element = new DcStateListElement(dco);
        } else if (dco instanceof DcStorageMedium) {
            element = new DcStorageMediumListElement(dco);
        } else if (module.getIndex() == DcModules._MOVIE) {
            if (hwElements)
                element = new DcMovieListHwElement(dco);
            else
                element = new DcMovieListElement(dco);
        } else if (module.getIndex() == DcModules._CONTACTPERSON) {
            element = new DcContactPersonListElement(dco);
        } else if (module.getIndex() == DcModules._SOFTWAREPLATFORM) {
            element = new DcPlatformListElement(dco);
        } else if (module.getIndex() == DcModules._AUDIOTRACK) {
            element = new DcAudioTrackListElement(dco);
        } else if (module.getIndex() == DcModules._IMAGE) {
            element = new DcImageListElement(dco);
        } else if (dco instanceof ITemplate) {
            element = new DcTemplateListElement(dco);
        } else { 
            if (hwElements)
                element = new DcBookListHwElement(dco);
            else
                element = new DcBookListElement(dco);
        }

        return element;
    }    
}
