package net.sf.dc.console.components.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.RefreshChildView;

public class ChildSearchTablePanel extends TableSearchView {

    private int parentModIdx;
    
    public ChildSearchTablePanel(int module, int parentModIdx) {
        super(DcModules.getModule(module), DcModules.getModule(module).getObjectName());
        
        this.parentModIdx = parentModIdx;
        
        table.getSelectionModel().removeListSelectionListener(rowSelectionListener);
        table.removeKeyListener(changeRowByKey);
    }
    
    public JComponent[] getExtraComponents() {
        JButton buttonAddTrack = ComponentFactory.getButton(DcResources.getText("lblAddTrack"));
        buttonAddTrack.addActionListener(new AddTrackAcionListener());
        buttonAddTrack.setMnemonic('T');
        return new JComponent[] {buttonAddTrack};
    }
    
    /**
     * Indicates whether there are unsaved changes
     */
    public boolean changesSaved() {
        return table.noChanges();
    }    
    
    protected void addAfterDeleteRequest(DcObject dco) {
        dco.addRequest(new RefreshChildView(null, parentModIdx, null, null, getParentID(), false));
    }    
    
    /**
     * Clears the table and its cache
     */
    public void clear() {
        table.clearTable();
        table.clearCache();
    }
    
    public void loadChildValues() {}
    
    private class AddTrackAcionListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            if (getParentID() != null && !getParentID().equals("")) {
                DcObject dco = getModule().getDcObject();
                dco.setIDs();
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
                open(dco, false);
            } else {
                new MessageBox(DcResources.getText("msgAddSelectAlbum"), MessageBox._ERROR);
            }
        }
    }  
}
