/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDateField;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.forms.LoanForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.objects.contactperson.ContactPerson;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.CloseWindowRequest;
import net.sf.ext.util.Utilities;

public class LoanPanel extends JPanel {

    private DcObject dco;
    
    private Collection objects;
    private Loan loan;
    private DcFrame owner;
    
    private DcDateField inputEndDate = ComponentFactory.getDateField();    
    private DcDateField inputStartDate = ComponentFactory.getDateField();
    private JComboBox comboPersons = ComponentFactory.getContactPersonCombo();
    private DcTextPane descriptionPane;
    
    private final HTMLEditorKit kit = new HTMLEditorKit();
    private final HTMLDocument document = new HTMLDocument();
    
    public LoanPanel(DcObject dco, DcFrame owner) {
        objects = new ArrayList();
        objects.add(dco);
        this.dco = dco;
        this.owner = owner;
        
        buildPanel(DataManager.getCurrentLoan(dco.getID()).isAvailable(dco.getID())); 
        setLoanInformation(DataManager.getCurrentLoan(dco.getID()));
    }    
    
    public LoanPanel(Collection objects, DcFrame owner) throws Exception {
        this.objects = objects;
        this.owner = owner;
        
        int counter = 0;
        Boolean available = null;
        Loan l = new Loan();
        for (Iterator iter = objects.iterator(); iter.hasNext(); counter++) {
            DcMediaObject o = (DcMediaObject) iter.next();
            dco = counter == 0 ? o : dco;
            boolean currentStatus = l.isAvailable(o.getID());
            available = available == null ? new Boolean(currentStatus) : available;
            
            if (available.booleanValue() != currentStatus) {
                new MessageBox(DcResources.getText("msgNotSameState"), MessageBox._WARNING);
                throw new Exception(DcResources.getText("msgNotSameState"));
            }
        }
        
        loan = DataManager.getCurrentLoan(dco.getID());
        buildPanel(loan.isAvailable(dco.getID())); 
        setLoanInformation(loan);
    }
    
    private void setLoanInformation(Loan loan) {
        Date start = (Date) loan.getValue(Loan._A_STARTDATE);
        start = start == null ? new Date() : start;
        inputStartDate.setValue(start); 
        
        String personID = (String) loan.getValue(Loan._C_CONTACTPERSONID);

        if (!loan.isAvailable(dco.getID())) { 
            if (objects.size() == 1) {
                String s;
                if (loan.getDaysLoaned().intValue() == 0)
                    s = DcResources.getText("msgLoanInformationToday", new String[] {dco.toString(), 
                            ((ContactPerson) DataManager.getObject(DcModules._CONTACTPERSON, personID)).toString()});
                else if (loan.getDaysLoaned().intValue() == 1)
                    s = DcResources.getText("msgLoanInformationYesterday", new String[] {dco.toString(),
                            ((ContactPerson) DataManager.getObject(DcModules._CONTACTPERSON, personID)).toString()});
                else
                    s = DcResources.getText("msgLoanInformation", new String[] {dco.toString(), 
                            ((ContactPerson) DataManager.getObject(DcModules._CONTACTPERSON, personID)).toString(), 
                            loan.getDaysLoaned().toString()});
                
                setDescriptionHtml(s);
            } else {
                setDescriptionHtml(DcResources.getText("msgAllItemsLoanInformation", "" + objects.size()));
            }
        } else {
            if (objects.size() == 1)
                setDescriptionHtml(DcResources.getText("msgItemIsAvailable"));
            else 
                setDescriptionHtml(DcResources.getText("msgAllItemsAreAvailable", "" + objects.size()));
        }
        
        if (personID != null && personID.length() > 0) {
            for (int i = 0; i < comboPersons.getItemCount(); i++) {
                Object o =  comboPersons.getItemAt(i);
                if (o instanceof ContactPerson) {
                    ContactPerson person = (ContactPerson) comboPersons.getItemAt(i);
                    if (person != null && person.getID().equals(personID)) {
                        comboPersons.setSelectedIndex(i);
                        break;
                    }
                }
            }
        }
    }
    
    protected void setDescriptionHtml(String s) {
        try {
            Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
            String hexColor = Utilities.getHexColor(ComponentFactory.getCurrentForegroundColor());
            
            String fontstyle = "";
            if (font.isItalic())
                fontstyle = ";font-style:italic";
            else if (font.isBold())
                fontstyle = ";font-weigth:bolder";
            
            String style = "style=" + '"' + 
                           ";font-family:" + font.getFamily() + 
                           ";font-size:" + font.getSize() + 
                           fontstyle + 
                           ";color:" + hexColor + ";" + '"';            
            
            String html = "<html><body " + style + ">" + s + "</body></html>";
            descriptionPane.setText("");
            kit.insertHTML(document, 0, html, 0, 0, null);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }    
    
    public void returnItems() {
        final Date endDate = (Date) inputEndDate.getValue();
        if (endDate == null) {
            new MessageBox(DcResources.getText("msgEnterReturnDate"), MessageBox._WARNING);
            return;
        }
        
        for (Iterator iter = objects.iterator(); iter.hasNext(); ) {
            DcObject o = (DcObject) iter.next();
            Loan currentLoan = DataManager.getCurrentLoan(o.getID());
            Date startDate = (Date) currentLoan.getValue(Loan._A_STARTDATE);
            if (startDate.compareTo(endDate) > 0) {
                new MessageBox(DcResources.getText("msgEndDateMustBeAfterStartDate"), MessageBox._WARNING);
                return;
            }
        }
        
        Thread thread = new Thread(new Runnable() {
            public void run() {
                try {
                    int i = 0;
                    for (Iterator iter = objects.iterator(); iter.hasNext(); i++) {
                        DcObject o = (DcObject) iter.next();
            
                        Loan currentLoan = DataManager.getCurrentLoan(o.getID());
                        if (currentLoan.getID() != null) {
                            currentLoan.setValue(Loan._D_OBJECTID, o.getID());
                            currentLoan.setValue(Loan._B_ENDDATE, endDate);
                
                            currentLoan.setBatch(i == 0);
                            currentLoan.setEndOfBatch(i == objects.size()  - 1);
                            currentLoan.setSilence(true);
                            
                            if (owner != null && currentLoan.isEndOfBatch()) 
                                currentLoan.addRequest(new CloseWindowRequest(owner));
                            else if (owner == null)
                                setLoanInformation(currentLoan);
                            
                            o.setValue(DcObject._SYS_AVAILABLE, new Boolean(currentLoan.isAvailable(o.getID())));
                            o.setValue(DcObject._SYS_LOANEDBY, currentLoan.getPersonDescription());
                            o.setValue(DcObject._SYS_LOANDURATION, currentLoan.getDaysLoaned());                
                            
                            //currentLoan.addRequest(new UpdateRowRequest(MainFrame._SEARCHTAB, o, true, true, false));
                            currentLoan.saveUpdate(true);
                        }
                        
                        try {
                            wait(100);
                        } catch (Exception ignore) {}                        
                    }        
                } catch (ValidationException exp) {
                    Messages.add(exp);

                }
             }}
        );
        thread.start();
        
        removeAll();
        buildPanel(true);
        revalidate();    
        repaint();
    }

    
    public void lendItems() {
        ContactPerson contactPerson = comboPersons.getSelectedItem() instanceof ContactPerson ? (ContactPerson) comboPersons.getSelectedItem() : null;
        final String contactPersonID = contactPerson != null ? contactPerson.getID() : null;
        final Date startDate = (Date) inputStartDate.getValue();
        
        if (contactPersonID == null) {
            new MessageBox(DcResources.getText("msgSelectPerson"), MessageBox._WARNING);
            return;
        } else if (startDate == null) {
            new MessageBox(DcResources.getText("msgEnterDate"), MessageBox._WARNING);
            return;
        }
        
        Thread thread = new Thread(new Runnable() {
            public void run() {
                try {
                    int i = 0;
                    for (Iterator iter = objects.iterator(); iter.hasNext(); i++) {
                        DcObject o = (DcObject) iter.next();
            
                        Loan currentLoan = DataManager.getCurrentLoan(o.getID());
                        currentLoan.setValue(Loan._D_OBJECTID, o.getID());
                        currentLoan.setValue(Loan._A_STARTDATE, startDate);
                        currentLoan.setValue(Loan._C_CONTACTPERSONID, contactPersonID);
                        currentLoan.setSilence(true);
            
                        currentLoan.setBatch(i == 0);
                        currentLoan.setEndOfBatch(i == objects.size() - 1);
                        
                        if (owner != null && currentLoan.isEndOfBatch())
                            currentLoan.addRequest(new CloseWindowRequest(owner));
                        else if (owner == null)
                            setLoanInformation(currentLoan);                        
        
                        o.setValue(DcObject._SYS_AVAILABLE, new Boolean(currentLoan.isAvailable(o.getID())));
                        o.setValue(DcObject._SYS_LOANEDBY, currentLoan.getPersonDescription());
                        o.setValue(DcObject._SYS_LOANDURATION, currentLoan.getDaysLoaned());                
                        
//                        currentLoan.addRequest(new UpdateRowRequest(MainFrame._SEARCHTAB, o, true, true, false));
                        
                        if (currentLoan.getID() != null && currentLoan.getID().length() > 0) 
                        	currentLoan.saveUpdate(true);
                        else 
                            currentLoan.saveNew(true);
                        
                        try {
                            wait(100);
                        } catch (Exception ignore) {}
                    }  
                } catch (ValidationException exp) {
                    Messages.add(exp);
                }
            }}
        );
        thread.start();    
        
        removeAll();
        buildPanel(false);
        repaint();
        revalidate();
    }
    
    private void buildPanel(boolean isAvailable) {
        setLayout(Layout.getGBL());

        //**********************************************************
        //Input panel
        //**********************************************************
        JPanel panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());
        
        JLabel labelEndDate = ComponentFactory.getLabel(DcResources.getText("lblEndDate"));
        JLabel labelStartDate = ComponentFactory.getLabel(DcResources.getText("lblStartDate"));
        JLabel labelPerson = ComponentFactory.getLabel(DcResources.getText("lblContactPerson"));
        
        if (isAvailable) {
            labelStartDate.setIcon(IconLibrary._icoCalendar);
            labelPerson.setIcon(IconLibrary._icoPersonModule16);
            
            panelInput.add(labelStartDate , Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5,  5), 0, 0));
            panelInput.add(inputStartDate , Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5,  5), 0, 0));
            panelInput.add(labelPerson ,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5,  5), 0, 0));
            panelInput.add(comboPersons ,   Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5,  5), 0, 0));
        } else {
            labelEndDate.setIcon(IconLibrary._icoCalendar);
            
            panelInput.add(labelEndDate , Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5,  5), 0, 0));
            panelInput.add(inputEndDate , Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5,  5), 0, 0));
        }
            
        //**********************************************************
        //Description panel
        //**********************************************************
        descriptionPane = ComponentFactory.getTextPane();
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);
        descriptionPane.setBounds(1,1,1,1);
        
        descriptionPane.setEditable(false);
        JScrollPane scroller = new JScrollPane(descriptionPane);
        scroller.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblDescription")));
        
        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions = new JPanel();
 
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        buttonCancel.addActionListener(new CloseButtonAction());
        buttonCancel.setMnemonic('C');

        if (isAvailable) {
            JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblLendItem"));
            buttonSave.addActionListener(new SaveButtonAction());
            panelActions.add(buttonSave);
        } else {
            JButton buttonReturn = ComponentFactory.getButton(DcResources.getText("lblReturnItem"));
            buttonReturn.addActionListener(new ReturnItemButtonAction());
            panelActions.add(buttonReturn);
        }
        
        if (owner != null && objects.size() > 1)
            panelActions.add(buttonCancel);

        
        //**********************************************************
        //Main
        //**********************************************************
        add( scroller,  Layout.getGBC( 0, 0, 1, 1, 5.0, 5.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 5, 5, 5, 5), 0, 0));
        add( panelInput,        Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 5, 5, 5), 0, 0));
        add( panelActions,      Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        
        
        if (objects.size() == 1) {
            DcObject dco = (DcObject) objects.toArray()[0];
            
            DcTable tableLoans = ComponentFactory.getDCTable(DcModules.getModule(DcModules._LOAN), true, false);
            JScrollPane scrollHistory = new JScrollPane(tableLoans);
            scrollHistory.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLoanHistory")));
            
            try {
                Collection loans = DataManager.getLoans(dco.getID()); 
                for (Iterator iter = loans.iterator(); iter.hasNext(); ) {
                    Loan loan = (Loan) iter.next();
                    
                    if (loan.getValue(Loan._B_ENDDATE) != null) 
                        tableLoans.addObject(loan, true);
                }
                
                add( scrollHistory,   Layout.getGBC( 0, 3, 1, 1, 4.0, 4.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                     new Insets( 5, 5, 5, 5), 0, 0));                

            } catch (Exception exp) {
                Messages.add(exp);
                new MessageBox(exp.getMessage(), MessageBox._ERROR);
            }
        }      
        
        if (owner != null && objects.size() == 1) {
            add( buttonCancel,   Layout.getGBC( 0, 4, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0)); 
        }
    }
    
    public class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    } 

    public class ReturnItemButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            returnItems();
        }
    }    
    
    public class SaveButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            lendItems();
        }
    }  
    
    private void close() {
        this.descriptionPane = null;
        this.dco = null;
        this.loan = null;
        this.comboPersons = null;
        this.objects.clear();
        this.objects = null;
        
        if (owner instanceof LoanForm) { 
            ((LoanForm) owner).close();
        } else {
            owner.dispose();
            owner.setVisible(false);
        }
        
        this.owner = null;
    }
}
