package net.sf.dc.console.components.panels;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.objects.onlinesearch.DcAmazonServer;
import net.sf.dc.core.objects.onlinesearch.DcMusicBrainzServer;
import net.sf.dc.core.objects.onlinesearch.DcRegion;
import net.sf.dc.core.objects.onlinesearch.DcServer;
import net.sf.dc.core.objects.onlinesearch.KeywordSearchMode;
import net.sf.dc.core.objects.onlinesearch.MbAlbumTitleSearch;
import net.sf.dc.resources.DcResources;

public class OnlineAudioServicePanel extends JPanel {

    private JComboBox comboServers = ComponentFactory.getComboBox();
    private JComboBox comboRegions = ComponentFactory.getComboBox();
    
    private JCheckBox checkUseOnlineService = ComponentFactory.getCheckBox(DcResources.getText("lblUseOnlineService"));

    public OnlineAudioServicePanel() {
        build();
    }
    
    public boolean useOnlineService() {
        return checkUseOnlineService.isSelected();
    }
    
    public DcServer getServer() {
        return checkUseOnlineService.isSelected() ? (DcServer) comboServers.getSelectedItem() : null;
    }
    
    public DcRegion getRegion() {
        return checkUseOnlineService.isSelected() ? (DcRegion) comboRegions.getSelectedItem() : null;
    }
    
    private void applyServer() {
        DcServer server = getServer();
        comboRegions.removeAllItems();

        for (Iterator iter = server.getRegions().iterator(); iter.hasNext(); )
            comboRegions.addItem(iter.next());
    
        repaint();
    }    
    
    private void toggleServer() {
        boolean b = checkUseOnlineService.isSelected();
        comboRegions.setEnabled(b);
        comboServers.setEnabled(b);
    } 
    
    private void build() {
        setLayout(Layout.getGBL());
        
        DcMusicBrainzServer musicBrainzServer = new DcMusicBrainzServer("MusicBrainz", "http://musicbrainz.org/ws/1/", true);
        musicBrainzServer.addRegion(new DcRegion("uk", "The main server", "http://musicbrainz.org/ws/1"));
        musicBrainzServer.addMode(new MbAlbumTitleSearch());
        
        DcServer amazonServer = new DcAmazonServer();
        amazonServer.addMode(new KeywordSearchMode("Music", "Keyword"));

        // set the servers
        comboServers.addItem(musicBrainzServer);
        comboServers.addItem(amazonServer);
        
        comboServers.addActionListener(new ServerChangedListener());
        checkUseOnlineService.addActionListener(new UseOnlineServiceListener());
        
        add(checkUseOnlineService, Layout.getGBC(0, 0, 2, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets(5, 5, 5, 5), 0, 0));
        add(comboServers, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(5, 5, 5, 5), 0, 0));
        add(comboRegions, Layout.getGBC(1, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(5, 5, 5, 5), 0, 0));
        
        setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblOnlineServiceConfig")));
        
        checkUseOnlineService.setSelected(true);
        comboServers.setSelectedIndex(0);
    }
    
    private class UseOnlineServiceListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            toggleServer();
        }
    }      
    
    private class ServerChangedListener implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            applyServer();
        }
    }     
}
