/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.ext.util.Utilities;

public abstract class QuickViewPanel extends JPanel {
    
    public static final int _DIRECTION_HORIZONTAL = 0;
    public static final int _DIRECTION_VERTICAL = 1;    
    
    protected final DcModule module;
    protected final boolean showImages;
    
    protected DcObject dco;
    private LinkedList pictures = new LinkedList();
    private LinkedList imagePanels = new LinkedList();

    private JTextPane descriptionPane;
    private JScrollPane scroller;
    private final JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
    
    private final HTMLEditorKit kit = new HTMLEditorKit();
    private final HTMLDocument document = new HTMLDocument();
    
    public QuickViewPanel(DcModule module, boolean showImages) {
        this.module = module;
        this.showImages = showImages;
        setLayout(Layout.getGBL());
        
        tabbedPane.addChangeListener(new TabListener());        
        
        buildPanel();
        changeSize();
    }    
    
    public void setFont(Font font) {
        super.setFont(font);

        if (tabbedPane != null)
            tabbedPane.setFont(ComponentFactory.getSystemFont());
        
        if (dco != null) {
            setObject(dco);            
        }
    }
    
    private void loadImage() {
        int index = tabbedPane.getSelectedIndex() - 1;
        Picture picture =  (Picture) pictures.get(index);
        picture.loadImage();
        
        JPanel panel =  (JPanel) imagePanels.get(index);
        Component[] components =  panel.getComponents();
        for (int i = 0; i < components.length; i++) {
            if (components[i] instanceof DcPictureField) {
                ((DcPictureField) components[i]).setValue(picture.getValue(Picture._D_IMAGE));
                components[i].repaint();
            }
        }
    }
    
    private void setImages() {
        clearImages();

        if (dco == null)
            return;
        
        DcFieldDefinitions definitions = 
            (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(dco.getModule().getFieldDefinitionsSettingsKey()); 
        
        for (Iterator iter =  definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            if (    dco.isEnabled(definition.getIndex()) && 
                    dco.getField(definition.getIndex()).getValueType() == Repository.datatypes._PICTURE) {
                
                Picture picture = (Picture) dco.getValue(definition.getIndex());
                if (picture != null) {// && !picture.isDeleted()) {
                    pictures.add(picture);    
                    
                    DcPictureField picField = ComponentFactory.getPictureField(true, false, false);
                    picField.setMaximumSize(new Dimension(150,150));
                    picField.setMinimumSize(new Dimension(150,150));
                    picField.setPreferredSize(new Dimension(150,150));
                    picField.setBorder(ComponentFactory.getBorder());
                    
                    JPanel panel = new JPanel();
                    panel.setLayout(Layout.getGBL());
                    panel.addMouseListener(new QuickViewMouseListener());
                    panel.add(picField, Layout.getGBC(0, 0, 1, 1, 10.0, 10.0,
                            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(2, 2, 2, 2), 0, 0));
                        
                    tabbedPane.addTab(dco.getLabel(definition.getIndex()), IconLibrary._icoPicture, panel);
                    imagePanels.add(panel);
                }
            }
        }
    }
    
    public void setObject(DcObject dco) {
        int tab = tabbedPane.getSelectedIndex();
        clear();
        this.dco = dco;
        pictures.clear();
        
        String html = "<html><body " + getBodyStyle() + ">\n" +
                        getDescriptionTable() +
                      "</body> </html>";
        
        setDescriptionHtml(html);
        descriptionPane.setCaretPosition(0);
        
        setImages();
        
        boolean error = true;
        tab += 1;
        // prevent endless loop
        int counter = 0;
        while (error && counter < 6) {
            counter ++;
            try {
                tab -= 1;
                tabbedPane.setSelectedIndex(tab);
                error = false;
            } catch (Exception ignore) {}
        }
    }
    
    public void clear() {
        try {
            dco = null;
            descriptionPane.setText("");
            tabbedPane.setSelectedIndex(0);
            clearImages();
        } catch (Exception ignore) {}
        
        revalidate();
        repaint();
    }
    
    private void clearImages() {
        tabbedPane.removeAll();
        tabbedPane.addTab("Description", IconLibrary._icoInformation ,scroller);
        
        for (Iterator iter = imagePanels.iterator(); iter.hasNext(); ) {
            JPanel panel = (JPanel) iter.next();
            Component[] components = panel.getComponents();
            for (int i = 0; i < components.length; i++) {
                if (components[i] instanceof DcPictureField)
                    ((DcPictureField) components[i]).flush();
            }
            panel.removeAll();
        }
        pictures.clear();
        imagePanels.clear();
    }
    
    protected abstract String getDescriptionTable();
    
    protected void setDescriptionHtml(String html) {
        try {
            descriptionPane.setText("");
            kit.insertHTML(document, 0, html, 0, 0, null);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    protected String addTableRow(String table, int index, int textDirection) {
        if (dco.isEnabled(index)) {
            Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
            boolean horizontal = textDirection == _DIRECTION_HORIZONTAL;

            Object value = dco.getDisplayString(index);
            String realValue = dco.getValue(index) == null ? "" : dco.getValue(index).toString();
            String displayString = value == null ? "" : value.toString();
            if (realValue != null && displayString.length() > 0 ) {
                table += "<tr><td>\n";
                table += "<b>" + dco.getLabel(index) + "</b>";
                
                if (!horizontal) {
                    table += "</td></tr>";
                    table += "<tr><td>\n";
                } else {
                    table += " ";
                }
                
                if (font.getStyle() == Font.BOLD) 
                    value = "<b>" + value + "</b>";

                table += value;
                table += "</td></tr>";
                table += "<tr><td></td></tr>";
            } else if (dco.getField(index).getValueType() == Repository.datatypes._PICTURE) {
                Picture picture = (Picture) dco.getValue(index);
                pictures.add(picture);
            }
        }
        return table;
    }    
    
    private void buildPanel() {
        // description panel
        descriptionPane = ComponentFactory.getTextPane();
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);
        descriptionPane.setBounds(1,1,1,1);
        descriptionPane.addMouseListener(new QuickViewMouseListener());

        scroller = new JScrollPane(descriptionPane);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        //scroller.setBorder(null);

        
        // tabbed pane
        tabbedPane.addTab("Description", IconLibrary._icoInformation ,scroller);
        add(tabbedPane, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(0, 0, 0, 0), 0, 0));
        
        
        String html = "<html><body " + getBodyStyle() + "></body> </html>";
        setDescriptionHtml(html);
    }    
    
    private void showPopupMenu(int x, int y) {
        PopupMenu popupMenu = new PopupMenu();
        popupMenu.show(this, x, y);
    }    
    
    protected String getBodyStyle() {
        Color colorEven = DcSettings.getValueAsColor(Repository.settings.stEvenRowColor);
        Color colorOdd = DcSettings.getValueAsColor(Repository.settings.stOddRowColor);
        
        
        Color color = colorEven.getRGB() < colorOdd.getRGB() ? colorOdd : colorEven;
        String background = Utilities.getHexColor(color);
       
        descriptionPane.setBackground(color);
        for (Iterator iter = imagePanels.iterator(); iter.hasNext(); ) {
            ((JPanel) iter.next()).setBackground(color);
        }
        
        String size = DcSettings.getValueAsString(Repository.settings.stQuickViewSize);
        int width = 205;
        if (size.equals("50%"))
            width = (int) (width * 1.5);
        
        String hexColor = Utilities.getHexColor(ComponentFactory.getCurrentForegroundColor());
        Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        
        String fontstyle = "";
        if (font.isItalic())
            fontstyle = ";font-style:italic";
        else if (font.isBold())
            fontstyle = ";font-weigth:bolder";
        
        String style = "style=" + '"' + 
                       "width:" + width +  
                       ";background:" + background + 
                       ";font-family:" + font.getFamily() + 
                       ";font-size:" + font.getSize() + 
                       fontstyle +
                       ";color:" + hexColor + ";" + '"';
        return style;
    }
    
    public class PopupMenu extends JPopupMenu {
        public PopupMenu() {
            JMenuItem menuToggleLabels = ComponentFactory.getMenuItem(DcResources.getText("lblToggleQuickView"));
            JMenuItem menuSize25 = ComponentFactory.getMenuItem("25%");
            JMenuItem menuSize50= ComponentFactory.getMenuItem("50%");
            
            menuToggleLabels.addActionListener(new ToggleQuickViewAction());
            menuSize25.addActionListener(new ToggleSize25Action());
            menuSize50.addActionListener(new ToggleSize50Action());

            this.add(menuToggleLabels);
            this.add(new JSeparator());
            this.add(menuSize25);
            this.add(menuSize50);
        }
    }   
    
    public void changeSize() {
        String size = DcSettings.getValueAsString(Repository.settings.stQuickViewSize);

        int width = 230;
        if (size.equals("50%")) {
            width = (int) (width * 1.5);
        }
        
        setMinimumSize(new Dimension(width, 10));
        setPreferredSize(new Dimension(width, 2000));
        setMaximumSize(new Dimension(width, 2000));
        
        if (dco != null) {
            setObject(dco);
        } else {
            String html = "<html><body " + getBodyStyle() + "></body> </html>";
            setDescriptionHtml(html);
        }
        
        revalidate();
        repaint();        
    }
    
    private class TabListener implements ChangeListener {
        public void stateChanged(ChangeEvent evt) {
            JTabbedPane pane = (JTabbedPane) evt.getSource();
            
            if (pane.getSelectedIndex() > 0)
                loadImage();
        }
    } 
    

    private class ToggleSize50Action implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcSettings.setValue(Repository.settings.stQuickViewSize, "50%");
            changeSize();
        }
    }
    
    private class ToggleSize25Action implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcSettings.setValue(Repository.settings.stQuickViewSize, "25%");
            changeSize();
        }
    }
    
    private class QuickViewMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                showPopupMenu(e.getX(), e.getY());
            } 

            if (e.getClickCount() == 2 && dco != null) {
            	dco.markAsUnchanged();
                ItemForm itemform = new ItemForm(null, false, true, dco, module.isImdbUpdateAllowed(), module.isAmazonUpdateAllowed());
                itemform.setVisible(true);
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
