/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Component;

import javax.swing.JComponent;
import javax.swing.JTable;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;

public class ComboBoxTableCellRenderer extends DcTableCellRenderer {
    
    private final DcField field;
    
    public ComboBoxTableCellRenderer(DcField field) {
        this.field = field;
    }
    
    public Component getTableCellRendererComponent(
                                                JTable table, 
                                                Object value, 
                                                boolean isSelected, 
                                                boolean hasFocus, 
                                                int row,
                                                int column) {
        
        JComponent c = (JComponent) super.getTableCellRendererComponent(table,  
                                            value,
                                            isSelected,
                                            hasFocus,
                                            row,
                                            column);        

        
        try {
            if (value != null) {
                DcObject o = null;
    			if (value instanceof DcObject) 
    				o = (DcObject) value;
    			else
    				o = DataManager.getObjectForDisplayValue(field.getModRef(), value.toString());
                
                if (o != null) {
                	setIcon(o.getIcon());
                	setText(o.toString());
                } else {
                    setIcon(null);
                }
                
                o = null;
                value = null;
            } else {
                setIcon(null);
                setText("");
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        return c;
    }
}
