/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.components.tables.DcTableModel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class FileImportDialog extends DcDialog {

    private DcTable tableMappings;

    private final DcFileField ffSource;
    private final JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
    private final JButton buttonRun = ComponentFactory.getButton(DcResources.getText("lblRun"));

    private final DcShortTextField textSep = ComponentFactory.getShortTextField(5);

    private final JTextArea textLog = ComponentFactory.getTextArea();
    private final JProgressBar progressBar = new JProgressBar();

    private String sep = "|";

    public FileImportDialog() {
        super(DataCrow.mainFrame);

        setTitle(DcResources.getText("lblFileImport"));
        setHelpIndex("dc.Tools.FileImport");

        ffSource = ComponentFactory.getFileField(false, false);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        buildDialog();
    }

    private void addMessage(String message) {
        textLog.insert(message + '\n', 0);
        textLog.setCaretPosition(0);
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }

    private void close() {
        DcSettings.setValue(Repository.settings.stFileImportDialogSize, getSize());
        dispose();
    }

    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());

        //**********************************************************
        //Create Import Panel
        //**********************************************************
        JPanel panelImport = new JPanel();
        panelImport.setLayout(Layout.getGBL());

        JLabel labelSep = ComponentFactory.getLabel(DcResources.getText("lblValueSeperator"));
        textSep.setText("Tab");

        buttonApply.addActionListener(new ApplyFileAction());
        panelImport.add(ffSource,       Layout.getGBC( 0, 0, 4, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImport.add(labelSep,        Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImport.add(textSep,        Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImport.add(buttonApply,    Layout.getGBC( 2, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 5, 5), 0, 0));

        panelImport.setBorder(
                ComponentFactory.getTitleBorder(DcResources.getText("lblSourceSelection")));

        //**********************************************************
        //Create Import Definition Panel
        //**********************************************************
        tableMappings = ComponentFactory.getDCTable(false, false);

        DcTableModel model = (DcTableModel) tableMappings.getModel();
        model.setColumnCount(2);

        TableColumn columnName = tableMappings.getColumnModel().getColumn(0);
        JTextField text = ComponentFactory.getTextFieldDisabled();
        columnName.setCellEditor(new DefaultCellEditor(text));
        columnName.setHeaderValue(DcResources.getText("lblSourceName"));

        TableColumn columnField = tableMappings.getColumnModel().getColumn(1);
        Collection fields = DcModules.getCurrentModule().getFields();
        JComboBox comboFields = ComponentFactory.getComboBox();
        columnField.setHeaderValue(DcResources.getText("lblTargetName"));

        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField  field = (DcField) iter.next();
            if (field.getValueType() != Repository.datatypes._PICTURE &&
                field.getValueType() != Repository.datatypes._DCPARENTREFERENCE &&
                field.getValueType() != Repository.datatypes._DCOBJECTREFERENCE)

                comboFields.addItem(field);
        }
        columnField.setCellEditor(new DefaultCellEditor(comboFields));

        tableMappings.applyHeaders();

        buttonRun.addActionListener(new ImportAction());

        JScrollPane tableScroller = new JScrollPane(tableMappings);
        tableScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        JPanel panelImportDef = new JPanel();
        panelImportDef.setLayout(Layout.getGBL());

        panelImportDef.add(tableScroller, Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImportDef.add(buttonRun,   Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 5, 5), 0, 0));

        panelImportDef.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblImportDefinitions")));

        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        textLog.setEditable(false);
        textLog.setEnabled(false);

        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));

        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));


        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());

        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));

        buttonApply.setMnemonic('P');
        buttonCancel.setMnemonic('C');
        buttonRun.setMnemonic('R');
        
        buttonCancel.addActionListener(new CloseAction());

        this.getContentPane().add(panelImport,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelImportDef,  Layout.getGBC( 0, 1, 1, 1, 20.0, 20.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelLog,     Layout.getGBC( 0, 2, 1, 1, 10.0, 10.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelProgress,Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(buttonCancel,  Layout.getGBC( 0, 4, 1, 1, 0.0, 0.0
                                 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        pack();

        Dimension size = DcSettings.getValueAsDimension(Repository.settings.stFileImportDialogSize);
        setSize(size);

        setCenteredLocation();
    }

    private boolean isCorrectSource(File file) {
        boolean correct = file != null && file.exists();
        if (file == null)

            new MessageBox(DcResources.getText("msgNoFileSelected"), MessageBox._WARNING);
        else if (!file.exists())
            new MessageBox(DcResources.getText("msgFileCannotBeUsed"), MessageBox._WARNING);
        return correct;
    }

    private void applySource() throws IOException {
        File file = ffSource.getFile();

        sep = textSep.getText().trim();

        if (sep.length() == 0)
            sep = "Tab";

        tableMappings.clearTable();
        if (isCorrectSource(file)) {
            BufferedReader in = new BufferedReader(new FileReader(file));
            int lines = 0;
            String line;
            String headers = "";
            while ((line = in.readLine()) != null) {
                headers = lines == 0 ? line : headers;
                lines++;
            }
            in.close();
            initProgressBar(lines);

            StringTokenizer st = new StringTokenizer(headers, getSepCharacter(sep));

            if (st.countTokens() == 1) {
                new MessageBox(DcResources.getText("msgNoValuesFoundToImport"), MessageBox._WARNING);
                return;
            }

            int i = 1;
            Collection fields = DcModules.getCurrentModule().getFields();
            while (st.hasMoreTokens()) {
                String columnValue = (String) st.nextElement();

                DcField field = null;
                for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
                    DcField fld = (DcField) iter.next();
                    if (fld.getLabel().toUpperCase().equals(columnValue.toUpperCase()))
                        field = fld;
                }

                tableMappings.addRow(
                        new Object[] {DcResources.getText("lblField") +
                                      " " + i++  + " (" + columnValue + ")", field});
            }
        }
    }

    private String getSepCharacter(String sep) {
        String character = sep;
        if (sep.toUpperCase().equals("TAB"))
            character = "\t";
        return character;
    }

    private Integer convertToInteger(String s, DcField field) {
        try {
            if (s != null && s.length() > 0) {
                return new Integer(Integer.parseInt(s));
            }
        } catch (Exception exp) {
            addMessage(DcResources.getText("msgCouldNotConvertToInt", new String[] {s, field.getLabel()}));
        }
        return null;
    }

    private class ImportThread extends Thread {
        public void run() {
            buttonRun.setEnabled(false);
            buttonApply.setEnabled(false);
            DataCrow.mainFrame.setSelectedTab(net.sf.dc.console.MainFrame._INSERTTAB);

            try {
                LinkedList fields = new LinkedList();
                for (int i = 0; i < tableMappings.getRowCount(); i++) {
                    DcField field = (DcField) tableMappings.getValueAt(i, 1, true);

                    if (field == null){
                        new MessageBox(DcResources.getText("msgNoDestination",
                                           (String) tableMappings.getValueAt(i, 0)), MessageBox._WARNING);
                        return;
                    } else if (fields.contains(field)) {
                        new MessageBox(DcResources.getText("msgDestinationInUse",
                                           (String) tableMappings.getValueAt(i, 0)), MessageBox._WARNING);
                        return;
                    } else {
                        fields.add(field);
                    }
                }


                File file = ffSource.getFile();

                try {
                    if (isCorrectSource(file)) {
                    	
                    	Reader i = new InputStreamReader(new FileInputStream(file), "UTF-8");
                        BufferedReader in = new BufferedReader(i);
                        
                        String line;
                        DcModule module = DcModules.getCurrentModule();
                        while ((line = in.readLine()) != null) {
                            DcObject dco = module.getDcObject();

                            StringTokenizer st = new StringTokenizer(line, getSepCharacter(sep));
                            int counter = 0;
                            while (st.hasMoreTokens()) {
                                String s = (String) st.nextElement();
                                DcField field = (DcField) fields.get(counter);
                                if (field.getValueType() == Repository.datatypes._INTEGER)
                                    dco.setValue(field.getIndex(), convertToInteger(s, field));
                                else
                                    dco.setValue(field.getIndex(),s);

                                counter++;
                            }
                            module.getCurrentInsertView().add(dco);
                        }
                        in.close();

                    }
                } catch (Exception exp) {
                	Messages.add(exp);

                }
            } finally {
                buttonRun.setEnabled(true);
                buttonApply.setEnabled(true);
            }
        }
    }

    private class ApplyFileAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            try {
                applySource();
            } catch (IOException ioExp) {
                new MessageBox(DcResources.getText("msgFileCannotBeUsed") + ": " + ioExp.getMessage(), MessageBox._ERROR);
                addMessage("Error while reading file : " + ioExp.getMessage());
                Messages.add(ioExp);
            }
        }
    }

    private class ImportAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (tableMappings.getRowCount() == 0) {
                new MessageBox(DcResources.getText("msgNoFieldsFound"), MessageBox._WARNING);
            } else {
                ImportThread thread = new ImportThread();
                thread.start();
            }
        }
    }

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
