package net.sf.dc.console.expert;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class PerformancePanel extends DcPanel {

    private final JComboBox comboHSqlCacheScale = ComponentFactory.getComboBox();
    private final JComboBox comboHSqlCacheSizeScale = ComponentFactory.getComboBox();
    private final DcNumberField numGarbageCollectionInterval = ComponentFactory.getNumberField();
    
    public PerformancePanel() {
        super(null, null);
        buildPanel();
        setCurrentValues();
    }
    
    private void buildPanel() {
        setLayout(Layout.getGBL());
        
        JLabel labelHSqlCacheScale = ComponentFactory.getLabel(DcResources.getText("lblHsqlCacheScale"));
        JLabel labelHSqlCacheSizeScale = ComponentFactory.getLabel(DcResources.getText("lblHsqlCacheSizeScale"));
        JLabel labelGarbageCollectionInterval = ComponentFactory.getLabel(DcResources.getText("lblGarbageCollectionInterval"));
        
        labelHSqlCacheScale.setToolTipText(DcResources.getText("tpHsqlCacheScale"));
        labelHSqlCacheSizeScale.setToolTipText(DcResources.getText("tpHsqlCacheSizeScale"));
        labelGarbageCollectionInterval.setToolTipText(DcResources.getText("tpGarbageCollectionIntervalMs"));
        
        for (int i = 8; i < 19; i++) {
            comboHSqlCacheScale.addItem(new Integer(i));
        }
        
        for (int i = 6; i < 21; i++) {
            comboHSqlCacheSizeScale.addItem(new Integer(i));
        }
        
        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        buttonSave.addActionListener(new SaveActionListener());
        
        comboHSqlCacheScale.setPreferredSize(new Dimension(100, 21));
        comboHSqlCacheSizeScale.setPreferredSize(new Dimension(100, 21));
        numGarbageCollectionInterval.setPreferredSize(new Dimension(100, 21));
        
        add(labelHSqlCacheScale,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 5), 0, 0));
        add(comboHSqlCacheScale,  Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        add(labelHSqlCacheSizeScale,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 5), 0, 0));
        add(comboHSqlCacheSizeScale,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        add(labelGarbageCollectionInterval,  Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 5), 0, 0));
        add(numGarbageCollectionInterval,    Layout.getGBC( 1, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        buttonSave.setMnemonic('S');
        
        add(buttonSave,    Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets(5, 5, 5, 5), 0, 0));        
    }
    
    private void setCurrentValues() {
        int cacheScale = DcSettings.getValueAsInt(Repository.settings.stHsqlCacheScale);
        int cacheSizeScale = DcSettings.getValueAsInt(Repository.settings.stHsqlCacheSizeScale);
        int gcInterval = DcSettings.getValueAsInt(Repository.settings.stGarbageCollectionIntervalMs);
        
        comboHSqlCacheScale.setSelectedItem(new Integer(cacheScale));
        comboHSqlCacheSizeScale.setSelectedItem(new Integer(cacheSizeScale));
        numGarbageCollectionInterval.setValue(gcInterval);
    }
    
    private void saveSettings() {
        Integer cacheScale = (Integer) comboHSqlCacheScale.getSelectedItem();
        Integer cacheSizeScale = (Integer) comboHSqlCacheSizeScale.getSelectedItem();
        Integer gcInterval = (Integer) numGarbageCollectionInterval.getValue();
            
        if (cacheScale != null)
            DcSettings.setValue(Repository.settings.stHsqlCacheScale, cacheScale);
        
        if (cacheSizeScale != null)
            DcSettings.setValue(Repository.settings.stHsqlCacheSizeScale, cacheSizeScale);

        if (gcInterval != null)
            DcSettings.setValue(Repository.settings.stGarbageCollectionIntervalMs, gcInterval);      
        
        DatabaseManager.applySettings();
        
        new MessageBox(DcResources.getText("msgSettingsSavedUnapplied"), MessageBox._INFORMATION);
    }
    
    private class SaveActionListener implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            saveSettings();
        }
    }
}
