/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.settings.DcSettings;

public class SystemInfoPanel extends JPanel {
    
    public SystemInfoPanel() {
        buildDialog();
    }
    
    private void buildDialog() {
        setLayout(Layout.getGBL());

        JLabel recordsLabel = ComponentFactory.getLabel("Records in database:");
        JLabel databaseNameLabel = ComponentFactory.getLabel("Database");
        JLabel databaseNameTextLabel = ComponentFactory.getLabel(DcSettings.getValueAsString(Repository.settings.stConnectionString));
        JLabel databaseDriverLabel = ComponentFactory.getLabel("Database driver");
        JLabel databaseDriverTextLabel = ComponentFactory.getLabel(DcSettings.getValueAsString(Repository.settings.stDatabaseDriver));
        
        int y = 0;
        add(recordsLabel, Layout.getGBC( 0, y++, 1, 1, 2.0, 0.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                          new Insets( 0, 5, 0, 0), 0, 0));

        int totalCount = 0;
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); y++) {
            DcModule module = (DcModule) iter.next();
            if (module.getIndex() != DcModules._GENERIC && module.isTopModule()) {
                int count = getRecordCount(module);
                totalCount += count;
                
                JLabel labelName = ComponentFactory.getLabel(module.getName());
                JLabel labelCount = ComponentFactory.getLabel("" + count);
                
                add(labelName,      Layout.getGBC( 0, y, 1, 1, 1.0, 0.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets( 0, 5, 0, 0), 0, 0));

                add(labelCount,     Layout.getGBC( 1, y, 1, 1, 1.0, 0.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets( 0, 5, 0, 0), 0, 0));                
            }
        }
        
        add(ComponentFactory.getLabel("Total"),
            Layout.getGBC( 0, y, 1, 1, 1.0, 0.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets( 0, 5, 0, 0), 0, 0));

        add(ComponentFactory.getLabel("" + totalCount),
            Layout.getGBC( 1, y++, 1, 1, 1.0, 0.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets( 0, 5, 0, 0), 0, 0));

        add(databaseNameLabel,       Layout.getGBC( 0, y, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        add(databaseNameTextLabel,   Layout.getGBC( 1, y++, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        add(databaseDriverLabel,     Layout.getGBC( 0, y, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));

        add(databaseDriverTextLabel, Layout.getGBC( 1, y++, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));
    }
    
    private int getRecordCount(DcModule module) {
    	DcObject[] o = DataManager.get(module.getIndex(), null);
    	return o.length;
    }    
}
