/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JWindow;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.ext.util.Utilities;

public final class SplashScreen extends JWindow {

    private final JLabel lblStatus = ComponentFactory.getLabel("");

    public SplashScreen() {
        buildPanel();
    }

    public void setStatusMsg(String message) {
    	lblStatus.setText(message);
    }

	public void splash() {
		setVisible(true);
	}

    private void buildPanel() {
        //**********************************************************
        //Logo
        //**********************************************************
        JLabel lblLogo = new JLabel();
        try {
            String s = DataCrow.baseDir + "icons_system/logo.jpg";
            byte[] bytes = Utilities.readFile(new File(s));
            lblLogo = ComponentFactory.getImageLabel(new ImageIcon(bytes));
        } catch (Exception exp) {
            new MessageBox(exp.toString(), MessageBox._ERROR);
        }

        //**********************************************************
        //Status 
        //**********************************************************
        lblStatus.setPreferredSize(new Dimension(500, 22));
        lblStatus.setText(Repository.system.version);
        lblStatus.setHorizontalAlignment(JLabel.CENTER);
        lblStatus.setVerticalAlignment(JLabel.CENTER);
        lblStatus.setFont(new Font("Arial", Font.BOLD, 11));
        lblStatus.setForeground(new Color(20,232,25));
        
        getContentPane().setBackground(Color.BLACK);
        lblStatus.setBackground(Color.BLACK);

        //**********************************************************
        //Main panel
        //**********************************************************
        getContentPane().setLayout(Layout.getGBL());

        // Build the panel
        this.getContentPane().add(   lblLogo,       Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 2, 2, 2, 2), 0, 0));
        this.getContentPane().add(   lblStatus,     Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
        		                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
												    new Insets( 2, 0, 0, 0), 0, 0));
        pack();

        setLocation(Utilities.getCenteredWindowLocation(getSize()));
    }
}

