/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.event.MouseEvent;

import javax.swing.MenuSelectionManager;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;

import net.sf.dc.messages.Messages;


/**
 * @author Golan
 */
public class DcEditPopupEventQueue extends EventQueue {

    private DcEditPopupMenu popupmenu;

    /**
     * Creates a new instance of DcPopupShortEventQueue
     */
    public DcEditPopupEventQueue() {
        super();
    }

    protected void dispatchEvent(AWTEvent event) {
        try {
            super.dispatchEvent(event);
    
            if (!(event instanceof MouseEvent)) {
                return;
            }
    
            MouseEvent me = (MouseEvent) event;
            if (!me.isPopupTrigger()) {
                if (me.getID() == MouseEvent.MOUSE_RELEASED && popupmenu != null
                        && popupmenu.isShowing())
                    popupmenu.setVisible(false);
    
                return;
            }
    
            Component comp = SwingUtilities.getDeepestComponentAt((Component) me
                    .getSource(), me.getX(), me.getY());
    
            if (!(comp instanceof JTextComponent))
                return;
    
            if (MenuSelectionManager.defaultManager().getSelectedPath().length > 0)
                return;
    
            popupmenu = new DcEditPopupMenu((JTextComponent) comp);
            popupmenu.validate();
            popupmenu.show((Component) me.getSource(), me.getX(), me.getY());
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
}
