/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcList;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;

public class DcModuleList extends DcList {

    private int currentIndex = -1;
    private final JPanel panelHeader = new JPanel(Layout.getGBL());
    
    private boolean listenForChanges = true;
    private Vector elements = new Vector();
    
	public DcModuleList() {
        super();
        addModules();
        addListSelectionListener(new moduleSelectAction());
        setCellRenderer(new CustomCellRenderer());
	}

    public void setBackground(Color color) {
        super.setBackground(Color.WHITE);
    }
    
    public void setFont(Font font) {
        Collection elements = getData();
        for (Iterator iter = elements.iterator(); iter.hasNext(); ) {
            Object value = iter.next();
            if (value instanceof JPanel)
                ((JPanel) value).setFont(font);
        }
    }    
    
	private void addModules() {
	    elements.add(panelHeader);
        
        addModule(DcModules.getModule(DcModules._SOFTWARE));
        addModule(DcModules.getModule(DcModules._MUSICALBUM));
        addModule(DcModules.getModule(DcModules._MOVIE));
        addModule(DcModules.getModule(DcModules._AUDIOCD));
        addModule(DcModules.getModule(DcModules._BOOK));
        addModule(DcModules.getModule(DcModules._IMAGE));
        addModule(DcModules.getModule(DcModules._CONTACTPERSON));
        addModule(DcModules.getModule(DcModules._GENERIC));

        setListData(elements);
	}
    
    private void addModule(DcModule module) {
        if (module.isEnabled()) {
            ModulePanel modulePanel = new ModulePanel(module);
            modulePanel.add(ComponentFactory.getLabel(module.getIcon()));
            modulePanel.add(ComponentFactory.getLabel(module.getObjectNamePlural()));
            elements.add(modulePanel);
        }
    }
    
    public void setSelectedModule(int module) {
        listenForChanges = false;
        
        for (Iterator iter = getData().iterator(); iter.hasNext(); ) {
            Object item = iter.next();
            if (item instanceof ModulePanel) {
                if (((ModulePanel) item).getModule() == module)
                    setSelectedValue(item, true);
            }
        }
        listenForChanges = true;
    }
    
    private void setActiveModule() {
        listenForChanges = false;
        Object item = getSelectedValue();
        if (item instanceof ModulePanel) {
            int module = ((ModulePanel) item).getModule();
            if (module != currentIndex) {
                currentIndex = module;
                if (DataCrow.mainFrame != null) 
                    DataCrow.mainFrame.changeModule(module, true);
            }
        }
        listenForChanges = true;
    }
    
    private class ModulePanel extends JPanel {
        private final int module;
        public ModulePanel(DcModule module) {
            super(new FlowLayout(FlowLayout.LEFT));
            this.module = module.getIndex();
        }
        
        public int getModule() {
            return module;
        }
        
        public void setFont(Font font) {
            Component[] components = getComponents();
            for (int i = 0; i < components.length; i++) {
                components[i].setFont(font);
                components[i].setForeground(ComponentFactory.getCurrentForegroundColor());
            }
        }          
    }
    
    private class moduleSelectAction implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent arg0) {
            if (listenForChanges)
                setActiveModule();
        }
    }
}