/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.actions.AddRowAction;
import net.sf.dc.core.actions.CreateReportAction;
import net.sf.dc.core.actions.DeleteAction;
import net.sf.dc.core.actions.ManageCategoriesAction;
import net.sf.dc.core.actions.ManagePlatformsAction;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageStorageMediaAction;
import net.sf.dc.core.actions.OpenAction;
import net.sf.dc.core.actions.OpenFieldsSettingsDialog;
import net.sf.dc.core.actions.OpenLoanFormAction;
import net.sf.dc.core.actions.RemoveRowAction;
import net.sf.dc.core.actions.SaveSelectedAction;
import net.sf.dc.core.actions.StartProgramAction;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.actions.UpdateAllAction;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

public class DcPopupMenu extends JPopupMenu {

    public static final int _INSERT = 0;
    public static final int _SEARCH = 1;
    
    public DcPopupMenu(DcObject dco, int modus) {
        DcModule module = dco.getModule();
        String name = "";
        if (DcModules.getCurrentModule().getIndex() != DcModules._GENERIC)
            name = module.getObjectName();
        else 
            module = DcModules.getCurrentModule();
        
        JMenuItem menuQuickView = ComponentFactory.getMenuItem(IconLibrary._icoSettings, DcResources.getText("lblToggleQuickView"));
        JMenuItem menuLoan = ComponentFactory.getMenuItem(IconLibrary._icoLoan, DcResources.getText("lblLoan"));
        JMenuItem menuOpen = ComponentFactory.getMenuItem(IconLibrary._icoOpen, DcResources.getText("lblOpenItem", ""));
        JMenuItem menuSave = ComponentFactory.getMenuItem(IconLibrary._icoSave, DcResources.getText("lblSaveItem", name));
        JMenuItem menuDelete = ComponentFactory.getMenuItem(IconLibrary._icoDelete, DcResources.getText("lblDeleteItem", name));
        JMenuItem menuRemoveRow = ComponentFactory.getMenuItem(IconLibrary._icoDelete, DcResources.getText("lblRemove"));
        JMenuItem menuAddRow = ComponentFactory.getMenuItem(IconLibrary._icoAdd, DcResources.getText("lblAdd"));
        JMenuItem menuReportPdf = ComponentFactory.getMenuItem(IconLibrary._icoReport, DcResources.getText("lblCreateReport"));
        JMenuItem menuUpdateAll = ComponentFactory.getMenuItem(IconLibrary._icoUpdateAll, DcResources.getText("lblUpdateAll"));
        JMenuItem menuManageCategories = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageCategories"));
        JMenuItem menuManageStates = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));
        JMenuItem menuManageStorageMedia = ComponentFactory.getMenuItem(IconLibrary._icoSave, DcResources.getText("lblManageStorageMedia"));
        JMenuItem menuManagePlatforms = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManagePlatforms"));  
        JMenuItem menuStartProgram = ComponentFactory.getMenuItem(IconLibrary._icoOpenApplication, DcResources.getText("lblStartProgram"));
        JMenuItem menuFieldDefinitions = ComponentFactory.getMenuItem(IconLibrary._icoSettings, DcResources.getText("lblFieldSettings"));
        
        menuLoan.addActionListener(new OpenLoanFormAction());
        menuQuickView.addActionListener(new ToggleQuickViewAction());
        menuOpen.addActionListener(new OpenAction());
        
        menuSave.addActionListener(new SaveSelectedAction());
        menuDelete.addActionListener(new DeleteAction(module.getIndex()));

        menuRemoveRow.addActionListener(new RemoveRowAction(dco.getModule().getIndex()));
        menuAddRow.addActionListener(new AddRowAction(dco.getModule().getIndex()));
        menuReportPdf.addActionListener(new CreateReportAction());
        menuUpdateAll.addActionListener(new UpdateAllAction(module.getIndex()));
        menuStartProgram.addActionListener(new StartProgramAction());
        menuFieldDefinitions.addActionListener(new OpenFieldsSettingsDialog());

        menuManageStorageMedia.addActionListener(new ManageStorageMediaAction());
        menuManageCategories.addActionListener(new ManageCategoriesAction());
        menuManageStates.addActionListener(new ManageStatesAction(module.getIndex()));
        menuManagePlatforms.addActionListener(new ManagePlatformsAction());
        
        if (module.getIndex() == DcModules._AUDIOTRACK && modus == _INSERT) {
            
        } else {
            if (module.getIndex() != DcModules._GENERIC)
                add(menuSave);
            
            add(menuOpen);
        }
        

        
        if (modus == DcPopupMenu._SEARCH) { 
            add(menuDelete);
        } else {
            add(menuRemoveRow);
            add(menuAddRow);
        }        
        
        if (modus == DcPopupMenu._SEARCH && module.canBeLended()) {
            add(new JSeparator(JSeparator.HORIZONTAL));
            add(menuLoan);
        }
        
        if (DcModules.getCurrentModule().getIndex() != DcModules._GENERIC) {
            if (    module.getCategoryModule() != null || module.getStateModule() != null || 
                    module.getIndex() == DcModules._SOFTWARE || module.getStorageMediaModule() != null)
                add(new JSeparator(JSeparator.HORIZONTAL));
            
            if (module.getCategoryModule() != null) 
                add(menuManageCategories);
    
            if (module.getStateModule() != null)
                add(menuManageStates);

            
            if (module.getIndex() == DcModules._SOFTWARE)
                add(menuManagePlatforms);
            
            if (module.getStorageMediaModule() != null)
                add(menuManageStorageMedia);
        }
        
        if (module.getCurrentSearchView().getIndex() == MasterView._TABLE_VIEW) {
            add(new JSeparator(JSeparator.HORIZONTAL));
            add(menuFieldDefinitions);

            if (modus == DcPopupMenu._SEARCH) 
                add(menuQuickView);
        }
        
        add(new JSeparator(JSeparator.HORIZONTAL));
        add(menuUpdateAll);
        add(menuStartProgram);
        
        if (module.getReportFieldDefinitions() != null && module.getReportFieldDefinitions().size() > 0) {
             this.add(new JSeparator(JSeparator.HORIZONTAL));
             this.add(menuReportPdf);
        }        
    }
}
