package net.sf.dc.console.menu;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JToolBar;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcComboBox;
import net.sf.dc.console.components.DcEditableComboBox;
import net.sf.dc.console.menu.DcToolBar.ToolBarButton;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class DcQuickFilterToolBar extends JToolBar {

    private final DcEditableComboBox comboCriteria = new DcEditableComboBox();
    private final DcComboBox comboFields = ComponentFactory.getComboBox();
    private final DcComboBox comboFilters = ComponentFactory.getComboBox();
    
    private final SearchOnEnterListener searchOnEnterListener = new SearchOnEnterListener();
    private final SearchOnDoubleClickListener searchOnDblClickListener = new SearchOnDoubleClickListener();
    private final FilterSelectionListener filterSelListener = new FilterSelectionListener();
    private final FieldSelectionListener fieldSelListener = new FieldSelectionListener();
    
    private final DcModule module;
    
    public DcQuickFilterToolBar(DcModule module) {
        this.module = module;
        build();
        comboFields.setSelectedIndex(0);
    }
    
    public void toggleLabels() {
        showLabels(!DcSettings.getValueAsBoolean(Repository.settings.stShowMenuBarLabels));
    }
    
    public void setFont(Font font) {
        Component c;
        int i = 0;
        while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
                ToolBarButton button = (ToolBarButton) c;
                button.setFont(ComponentFactory.getSystemFont());
            }
        }
    }    
    
    public void showLabels(boolean show) {
        Component c;
        int i = 0;
        while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
                ToolBarButton button = (ToolBarButton) c;
                if (show) {
                    button.setText(button.getToolTipText());
                } else {
                    button.setText(null);
                }
            }
        }
        DcSettings.setValue(Repository.settings.stShowMenuBarLabels, new Boolean(show));
    }
    
    private void search() {
        DataManager.bindData(module.getSearchView(), module.getIndex(), getDataFilter(), true);
    }
    
    private void search(DataFilter df) {
        DataManager.bindData(module.getSearchView(), module.getIndex(), df, true);
    }
    
    private DataFilter getDataFilter() {
        DcField field = (DcField) comboFields.getSelectedItem();
        
        DcObject baseDco = module.getDcObject();
        
        Object value;
        if (field.getFieldType() ==  ComponentFactory._RATINGCOMBOBOX) {
            value = comboCriteria.getSelectedItem();
            if (value != null && value.equals(new Integer(-1)))
                value = null;
            
        } else {
            value = comboCriteria.getSelectedItem();
        }
        
        baseDco.markAsUnchanged();
        
        String s = value == null ? "" : value.toString();
        if (s.length() > 0) {
            baseDco.setValue(field.getIndex(), value);
            baseDco.setChanged(field.getIndex(), true);
        }
        
        DataFilter df = DataFilters.getDefaultDataFilter(module.getIndex());
        df.setBaseDco(baseDco);
        return df;
    }

    private void setSearchField(DcField field) {
        JComponent c = ComponentFactory.getComponent(field.getModule(), field.getFieldType(), field.getLabel(), 255);
        comboCriteria.removeAllItems();
        
        if (c instanceof JComboBox) {
            JComboBox combo = (JComboBox) c;
            for (int i = 0; i < combo.getItemCount(); i++) {
                comboCriteria.addItem(combo.getItemAt(i));
            }
        } else {
        	comboCriteria.addItem("");
        }
        comboCriteria.addItem(DcResources.getText("lblEmptySearchValue"));

        revalidate();
    }
    
    private void build() {
        DcFieldDefinitions definitions = (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(module.getFieldDefinitionsSettingsKey());
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            
            DcField field = module.getField(definition.getIndex());
            
            if (field.isEnabled() && field.isSearchable() && !field.isUiOnly())
                comboFields.addItem(field);
        }
        
        Collection filters = DataFilters.get(module.getIndex());
        comboFilters.addItem(" ");
        for (Iterator iter = filters.iterator(); iter.hasNext(); ) {
            comboFilters.addItem(iter.next());
        }
        
        comboCriteria.getEditor().getEditorComponent().addKeyListener(searchOnEnterListener);
        comboCriteria.getEditor().getEditorComponent().addMouseListener(searchOnDblClickListener);
        
        comboFilters.addActionListener(filterSelListener);
        comboFields.addActionListener(fieldSelListener);
        
        comboFilters.setPreferredSize(new Dimension(200, 21));
        comboFields.setPreferredSize(new Dimension(200, 21));
        comboCriteria.setPreferredSize(new Dimension(200, 21));
        comboFilters.setMaximumSize(new Dimension(200, 21));
        comboFields.setMaximumSize(new Dimension(200, 21));
        comboCriteria.setMaximumSize(new Dimension(200, 21));
        
        
        add(ComponentFactory.getLabel(DcResources.getText("lblQuickFilter") + " "));
        add(comboFields);
        add(comboCriteria);
        
        if (filters.size() > 0) {
            addSeparator();
            
            add(ComponentFactory.getLabel(DcResources.getText("lblFilters") + " "));
            add(comboFilters);
        }
        
        boolean showLabels = DcSettings.getValueAsBoolean(Repository.settings.stShowMenuBarLabels);
        showLabels(showLabels);        
    }    
    
    private class SearchOnEnterListener implements KeyListener {

        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                search();
            }
        }

        public void keyPressed(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}
    }

    private class SearchOnDoubleClickListener implements MouseListener {

        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                search();
            }            
        }
    }
    
    private class FieldSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            comboCriteria.removeAllItems();
            
            DcComboBox comboFields = (DcComboBox) e.getSource();
            DcField field = (DcField) comboFields.getSelectedItem();
            setSearchField(field);
            
        }
    }
    
    private class FilterSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DataFilter df = comboFilters.getSelectedIndex() > 0 ? (DataFilter) comboFilters.getSelectedItem() : null;
            search(df);
        }
    } 
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        
        super.paintComponent( g2d );
    }
}
