/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;
import javax.swing.JToolTip;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcMultiLineToolTip;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.CreateNewAction;
import net.sf.dc.core.actions.CreateReportAction;
import net.sf.dc.core.actions.DeleteAction;
import net.sf.dc.core.actions.HelpAction;
import net.sf.dc.core.actions.ImportAction;
import net.sf.dc.core.actions.InternetUpdateAction;
import net.sf.dc.core.actions.LogAction;
import net.sf.dc.core.actions.NewItemWizardAction;
import net.sf.dc.core.actions.OpenAction;
import net.sf.dc.core.actions.SaveAllAction;
import net.sf.dc.core.actions.SaveSelectedAction;
import net.sf.dc.core.actions.SearchAction;
import net.sf.dc.core.actions.SearchAgainAction;
import net.sf.dc.core.actions.SettingsAction;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcToolBar extends JToolBar {

	private final PopupMenu popupMenu = new PopupMenu();

    public DcToolBar(DcModule module) {
        addMouseListener(new ToolBarMouseListener());

        ToolBarButton buttonNewWizard = new ToolBarButton(IconLibrary._icoWizard, DcResources.getText("lblWizard"));
        ToolBarButton buttonNew = new ToolBarButton(IconLibrary._icoAdd, DcResources.getText("lblNewItem", ""));
        ToolBarButton buttonOpen = new ToolBarButton(IconLibrary._icoOpen, DcResources.getText("lblOpen"));
        ToolBarButton buttonDel = new ToolBarButton(IconLibrary._icoDelete, DcResources.getText("lblDelete"));
        ToolBarButton buttonSave = new ToolBarButton(IconLibrary._icoSave, DcResources.getText("lblMenuSave"));
        ToolBarButton buttonSaveAll = new ToolBarButton(IconLibrary._icoSaveAll, DcResources.getText("lblSaveAll"));
        ToolBarButton buttonFind = new ToolBarButton(IconLibrary._icoSearch, DcResources.getText("lblFilter"));
        ToolBarButton buttonFindAgain = new ToolBarButton(IconLibrary._icoSearchAgain, DcResources.getText("lblApplyFilter"));
        ToolBarButton buttonHelp = new ToolBarButton(IconLibrary._icoHelp, DcResources.getText("lblHelp"));
        ToolBarButton buttonSettings = new ToolBarButton(IconLibrary._icoSettings, DcResources.getText("lblSettings"));
        ToolBarButton buttonLog = new ToolBarButton(IconLibrary._icoEventLog, DcResources.getText("lblShowLog"));
        ToolBarButton buttonReport = new ToolBarButton(IconLibrary._icoReport, DcResources.getText("lblReport"));
        ToolBarButton buttonImportCD = new ToolBarButton(IconLibrary._icoImport, DcResources.getText("lblImport"));
        ToolBarButton buttonInternetUpdate = new ToolBarButton(IconLibrary._icoSearchOnline, DcResources.getText("lblOnlineSearch"));
        
        buttonReport.addActionListener(new CreateReportAction());
        buttonNewWizard.addActionListener(new NewItemWizardAction());
        buttonNew.addActionListener(new CreateNewAction());
        buttonOpen.addActionListener(new OpenAction());
        buttonDel.addActionListener(new DeleteAction());
        buttonSave.addActionListener(new SaveSelectedAction());
        buttonSaveAll.addActionListener(new SaveAllAction());
        buttonFind.addActionListener(new SearchAction());
        buttonFindAgain.addActionListener(new SearchAgainAction());
        buttonSettings.addActionListener(new SettingsAction());
        buttonLog.addActionListener(new LogAction());
        buttonHelp.addActionListener(new HelpAction());
        buttonImportCD.addActionListener(new ImportAction());
        buttonInternetUpdate.addActionListener(new InternetUpdateAction());
        
        add(buttonNewWizard);
        
        addSeparator();
        
		add(buttonNew);
        add(buttonOpen);
        add(buttonSave);
        add(buttonSaveAll);
        add(buttonDel);
        
        addSeparator();
        
        add(buttonFind);
        add(buttonFindAgain);
        
        if (    module.isAmazonUpdateAllowed() || module.isFreedBUpdateAllowed() || module.isImdbUpdateAllowed())
            add(buttonInternetUpdate);
        
        addSeparator();
        
        add(buttonSettings);
        add(buttonReport);
        
        if (    module.getIndex() != DcModules._BOOK &&
                module.getIndex() != DcModules._GENERIC && module.getIndex() != DcModules._CONTACTPERSON)
            add(buttonImportCD);
        
        addSeparator();
        
        add(buttonLog);
        add(buttonHelp);
        
        boolean showLabels = DcSettings.getValueAsBoolean(
                                Repository.settings.stShowMenuBarLabels);
        
        showLabels(showLabels);
	}
    
    private void showPopupMenu(int x, int y) {
        popupMenu.show(this, x, y);
    }

    public void toggleLabels() {
        showLabels(!DcSettings.getValueAsBoolean(Repository.settings.stShowMenuBarLabels));
    }
    
    public void setFont(Font font) {
        Component c;
        int i = 0;
        while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
                ToolBarButton button = (ToolBarButton) c;
                button.setFont(ComponentFactory.getSystemFont());
            }
        }
    }    
    
	public void showLabels(boolean show) {
		Component c;
		int i = 0;
		while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
    			ToolBarButton button = (ToolBarButton) c;
    			if (show) {
    				button.setText(button.getToolTipText());
                } else {
    				button.setText(null);
                }
            }
		}
        DcSettings.setValue(Repository.settings.stShowMenuBarLabels, new Boolean(show));
	}
    
	public class ToolBarButton extends JButton {

		private static final long serialVersionUID = 4L;

		public ToolBarButton(Icon icon, String text) {
			super(icon);
            setFont(ComponentFactory.getSystemFont());
            setText(text);
            setVerticalTextPosition(BOTTOM);
			setHorizontalTextPosition(CENTER);
            setToolTipText(text);
            addMouseListener(new ToolBarMouseListener());
		}
        
        public JToolTip createToolTip() {
            return new DcMultiLineToolTip();
        }
	}
    
    public class PopupMenu extends JPopupMenu {

		private static final long serialVersionUID = 6L;

		public PopupMenu() {
            JMenuItem menuToggleLabels = new JMenuItem(DcResources.getText("lblToggleMenuLabels"));

            menuToggleLabels.addActionListener(new ToggleLabelsAction());
            this.add(menuToggleLabels);
        }
    }
    
    private class ToggleLabelsAction implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
            boolean showLabels = DcSettings.getValueAsBoolean(Repository.settings.stShowMenuBarLabels);
			showLabels(!showLabels);
		}
    }
    
    private class ToolBarMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                showPopupMenu(e.getX(), e.getY());
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    } 

    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        
        super.paintComponent( g2d );
    }      
}