/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.movie;

import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.console.forms.OnlineSearchForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.onlinesearch.AsinSearchMode;
import net.sf.dc.core.objects.onlinesearch.BarcodeSearchMode;
import net.sf.dc.core.objects.onlinesearch.DcAmazonServer;
import net.sf.dc.core.objects.onlinesearch.DcCombinedAmazonImdbServer;
import net.sf.dc.core.objects.onlinesearch.DcImdbServer;
import net.sf.dc.core.objects.onlinesearch.DcServer;
import net.sf.dc.core.objects.onlinesearch.KeywordSearchMode;
import net.sf.dc.processes.http.AmazonImdbMovieUpdate;
import net.sf.dc.processes.http.AmazonMovieUpdate;
import net.sf.dc.processes.http.IMDBMovieUpdate;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class OnlineMovieSearchForm extends OnlineSearchForm {
    
    public OnlineMovieSearchForm(DcObject dco, ItemForm itemForm, boolean advanced) {
        super(itemForm, dco, itemForm, advanced);
    }
    
    public OnlineMovieSearchForm(DcObject dco, int tab, boolean advanced) {
        super(DataCrow.mainFrame, dco, tab, advanced);
    }
    
    public String getDefaultServer() {
        return DcSettings.getValueAsString(Repository.settings.stMovieOnlineUpdateServer);
    }    

    public JButton getSearchButton() {
        JButton button = ComponentFactory.getButton(DcResources.getText("lblFind"));
        button.addActionListener(new SearchAction());
        return button;
    }    
    
    public Collection getServerDefinitions() {
        DcServer amazonServer = new DcAmazonServer();
        amazonServer.addMode(new KeywordSearchMode("DVD", "DVD"));
        amazonServer.addMode(new KeywordSearchMode("VHS", "VHS"));
        amazonServer.addMode(new BarcodeSearchMode("Video", "Barcode"));
        amazonServer.addMode(new AsinSearchMode("Asin"));
        
        DcCombinedAmazonImdbServer combinedServer = new DcCombinedAmazonImdbServer();
        combinedServer.addMode(new KeywordSearchMode("DVD", "DVD"));
        combinedServer.addMode(new KeywordSearchMode("VHS", "VHS"));
        combinedServer.addMode(new BarcodeSearchMode("Video", "Barcode"));
        combinedServer.addMode(new AsinSearchMode("Asin"));
        
        DcServer imdbServer = new DcImdbServer();
        
        Collection servers = new ArrayList();
        servers.add(combinedServer);
        servers.add(amazonServer);
        servers.add(imdbServer);
        
        return servers;
    }
    
    public void saveSettings() {
        super.saveSettings();
        DcSettings.setValue(Repository.settings.stMovieOnlineUpdateServer, 
                            getServer().getName() + "," + getServerAddress());
    }   
    
    public void startSearch() {
        setCancelMode();
        saveSettings();
        
        String keywords = getSearchString();
        if (keywords.length() >= 2) {
            DcServer server = (DcServer) comboServers.getSelectedItem();
            if (server instanceof DcCombinedAmazonImdbServer) 
                process  = new AmazonImdbMovieUpdate(this, keywords, getMode(), getRegion());
            else if (server instanceof DcAmazonServer) 
                process  = new AmazonMovieUpdate(this, keywords, getMode(), getRegion());
            else
                process  = new IMDBMovieUpdate(this, keywords, getRegion());
            
            process.setPriority(Thread.MAX_PRIORITY);
            process.start();
        } else {
            setSearchMode();
            new MessageBox(DcResources.getText("msgSearchStringLength", "2"),
                           MessageBox._WARNING);            
        }        
    }
}
