package net.sf.dc.console.musicfile;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.DcProgressBar;
import net.sf.dc.console.components.panels.OnlineAudioServicePanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.views.View;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.musicfiles.MusicAlbum;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.core.objects.musicfiles.MusicTrack;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import entagged.audioformats.exceptions.CannotReadException;

public class MusicFileSynchronizerDialog extends DcDialog {
	
    private DcProgressBar progressBar = new DcProgressBar();
    private JTextArea textLog = ComponentFactory.getTextArea();
    private JButton buttonStart = ComponentFactory.getButton(DcResources.getText("lblRun"));
    private JButton buttonStop = ComponentFactory.getButton(DcResources.getText("lblStop"));
    private JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
    private DcLongTextField textHelp = ComponentFactory.getLongTextField(8000);
    
    private JCheckBox checkReparseFiles = ComponentFactory.getCheckBox(DcResources.getText("lblReparseMusicFiles"));
    
    private OnlineAudioServicePanel panelServer;
    
    private boolean keepOnRunning = true;
    
	public MusicFileSynchronizerDialog() {
		super(DataCrow.mainFrame);
		
		buildDialog();
		
		setTitle(DcResources.getText("lblMusicSynchronizer"));
        setHelpIndex("dc.Tools.MusicFileSynchronizer");
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	close();
            }
        });		
		
		pack();
		setSize(DcSettings.getValueAsDimension(Repository.settings.stMusicFileSynchronizerDialogSize));
		setCenteredLocation();
		
		allowActions();
	}
	
	private void updateView() {
        if (!checkReparseFiles.isSelected() && !panelServer.useOnlineService()) {
            new MessageBox(DcResources.getText("msgSynchronizeMethodHasNotBeenDefined"), MessageBox._WARNING);
            return;
        }
        
		ViewUpdater updater = new ViewUpdater();
		updater.setPriority(5);
		updater.start();
	}
	
    private void cancel() {
    	keepOnRunning = false;
    	allowActions();
    }
    
    private void close() {
    	cancel();
        
        keepOnRunning = false;
    	
    	progressBar = null;
    	textLog = null;
    	buttonStart = null;
        buttonCancel = null;
        textHelp = null;
        buttonStop = null;
        
        DcSettings.setValue(Repository.settings.stMusicFileSynchronizerDialogSize, getSize());

        dispose();
        setVisible(false);
    }    
    
    private void allowActions() {
        if (buttonStart != null)
            buttonStart.setEnabled(true);
        
        if (buttonStop != null)
            buttonStop.setEnabled(false);
    }    
    
    private void denyActions() {
        progressBar.setValue(0);
        textLog.setText("");

        if (buttonStart != null)
            buttonStart.setEnabled(false);
        
        if (buttonStop != null)
            buttonStop.setEnabled(true);
    }	
	
    private void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }

	
    private void initProgressBar(int maxValue) {
		progressBar.setValue(0);
		progressBar.setMaximum(maxValue);
    }

    private void updateProgressBar() {
        if (progressBar != null)
            progressBar.setValue(progressBar.getValue() + 1);
    }
    
    private void setValue(DcObject dco, int field, Object newValue) {
    	if (dco.isFilled(field)) {
        	if (newValue != null && !newValue.equals("") && !newValue.equals("-1"))
        		dco.setValue(field, newValue);
    	} else {
    		dco.setValue(field, newValue);
    	}
    }
	
    private class ViewUpdater extends Thread {
    	
    	public void run() {
    		try {
    			denyActions();
    			
                keepOnRunning = true;
                
	    		View view = DcModules.getCurrentModule().getCurrentSearchView();
	    		
                int items = view.getRows();
	    		initProgressBar(items);
	    		for (int i = 0; i < items && keepOnRunning; i++) {
	    			DcObject dco = view.getObject(i);
	    			
	    			dco.loadChildren();
	    			Collection children = dco.getChildren();
	
	    			boolean updated = false;
                    
                    if (checkReparseFiles.isSelected()) {
    	    			for (Iterator iter = children.iterator(); iter.hasNext() && keepOnRunning; ) {
    	    				DcObject child = (DcObject) iter.next();
    	    				
    	    				String filename = child.getFilename();
    	    				File tst = new File(filename);
    	    				if (!tst.exists())
    	    					filename = filename.replaceAll("`", "'");
    	    				
    	    				try {
                                addMessage(DcResources.getText("msgParsing", filename));
                                
                                MusicFile musicFile = new MusicFile(filename);
    	
    	    	                dco.setValue(MusicAlbum._A_TITLE, musicFile.getAlbum());
    	    	                dco.setValue(MusicAlbum._F_ARTIST, musicFile.getArtist());
    	
                                setValue(child, MusicTrack._K_QUALITY, new Integer(musicFile.getBitrate()));
    	    	                setValue(child, MusicTrack._J_PLAYLENGTH, new Integer(musicFile.getLength()));
    	    	                setValue(child, MusicTrack._L_ENCODING, musicFile.getEncodingType());
    	    	                setValue(child, MusicTrack._A_TITLE, musicFile.getTitle());
    	    	                setValue(child, MusicTrack._G_ARTIST, musicFile.getArtist());
    	    	                setValue(child, MusicTrack._C_YEAR, musicFile.getYear());
    	    	                setValue(child, MusicTrack._H_GENRE, musicFile.getGenre());
                                setValue(child, MusicTrack._F_TRACKNUMBER, new Integer(musicFile.getTrack()));
    		                    
    		                    child.setSilence(true);
    		                    updated = true;
    	    				} catch (CannotReadException exp) {
    	    					child.markAsUnchanged();
    	    					addMessage(exp.getMessage());
    	    				}
    	    			}
                    }
	    			
                    if (panelServer.useOnlineService()) {
                        addMessage(DcResources.getText("msgSearchingOnlineFor", "" + dco));
                        updated = new MusicFile().onlineUpdate((MusicAlbum) dco, panelServer.getServer(), panelServer.getRegion());
                    }

                    updateProgressBar();
                    
	                try {
	                	if (updated) {
	                		dco.setSilence(true);
	                		dco.saveUpdate(true);
	                		
	                		while (DatabaseManager.getQueueSize() > 0) {
	                			try {
	                				sleep(100);
	                			} catch (Exception exp) {}
	                		}
	                	}
	                } catch (ValidationException ve) {
	                	addMessage(ve.getMessage());
	                }
	    		}
    		} finally {
                addMessage(DcResources.getText("msgSynchronizerEnded"));
    			allowActions();
    		}
    	}
    }
    
	private void buildDialog() {
		getContentPane().setLayout(Layout.getGBL());

        //**********************************************************
        //Help panel
        //**********************************************************
        textHelp.setText(DcResources.getText("msgMusicFileSynchronizerHelp"));
        textHelp.setBorder(null);

        JScrollPane helpScroller = new JScrollPane(textHelp);
        helpScroller.setBorder(null);
        helpScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        helpScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        textHelp.setEditable(false);
        
        //**********************************************************
        //Online Server panel
        //**********************************************************
        panelServer = new OnlineAudioServicePanel();

        //**********************************************************
        //Reparse panel
        //**********************************************************
        JPanel panelReparse = new JPanel();
        panelReparse.setLayout(Layout.getGBL());
        panelReparse.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblMusicFileProcessingConfig")));
        
        panelReparse.add(checkReparseFiles, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets(5, 5, 5, 5), 0, 0));
        checkReparseFiles.setSelected(true);
        
        //**********************************************************
        //Log panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());
        
        JScrollPane logScroller = new JScrollPane(textLog);
        logScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));
        panelLog.add(logScroller, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                    new Insets(5, 5, 5, 5), 0, 0));


        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        
        buttonStart.addActionListener(new StartViewUpdate());
        buttonCancel.addActionListener(new CancelAction());
        buttonStop.addActionListener(new StopAction());
        
        panelActions.add(buttonStart);
        panelActions.add(buttonStop);
        panelActions.add(buttonCancel);
        
        buttonStart.setMnemonic('S');
        buttonStop.setMnemonic('T');
        buttonCancel.setMnemonic('C');  
        
        //**********************************************************
        //Main
        //**********************************************************
        getContentPane().add(helpScroller,      Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelServer,       Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelReparse,      Layout.getGBC(0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,      Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                 new Insets(0, 0, 0, 0), 0, 0));
        getContentPane().add(panelLog,          Layout.getGBC( 0, 4, 1, 1, 5.0, 5.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                 new Insets(0, 0, 0, 0), 0, 0));
        getContentPane().add(progressBar,       Layout.getGBC( 0, 5, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(0, 0, 0, 0), 0, 0));
        
	}

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
        	close();
        }
    }

    private class StopAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancel();
        }
    }
    
    private class StartViewUpdate implements ActionListener {
        public void actionPerformed(ActionEvent e) {
        	updateView();
        }
    }
    
   
}
