package net.sf.dc.console.musicfile;

import java.util.Collection;

import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.musicfiles.MusicAlbum;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.RefreshChildView;
import net.sf.dc.wf.requests.RequestCollection;

public class MusicTrackForm extends DcSimpleItemView {

    private final String parentID;
    
    public MusicTrackForm(ItemForm form, MusicAlbum musicAlbum, int module, boolean readonly) {
        super(form, module, readonly);
        this.parentID = musicAlbum.getID();
        list.setEnabled(!readonly);
    }
   
    public void addObjects() {
        list.clear();
        list.add(DataManager.getChildren(parentID, getModuleIdx()));
    }
    
    public void setObjects(Collection objects) {
        list.clear();
        list.add(objects);
    }
    
    protected void close() {
        DcSettings.setValue(Repository.settings.stMusicTrackItemViewSize, getSize());
        dispose();        
    }
    
    public void open() {
        DcObject dco = list.getSelectedObject();
        if (dco != null) {
            dco.markAsUnchanged();
            MusicTrackItemForm itemForm = new MusicTrackItemForm(true, dco, this);
            itemForm.setVisible(true);            
        }
    }
    
    public void createNew() {
        DcObject dco = getModule().getDcObject();
        dco.setValue(dco.getParentReferenceFieldIndex(), parentID);
        
        MusicTrackItemForm itemForm = new MusicTrackItemForm(false, dco, this);
        itemForm.setVisible(true);
    }

    protected String getSizeSettingsKey() {
        return Repository.settings.stMusicTrackItemViewSize;
    }

    public RequestCollection getAfterDeleteRequests() {
        RequestCollection requests = new RequestCollection();
        requests.addRequest(new RefreshChildView(this, DcModules._MUSICALBUM, null, null, parentID, false));
        return requests;
    }   
}
