package net.sf.dc.console.musicfile;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.musicfiles.MusicTrack;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.CloseWindowRequest;
import net.sf.dc.wf.requests.RefreshChildView;

public class MusicTrackItemForm extends ItemForm {

    private MusicTrackForm form;
    
    public MusicTrackItemForm(boolean update, DcObject o, MusicTrackForm form) {
        super(DataCrow.mainFrame, false, update, o, false, false);
        this.form = form;
    }
    
    protected void saveValues() {
        MusicTrack clone = (MusicTrack) getObjectForSaving();
        clone.addRequest(new CloseWindowRequest(this));
        
        if (!update)  {
            try {
                clone.addRequest(new RefreshChildView(form, DcModules._MUSICALBUM, clone, clone.getID(), (String) clone.getValue(clone.getParentReferenceFieldIndex()), true));
                clone.setSilence(true);
                clone.saveNew(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (isChanged()) {
            try {
                clone.addRequest(new RefreshChildView(form, DcModules._MUSICALBUM, clone, clone.getID(), (String) clone.getValue(clone.getParentReferenceFieldIndex()), false));
                clone.setSilence(true);
                clone.saveUpdate(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (! isChanged()) {
            new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
        }
    }  
    
    public void close(boolean aftersave) { 
        super.close(aftersave);
        form = null;
    }
    
    /**
     * Deletes this item from the database
     */
    protected void deleteValues() {
        new QuestionBox(DcResources.getText("msgDeleteQuestion"), this);
        
        if (QuestionBox.result.booleanValue()) {
            String id = dco.getID();
            dco.clearValues();
            dco.setValue(DcObject._ID, id);
            dco.setSilence(true);

            dco.addRequest(new CloseWindowRequest(this));
            dco.addRequest(new RefreshChildView(form, DcModules._MUSICALBUM, null, id, (String) dco.getValue(dco.getParentReferenceFieldIndex()), false));
            dco.delete();
        }
    }
}
