/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.software;

import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.forms.AmazonSearchForm;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.onlinesearch.AsinSearchMode;
import net.sf.dc.core.objects.onlinesearch.BarcodeSearchMode;
import net.sf.dc.core.objects.onlinesearch.DcAmazonServer;
import net.sf.dc.core.objects.onlinesearch.DcServer;
import net.sf.dc.core.objects.onlinesearch.KeywordSearchMode;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class AmazonSoftwareSearchForm extends AmazonSearchForm {

    public AmazonSoftwareSearchForm(DcObject dco, ItemForm itemForm, boolean advanced) {
        super(dco, itemForm, advanced);
    }
    
    public AmazonSoftwareSearchForm(DcObject dco, int tab, boolean advanced) {
        super(dco, tab, advanced);
    }    

    public JButton getSearchButton() {
        JButton button = ComponentFactory.getButton(DcResources.getText("lblFind"));
        button.addActionListener(new SearchAction());
        return button;
    }    
    
    public String getDefaultServer() {
        return DcSettings.getValueAsString(Repository.settings.stSoftwareOnlineUpdateServer);
    }      
    
    public Collection getServerDefinitions() {
        DcServer server = new DcAmazonServer();
        server.addMode(new KeywordSearchMode("Software", "PC software and games"));
        server.addMode(new KeywordSearchMode("VideoGames", "Videogames (any platform)"));
        server.addMode(new BarcodeSearchMode("Software", "Software Barcode"));
        server.addMode(new BarcodeSearchMode("VideoGames", "Videogames Barcode"));
        server.addMode(new AsinSearchMode("Asin"));
        Collection servers = new ArrayList();
        servers.add(server);
        return servers;
    }
    
    public void saveSettings() {
        super.saveSettings();
        DcSettings.setValue(Repository.settings.stSoftwareOnlineUpdateServer, 
                            getServer().getName() + "," + getServerAddress());
    }     
}
