/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.software;

import javax.swing.JFrame;

import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilterOptions;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.RefreshSimpleItemViewRequest;
import net.sf.dc.wf.requests.RequestCollection;

public class SoftwarePlatformForm extends DcSimpleItemView {
    
    public SoftwarePlatformForm(JFrame parent, int module) {
        super(parent, module, false);
    }
    
    public void addObjects() {
        list.clear();
        DcObject dco = getModule().getDcObject();
        
        DataFilterOptions dfo = new DataFilterOptions(new DcField[] {dco.getField(SoftwarePlatform._A_NAME)}, false, false);
        DataFilter df = new DataFilter(getModule().getIndex(), dco, dfo);
        list.add(DataManager.get(getModule().getIndex(), df));
    }
    
    protected void close() {
        DcSettings.setValue(Repository.settings.stSoftwarePlatformFormSize, getSize());
        dispose();
    }
    
    public void open() {
        DcObject dco = list.getSelectedObject();
        if (dco != null) {
            dco.markAsUnchanged();
            SoftwarePlatformItemForm softwarePlatformItemForm = new SoftwarePlatformItemForm(true, dco, this);
            softwarePlatformItemForm.setVisible(true);
        }
    }
    
    public void createNew() {
        SoftwarePlatformItemForm softwarePlatformItemForm = new SoftwarePlatformItemForm(false, getModule().getDcObject(), this);
        softwarePlatformItemForm.setVisible(true);
    }

    public RequestCollection getAfterDeleteRequests() {
        RequestCollection requests = new RequestCollection();
        requests.addRequest(new RefreshSimpleItemViewRequest(this));
        return requests;
    }

    protected String getSizeSettingsKey() {
        return Repository.settings.stSoftwarePlatformFormSize;
    }
}