package net.sf.dc.console.software;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.CloseWindowRequest;
import net.sf.dc.wf.requests.RefreshSimpleItemViewRequest;

public class SoftwarePlatformItemForm extends ItemForm {
    
    private DcSimpleItemView parent;
    
    public SoftwarePlatformItemForm(boolean update, DcObject o, DcSimpleItemView parent) {
        super(DataCrow.mainFrame, false, update, o, false, false);
        this.parent = parent;
    }
    
    protected void saveValues() {
        DcObject clone = getObjectForSaving();
        
        clone.addRequest(new RefreshSimpleItemViewRequest(parent));
        clone.addRequest(new CloseWindowRequest(this));
        
        if (!update)  {
            try {
                clone.setSilence(true);
                clone.saveNew(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (isChanged()) {
            try {
                clone.setSilence(true);
                clone.saveUpdate(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (! isChanged()) {
            new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
        }
    }  
    
    /**
     * Deletes this item from the database
     */
    protected void deleteValues() {
        new QuestionBox(DcResources.getText("msgDeleteQuestion"), this);
        
        if (QuestionBox.result.booleanValue()) {
            String id = dco.getID();
            dco.clearValues();
            dco.setValue(DcObject._ID, id);
            dco.setSilence(true);

            dco.addRequest(new CloseWindowRequest(this));
            dco.addRequest(new RefreshSimpleItemViewRequest(parent));
            dco.delete();
        }
    }
    
    protected void addInputPanel() {
        JPanel panelInfo = new JPanel();
        panelInfo.setLayout(Layout.getGBL());
        
        DcField field = dco.getField(SoftwarePlatform._A_NAME);
        JLabel labelName = (JLabel) labels.get(field);
        JComponent inputName = (JComponent) fields.get(field);
        inputName.setPreferredSize(new Dimension(200, 20));
        labelName.setPreferredSize(new Dimension(100, 20));
            
        panelInfo.add(labelName,    Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 0, 5), 0, 0));
        panelInfo.add(inputName,    Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 0, 0), 0, 0));
        
        
        field = dco.getField(SoftwarePlatform._B_ICON);
        JLabel labelIcon = (JLabel) labels.get(field);
        JComponent inputIcon = (JComponent) fields.get(field);
        inputIcon.setPreferredSize(new Dimension(200, 20));
        labelIcon.setPreferredSize(new Dimension(100, 20));
            
        panelInfo.add(labelIcon,    Layout.getGBC(0, 2, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(0, 5, 0, 5), 0, 0));
        panelInfo.add(inputIcon,    Layout.getGBC(1, 2, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                  new Insets(0, 5, 0, 0), 0, 0));        

        tabbedPane.addTab(DcResources.getText("lblDescription"), IconLibrary._icoInformation, panelInfo);
    } 
}
