/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views;

import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JComponent;

import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;

public abstract class View extends DcPanel {

    protected DataTask task;
    private final int index; 
    
    public View(String title, ImageIcon icon, int index) {
        super(title, icon);  
        this.index = index;
    }    
    
    protected boolean isTaskRunning() {
        boolean isTaskRunning = task != null && task.isRunning();
        if (isTaskRunning)
            new MessageBox(DcResources.getText("msgJobRunning"), MessageBox._WARNING); 
        
        return isTaskRunning; 
    }      
    
    public DataTask getCurrentTask() {
        return task;
    }
    
    public void markAsUnchanged() {
    }
    
    public int getIndex() {
        return index;
    }
    
    public abstract JComponent[] getExtraComponents();
    
    public abstract DcObject getObject(int row);
    
    public abstract Collection getSelectedObjects();
    public abstract DcObject getSelectedObject();
    
    public abstract DcObject getDcObject(String id);
    
    public abstract int[] getSelectedRows();
    public abstract int getRows();
    
    public abstract void applySettings();
    
    public abstract boolean isChangesSaved();
    public abstract void add(DcObject dco);
    public abstract void add(DcObject dco, boolean select);
    public abstract void add(Collection objects);
    public abstract void add(DcObject[] objects);
    public abstract void clear();
    public abstract void open();
    public abstract void createReport();
    public abstract void delete();
    
    public abstract void removeRows(int[] rows);
    public abstract void removeRows(String[] ids);
    public abstract void openUpdateAllDialog();
    
    public abstract void save();
    public abstract void saveSelected();
    public abstract void updateRow(String ID, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark);
    public abstract void updateRow(int row, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark);
    
    public abstract void startAmazonUpdate();
    public abstract void startIMDBUpdate();
    
    public abstract void showQuickView(boolean b);
    public abstract void removeFromCache(String id);    
    
    public abstract void applyDefinitions();
}
