/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.table;

import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.book.AmazonBookSearchForm;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.components.renderers.DcTableCellRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.forms.AmazonSearchForm;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.movie.OnlineMovieSearchForm;
import net.sf.dc.console.software.AmazonSoftwareSearchForm;
import net.sf.dc.console.views.BaseView;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.AddRowAction;
import net.sf.dc.core.actions.RemoveRowAction;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public abstract class TableBaseView extends BaseView {

    protected DcTable table;

    protected boolean updateQuickView = true;

    protected QuickViewPanel quickView;
    protected JPanel panelInput  = new JPanel();
    protected JPanel panelStatus = getStatusPanel();
    protected JScrollPane scrollResults1;
    protected JScrollPane scrollResults2;

    protected TableBaseView childView;
    
    protected TableActionsPanel panelTableActions;

    protected final TableMouseListener tableMouseListener = new TableMouseListener();

    private final DcModule moduleChild;

    private final Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();

    public TableBaseView(DcModule module, String title, ImageIcon icon) {
        super(module, title, icon, MasterView._TABLE_VIEW);
    	this.moduleChild = module != null ? module.getChildModule() : null;
    }

    public JComponent[] getExtraComponents() {
        return new JComponent[0];
    }

    public void markAsUnchanged() {
        table.clearCache();
    }     
    
    public DcModule getChildModule() {
        return moduleChild;
    }
    
    public DcObject getObject(int row) {
    	return table.getDCObject(row);
    }
    
    public DcObject getDcObject(String ID) {
        int row = table.getRowNumberWithID(ID);
        
        DcObject dco = null;
        if (row > -1) 
            dco = table.getDCObject(row);
        
        return dco;
    }  

    public void applyDefinitions() {
        table.applyDefinitions();
    }

    public int getRows() {
        return table.getRowCount();
    }

    public int[] getSelectedRows() {
        return table.getSelectedRows();
    }

    public void applySettings() {
        setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        table.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        panelTableActions.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));

        if (quickView != null)
            quickView.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));

        panelInput.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));
        panelStatus.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));
    }

    public void updateRow(int index, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        table.updateRowAt(index, dco, overwrite, allowDeletes, mark);
    }

    public boolean hasChild() {
        return getChildModule() != null;
    }

    protected boolean isTaskRunning() {
        return task != null && task.isRunning();
    }

    public void add(Collection c) {
        DcObject[] objects = new DcObject[c.size()];
        objects = (DcObject[]) c.toArray(objects);

        task = new FillerThread(objects, this instanceof TableSearchView);
        task.start();
    }

    public void add(DcObject[] objects) {
        task = new FillerThread(objects, this instanceof TableSearchView);
        task.start();
    }
    
    protected void setSelectedRow() {
        try {
            if (this instanceof TableSearchView) {
                table.setSelectedRow(0);
                ((TableSearchView) this).setDataForSelectedRow();
            } else {
                table.setSelectedRow(0);
            }
        } catch (Exception exp) {}        
    }
    
    public void clear() {
        ClearThread thread = new ClearThread();
        thread.start();
    }  
    
    private class ClearThread extends Thread {

        public void run() {
            
            boolean saved = isChangesSaved();
            if (!saved && getCurrentTask() != null) {
                try {
                    getCurrentTask().join();
                } catch (Exception exp) {
                    Messages.add(exp);
                }
            }
            
            table.clearTable();
            
            if (hasChild()) {
                childView.clear();
                
                if (childView instanceof TableSearchView)
                    ((TableSearchView) childView).setParentID(null);
                else 
                    ((TableInsertView) childView).setParentID(null);
            }
    
            if (quickView != null)
                quickView.clear();
            
            setStatus(DcResources.getText("msgTableCleared"));
        }
    }      
    
    public void updateRow(String id, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        int row = table.getRowNumberWithID(id);
        
        if (row != -1)
            table.updateRowAt(row, dco, overwrite, allowDeletes, mark);
    }

    public void denyActions() {
    	table.setCursor(new Cursor(Cursor.WAIT_CURSOR));
    	
    	removeMouseListeners();

    	panelTableActions.buttonClear.setEnabled(false);
        panelTableActions.buttonSave.setEnabled(false);
    }

    public void allowActions() {
    	table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    	removeMouseListeners();
        table.addMouseListener(tableMouseListener);
        panelTableActions.buttonClear.setEnabled(true);
        panelTableActions.buttonSave.setEnabled(true);
    }

    private void removeMouseListeners() {
        MouseListener[] listeners = table.getMouseListeners();
        if (listeners != null) {
            for (int i = 0; i < listeners.length; i++) {
                if (listeners[i] instanceof TableMouseListener)
                    table.removeMouseListener(listeners[i]);
            }
        }
    }
    
    public void removeRows(String[] ids) {
        for (int i = 0; i < ids.length; i++) {
            int row = table.getRowNumberWithID(ids[i]);
            
            if (row > -1) {
                table.removeRow(row);
                removeFromCache(ids[i]);
            }
        }
    }

    public void removeRows(int[] rows) {
        for (int i = 0; i < rows.length; i++) {
            table.removeRow(rows[i]);
        }
    }

    public void cancelTask() {
        allowActions();
        if (isTaskRunning()) {
            task.cancel();
        }
        updateProgressBar(0);
    }

    public DcObject[] getObjects() {
    	return table.getDCObjects();
    }

    protected void addCopyPaste(DcPopupMenu popupMenu) {
        JMenuItem menuItemCopy = new JMenuItem(DcResources.getText("lblCopy"), IconLibrary._icoCopy);
        menuItemCopy.addActionListener(new CopyAction());

        JMenuItem menuItemPaste = new JMenuItem(DcResources.getText("lblPaste"), IconLibrary._icoPaste);
        menuItemPaste.addActionListener(new PasteAction());

        popupMenu.add(new JSeparator());
        popupMenu.add(menuItemCopy);
        popupMenu.add(menuItemPaste);
    }

    public void cancelEdit() {
        int selectedRow = table.getSelectedRow();
        if (selectedRow != -1) {
            for (int i = 0; i < table.getColumnCount(); i++) {
                boolean success = false;
                try {
                    table.getCellEditor(selectedRow, i).stopCellEditing();
                    success = true;
                } catch (Exception ignore) {}

                if (!success) {
                    table.getCellEditor(selectedRow, i).cancelCellEditing();
                }
            }
        }
    }

    public void startAmazonUpdate() {
        cancelEdit();
        AmazonSearchForm searchForm = null;
        if (getModule().getIndex() == DcModules._SOFTWARE)
            searchForm = new AmazonSoftwareSearchForm(null, DataCrow.mainFrame.getSelectedTab(), true);
        else
            searchForm = new AmazonBookSearchForm(null, DataCrow.mainFrame.getSelectedTab(), true);

        searchForm.setVisible(true);
    }

    public void startIMDBUpdate() {
        cancelEdit();
        OnlineMovieSearchForm searchForm = new OnlineMovieSearchForm(null, DataCrow.mainFrame.getSelectedTab(), true);
        searchForm.setVisible(true);
    }

    public abstract DcPopupMenu getPopupMenu();
    public abstract void open();
    public abstract void setParentID(String ID);
    public abstract void createReport();
    public abstract boolean changesSaved();


    public void add(DcObject dco) {
    	add(dco, true);
    }

    public void add(DcObject dco, boolean select) {
        if (dco.getID() == null || dco.getID().equals("")) 
        	dco.setIDs();
        
        try {
        	denyActions();
        	table.addObject(dco, select);
        } finally {
        	allowActions();
        }
    }

    public class TableActionsPanel extends JPanel {
        private JButton buttonClear;
        private JButton buttonSave;
        private JButton buttonAdd;
        private JButton buttonRemove;
        private JButton buttonCancel;
        
        private JPanel panelLeft = new JPanel();
        private JPanel panelRight = new JPanel();

        public TableActionsPanel(boolean insertModus, JComponent[] extraComponents) {
            setLayout(Layout.getGBL());

            panelLeft.setLayout(new FlowLayout(FlowLayout.LEFT));
            panelRight.setLayout(new FlowLayout(FlowLayout.LEFT));
            
            // Create the components
            buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
            buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));
            buttonAdd = ComponentFactory.getButton(DcResources.getText("lblAdd"));
            buttonRemove = ComponentFactory.getButton(DcResources.getText("lblRemove"));
            buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));

            buttonCancel.setMnemonic(KeyEvent.VK_X);
            buttonClear.setMnemonic(KeyEvent.VK_L);
            buttonCancel.setMnemonic(KeyEvent.VK_C);
            buttonSave.setMnemonic(KeyEvent.VK_S);
            buttonAdd.setMnemonic(KeyEvent.VK_A);
            buttonRemove.setMnemonic(KeyEvent.VK_R);
            
            buttonSave.setToolTipText(DcResources.getText("tpSaveChanges"));
            buttonClear.setToolTipText(DcResources.getText("tpClear"));
            buttonAdd.setToolTipText(DcResources.getText("tpAddRow"));
            buttonRemove.setToolTipText(DcResources.getText("tpRemoveRow"));
            buttonCancel.setToolTipText(DcResources.getText("tpCancel"));

            buttonClear.addActionListener(new ClearButtonAction());
            buttonAdd.addActionListener(new AddRowAction(getModule().getIndex()));
            buttonRemove.addActionListener(new RemoveRowAction(getModule().getIndex()));
            buttonCancel.addActionListener(new CancelAction());

            // Build the panel
            if (insertModus) {
                panelLeft.add(buttonAdd);
                panelLeft.add(buttonRemove);
            }

            int counter = 2;
            if (extraComponents != null) {
                for (int i = 0; i < extraComponents.length; i++) {
                    panelLeft.add(extraComponents[i]);
                    counter++;
                }
            }

            panelRight.add(buttonSave);
            
            if (insertModus)
                panelRight.add(buttonClear);
            
            panelRight.add(buttonCancel);
            
            add(panelLeft,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                            new Insets(0, 5, 0, 5), 0, 0));
            add(panelRight, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                            new Insets(0, 5, 0, 5), 0, 0));
        }
        
        public void setFont(Font font) {
            super.setFont(font);
            if (buttonClear != null) {
                buttonClear.setFont(font);
                buttonSave.setFont(font);
                buttonAdd.setFont(font);
                buttonRemove.setFont(font);
                buttonCancel.setFont(font);
            }
        }

        public void addSaveActionListener(ActionListener listener) {
            buttonSave.addActionListener(listener);
        }

        public void removeSaveActionListener(ActionListener listener) {
            buttonSave.removeActionListener(listener);
        }
    }

    public class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clear();
        }
    }

    public class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelTask();
        }
    }

    protected class CopyAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
          StringBuffer sb = new StringBuffer();
          int[] rows = getSelectedRows();
          DcFieldDefinitions definitions = (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(getModule().getFieldDefinitionsSettingsKey());

          for (int i = 0; i < rows.length; i++) {
              DcObject dco = table.getDCObject(rows[i]);
              int fieldCount = 0;
              for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                  DcFieldDefinition fieldDef = (DcFieldDefinition) iter.next();
                  if (fieldDef.isVisible() && fieldDef.isEnabled()) {
                      if (fieldCount > 0)
                          sb.append("\t");

                      int field = fieldDef.getIndex();
                      Object o = dco.getValue(field);
                      String value = "";
                      value = value == null ? "" : value;
                      if (dco.getField(field).getValueType() == Repository.datatypes._PICTURE) {
                          value = "";
                      } else {
                          value = o == null ? "" : o.toString();
                      }

                      sb.append(value);
                      fieldCount++;
                  }
             }
             sb.append("\n");
          }
          StringSelection stSelection = new StringSelection(sb.toString());
          clipboard.setContents(stSelection, stSelection);
       }
    }

    protected class PasteAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
           int row = table.getSelectedRow();
           DcFieldDefinitions definitions = (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(getModule().getFieldDefinitionsSettingsKey());

           int[] fields = new int[getModule().getDcObject().getVisibleFields().size()];
           int counter = 0;
           for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
               DcFieldDefinition fieldDef = (DcFieldDefinition) iter.next();
               if (fieldDef.isVisible() && fieldDef.isEnabled()) {
                   fields[counter++] = fieldDef.getIndex();
               }
           }

           try {
              String text = (String) (clipboard.getContents(this).getTransferData(DataFlavor.stringFlavor));
              StringTokenizer rowTokenizer = new StringTokenizer(text, "\n");
              for (int i = 0; rowTokenizer.hasMoreTokens(); i++) {
                 String values = rowTokenizer.nextToken();
                 DcObject dco = getModule().getDcObject();

                 int fldCounter = 0;
                 int index = values.indexOf("\t");
                 while (index > -1) {
                     String value = values.substring(0, index);
                     if (fldCounter <= fields.length) {
                         DcField field = dco.getField(fields[fldCounter]);
                         if (field.getValueType() == Repository.datatypes._INTEGER) {
                             try {
                                 dco.setValue(fields[fldCounter], new Integer(value.trim()));
                             } catch (Exception exp) {}
                         } else {
                             dco.setValue(fields[fldCounter], value);
                         }
                     }

                     fldCounter++;
                     values = values.substring(index + 1, values.length());
                     index = values.indexOf("\t");
                 }
                 dco.setValue(fields[fldCounter], values);
                 table.updateRowAt(row++, dco, true, true, true);
             }
          } catch (Exception exp){
              Messages.add(exp);
          }
       }
    }

    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {

            try {
                table.setListeningForChanges(false);
            
                if (SwingUtilities.isRightMouseButton(e)) {
                    int row = table.rowAtPoint(e.getPoint());
    
                    if (table.getSelectedRows() != null && table.getSelectedRows().length < 2) {
    
                        int col = table.getSelectedColumn() != -1 ? table.getSelectedColumn() :
                                                                    table.columnAtPoint(e.getPoint());
                        table.addRowSelectionInterval(row, row);
                        table.addColumnSelectionInterval(col, col);
    
                        ((DcTableCellRenderer) table.getCellRenderer(row, col)).
                            getTableCellRendererComponent(
                                    table,
                                    table.getValueAt(row, col, true),
                                    true,
                                    true,
                                    row,
                                    col);
    
                    }
    
                    DcPopupMenu menu = getPopupMenu();
                    if (menu != null)
                        menu.show(table, e.getX(), e.getY());
                }
    
                if (e.getClickCount() == 2) {
                	cancelEdit();
                    open();
                }
            } finally {
                table.setListeningForChanges(true);
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }

    public class FillerThread extends DataTask {
        
        private final DcObject[] objects;
        private final boolean clearViewFirst;
        
        public FillerThread(DcObject[] objects, boolean clearViewFirst) {
            super(null);
            this.objects = objects;
            this.clearViewFirst = clearViewFirst;
        }
        
        public void run() {
            boolean listening = table.isListeningForChanges();
            
            try {
                table.setListeningForChanges(false);
                
                denyActions();
                
                if (clearViewFirst) {
                    ClearThread thread = new ClearThread();
                    thread.start();
                    
                    try {
                        thread.join();
                    } catch (Exception exp) {
                        Messages.add(exp);
                    }
                }
                
                updateProgressBar(0);
                initProgressBar(objects.length);
                updateQuickView = false;
                
                int i = 0;
                
                table.setVisible(false);
                for (; i < objects.length; i++) {
                    DcObject dco = objects[i];

                    add(dco, false);
                    
                    setStatus(DcResources.getText("msgAddingXToView", dco.toString()));
                    updateProgressBar(i + 1);
                }
    
                if (clearViewFirst)
                    setStatus(DcResources.getText("msgSearchResult", "" + i));
                else 
                    setStatus(DcResources.getText("msgAddedXItemsToView", "" + i));
            } catch (Exception exp) {
                Messages.add(exp);
            } finally {
            	table.setVisible(true);
                updateQuickView = true;
                allowActions();
                table.setListeningForChanges(listening);
            }
        }
    }     
    
    public class CreateReportAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createReport();
        }
    }
}

