/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.table;

import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.StatusUpdateRequest;
import net.sf.dc.wf.requests.UpdateStatusProgressBarRequest;

public class TableInsertView extends TableBaseView {

    protected String parentID = null;
    

    public TableInsertView (DcModule module, String title, ImageIcon icon) {
        super(module, title, icon);
        setHelpIndex("dc.Inserting");

        JComponent[] extraComponents = getAllExtraComponents();

        panelTableActions = new TableActionsPanel(true, extraComponents);
        panelTableActions.addSaveActionListener(new SaveNewValuesAction());

        buildPanel();
    }

    public void add(DcObject dco, boolean select) {
        if (dco.getID() == null || dco.getID().equals(""))
            dco.setIDs();

        table.addObject(dco, select);

        if (hasChild()) {
            childView.add(dco.getChildren());
            childView.setParentID(dco.getID());
        }
    }

    public void delete() {
        table.removeRows(table.getSelectedRows());
    }

    public void showQuickView(boolean b) {}

    public void open() {
        cancelEdit();

        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(DcResources.getText("msgSelectRowToOpen"), MessageBox._WARNING);
        } else {
        	DcObject dco = getObjectForSaving(selectedRow);
            dco.removeRequestors();
            open(dco, false);
        }
    }

    public void createReport() {
        Collection objects = new ArrayList();
        for (int i = 0; i < table.getRowCount(); i++) {
            objects.add(getObjectForSaving(i));
        }
        ReportingDialog dialog = new ReportingDialog(objects);
        dialog.setVisible(true);
    }

    public void setParentID(String sID) {}

    public boolean changesSaved() {
        return table.getRowCount() == 0;
    }
    
    public boolean isChangesSaved() {
        cancelEdit();

        boolean isChangesSaved = true;
        boolean saved = changesSaved();
        saved = hasChild() ? saved && childView.changesSaved() : saved;
        if (!saved) {
            new QuestionBox(DcResources.getText("msgNotSaved"));
            if (QuestionBox.result.equals(Boolean.TRUE)) {
                save();
                if (hasChild())
                    childView.save();

                isChangesSaved = false;
            } else {
                table.clearCache();
            }
        }
        return isChangesSaved;
    }    
    
    public void clearTable(boolean cache) {}

    private JComponent[] getAllExtraComponents() {
        JComponent[] components = new JComponent[0];
        if (hasChild()) {
            JComponent[] childComponents = getChildModule().getInsertView().get(MasterView._TABLE_VIEW).getExtraComponents();
            JComponent[] parentComponents = getExtraComponents();

            components = new JComponent[parentComponents.length + childComponents.length];
            int counter = 0;
            for (int i = 0; i < childComponents.length; i++) {
                components[counter] = childComponents[i];
                counter++;
            }
            for (int i = 0; i < parentComponents.length; i++) {
                components[counter] = parentComponents[i];
                counter++;
            }
        }
        return components;
    }

    public DcObject getSelectedObject() {
        int row = table.getSelectedRow();

        DcObject dco = null;
        if (row == -1) {
            new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
        } else {
            dco = getObjectForSaving(row);
        }
        return dco;
    }

    public DcObject getObjectForSaving(int row) {
        DcObject dco = table.getDCObject(row);

        dco.setBatch(true);
        dco.setSilence(true);
        dco.setRow(row);
        dco.setValue(DcObject._ID, dco.getID());

        if (hasChild()) {
            childView.setParentID(dco.getID());
            Collection children = ((TableInsertView) childView).getChildren(dco.getID());

            for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            	DcObject child = (DcObject) iter.next();
            	dco.addChild(child);
            }
        }
        return dco;
    }

    public Collection getSelectedObjects() {
        int[] rows = table.getSelectedRows();

        Collection objects = new ArrayList();
        if (rows.length > 0) {
            for (int i = 0; i < rows.length; i++) {
                int row = rows[i];
                objects.add(getObjectForSaving(row));
            }
        } else {
            new MessageBox(DcResources.getText("msgNoRowsSelected"),
                           MessageBox._WARNING);
        }

        return objects;
    }

    public void save(int[] rows) {
        if (rows.length > 0) {
        	if (!isTaskRunning()) {
                task = new SavingThread(rows, getClass());
                task.start();
            }
        } else {
            new MessageBox(DcResources.getText("msgNoItemsToSave"), MessageBox._WARNING);
        }
    }

    public void save() {
        int rowCount = table.getRowCount();
        int[] rows = new int[rowCount];
        for (int i = 0; i < rowCount; i++) {
        	rows[i] = i;
        }
        save(rows);
    }

    public void saveSelected() {
        cancelEdit();
        save(table.getSelectedRows());
    }

    public DcPopupMenu getPopupMenu() {
        DcObject dco = getObjectForSaving(table.getSelectedRow());
        DcPopupMenu popupMenu = new DcPopupMenu(dco, DcPopupMenu._INSERT);
        addCopyPaste(popupMenu);
        return popupMenu;
    }

    public String getParentID() {
    	return parentID;
    }

    public void delete(String[] ids) {
        if (hasChild()) {
            childView.clear();
            childView.setParentID(null);
        }
        super.removeRows(ids);
    }

    protected Collection getChildren(String parentID) {
        return null;
    }

    public JComponent[] getExtraComponents() {
        return null;
    }

    private void buildPanel() {
        //**********************************************************
        //Input panel
        //**********************************************************
        panelInput.setLayout(Layout.getGBL());

        table = ComponentFactory.getDCTable(getModule(), false, false);
        table.addMouseListener(tableMouseListener);

        scrollResults1 = new JScrollPane(table);
        scrollResults1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        panelInput.add( scrollResults1,      Layout.getGBC( 0, 0, 7, 1, 20.0, 20.0
                       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));

        if (hasChild()) {
            childView = (TableInsertView) getChildModule().getInsertView().get(MasterView._TABLE_VIEW);

            table.getSelectionModel().addListSelectionListener(new RowSelectionListener());
            table.addKeyListener(new ChangeRowByKey());

            scrollResults2 = new JScrollPane(childView.table);
            scrollResults2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

            panelInput.add( scrollResults2,   Layout.getGBC( 0, 1, 7, 1, 20.0, 20.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        }

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        add(    panelInput,       Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 0, 5), 0, 0));
        add(    panelTableActions,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));
        add(    panelStatus,      Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************

    private class SavingThread extends DataTask {
        public SavingThread(int[] rows, Class clazz) {
            super(rows);
        }

        public void run() {
        	try {
	            updateProgressBar(0);
	            initProgressBar(rows.length);
	            setStatus("Saving " + rows.length + " items to the database");
	            table.setCursor(new Cursor(Cursor.WAIT_CURSOR));

                if (hasChild())
                    childView.setParentID(null);

	            denyActions();
	            startRunning();
	            cancelEdit();

	            if (rows.length == 0) {
	                new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._INFORMATION);
	                stopRunning();
	            }

	            boolean ignoreErrors = false;
	            int counter = 1;
	            for (int i = rows.length - 1; i > -1 && keepOnRunning(); i--) {
                    table.setSelectedRow(rows[i]);
                    
	                DcObject dco = getObjectForSaving(rows[i]);
	                dco.setRow(rows[i]);
                    dco.setSilence(true);

                    if (i == 0) {
                        dco.addRequest(new StatusUpdateRequest(
                                dco.getModule().getIndex(), MainFrame._INSERTTAB, "msgDataSaved"));
                        dco.setEndOfBatch(true);
                        dco.setSilence(false);
                    }

                    dco.addRequest(new UpdateStatusProgressBarRequest(
                            dco.getModule().getIndex(), MainFrame._INSERTTAB, counter++));

	                try {
	                    dco.saveNew(true);
	                } catch (ValidationException vExp) {
	                    if (!ignoreErrors) {
	                        SaveQuestionBox question = new SaveQuestionBox(vExp);
	                        if (question.getResult() == SaveQuestionBox._CANCEL) {
	                            stopRunning();
	                        } else if (question.getResult() == SaveQuestionBox._IGNORE) {
	                            ignoreErrors = true;
	                        }
	                    }
	                }

	                try {
	                	while (DatabaseManager.getQueueSize() > 0)
	                		sleep(300);
	                } catch (Exception ignore) {}
	            }
        	} finally {
        		table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        		allowActions();
        		stopRunning();

                if (hasChild() && table.getRowCount() > 0) {
                	table.setSelectedRow(0);
                    int col = table.getColumnIndexForField(DcObject._ID);
                    childView.setParentID(table.getValueAt(0, col, true).toString());
                }
        	}
        }
    }

    private class ChangeRowByKey implements KeyListener {

        private int rowFrom = -1;
        private int rowTo = -1;

        public void keyPressed(KeyEvent e) {
           if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN) {
           	   rowFrom = table.getSelectedRow();
           }
        }

		public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN) {
    			rowTo = table.getSelectedRow();

                if (rowTo != rowFrom) {
                    String idTo =  table.getObjectID(rowTo);
                    childView.setParentID(idTo);
                } else if (table.getRowCount() == 1) {
                    String id = table.getObjectID(0);
                    childView.setParentID(id);
                }
            }
        }

		public void keyTyped(KeyEvent arg0) {}
    }

    private class RowSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e){
            try {
                if (hasChild()) {
                    String idTo = "";
                    if (e.getFirstIndex() == table.getSelectedRow()) {
                        idTo = "" + table.getObjectID(e.getFirstIndex());
                    } else { 
                        idTo = "" + table.getObjectID(e.getLastIndex());
                    }
                    childView.setParentID(idTo);
                }
            } catch (Exception ignore) {}
        }
    }

    private class SaveNewValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }

    public void removeFromCache(String id) {}
}
