/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.table;

import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.RemoveFromCacheRequest;
import net.sf.dc.wf.requests.StatusUpdateRequest;
import net.sf.dc.wf.requests.UpdateStatusProgressBarRequest;

public class TableSearchView extends TableBaseView {

    protected RowSelectionListener rowSelectionListener = new RowSelectionListener();
    protected ChangeRowByKey changeRowByKey = new ChangeRowByKey();

    private String parentID = "";

    public TableSearchView(DcModule module, String title) {
        super(module, title, module.getIconSmall());
        setHelpIndex("dc.Searching.SearchResult");

        JComponent[] extraComponents = getAllExtraComponents();

        panelTableActions = new TableActionsPanel(false, extraComponents);
        panelTableActions.addSaveActionListener(new SaveUpdatedValuesAction());

        buildPanel();
    }

    public void showQuickView(boolean b) {
        quickView.setVisible(b);
    }

    /**
     * Retrieves the module specific components
     */
    private JComponent[] getAllExtraComponents() {
        JComponent[] components = new JComponent[0];

        JComponent[] childComponents = new JComponent[0];
        if (hasChild())
        	childComponents = getChildModule().getSearchView().get(MasterView._TABLE_VIEW).getExtraComponents();

        JComponent[] parentComponents = getExtraComponents();
        components = new JComponent[parentComponents.length + childComponents.length];
        int counter = 0;

        if (childComponents != null) {
            for (int i = 0; i < childComponents.length; i++) {
                components[counter] = childComponents[i];
                counter++;
            }
        }

        if (parentComponents != null) {
            for (int i = 0; i < parentComponents.length; i++) {
                components[counter] = parentComponents[i];
                counter++;
            }
        }
        return components;
    }

    public JComponent[] getExtraComponents() {
        return new JComponent[0];
    }

    public void setParentID(String sID) {
        parentID = sID;
    }

    public String getParentID() {
    	return parentID;
    }

    public boolean isChangesSaved() {
        cancelEdit();

        boolean isChangesSaved = true;
        boolean saved = changesSaved();
        saved = hasChild() ? saved && ((TableSearchView) childView).changesSaved() : saved;
        if (!saved) {
            new QuestionBox(DcResources.getText("msgNotSaved"));
            if (QuestionBox.result.equals(Boolean.TRUE)) {
                save();
                if (hasChild()) {
                    childView.save();
                }
                isChangesSaved = false;
            } else {
                table.clearCache();
            }
        }
        
        return isChangesSaved;
    }

    public void removeFromCache(String sID) {
    	table.removeFromCache(sID);
        removeNewChild(sID);
    }

    public Collection getSelectedObjects() {
        int[] rows = table.getSelectedRows();

        Collection objects = new ArrayList();
        if (rows.length > 0) {
            for (int i = 0; i < rows.length; i++) {
                int row = rows[i];
                objects.add(getObjectForRow(row, false));
            }
        } else {
            new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
        }

        return objects;
    }

    public DcObject getSelectedObject() {
    	int row = table.getSelectedRow();
    	DcObject dco = null;
    	if (row != -1) {
    		dco = getObjectForRow(row, true);
    	} else {
    		new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
    	}
    	return dco;
    }

    public DcModule getModuleForRow(int row) {
        return table.getModuleForRow(row);
    }

    public void open() {
        cancelEdit();
        int row = table.getSelectedRow();
        if (row == -1) {
            new MessageBox(DcResources.getText("msgSelectRowToOpen"), MessageBox._WARNING);
        } else {
            DcObject dco = getObjectForRow(row, true);
            if (DcModules.getCurrentModule().getIndex() == DcModules._GENERIC) {
                dco.reload();
                dco.loadChildren();
            }
            
            open(dco, true);
        }
    }

    public void updateRow(String id, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        super.updateRow(id, dco, overwrite, allowDeletes, mark);
        
        int row = table.getRowNumberWithID(id);
        if (quickView != null && row > -1)
            quickView.setObject(getObjectForRow(table.getSelectedRow(), true));
    }

    public void createReport() {
        Collection objects = new ArrayList();
        for (int i = 0; i < table.getRowCount(); i++) {
            objects.add(getObjectForRow(i, false));
        }

        if (objects.size() > 0) {
            ReportingDialog dialog = new ReportingDialog(objects);
            dialog.setVisible(true);
        } else {
            new MessageBox(DcResources.getText("msgReportNoItems"), MessageBox._WARNING);
        }
    }

    public void removeNewChild (String sID) {}

    public void delete() {
        if (!isTaskRunning()) {
            int[] rows = table.getSelectedRows();
            if (rows.length > 0) {
                task = new DeletingThread(rows);
                task.start();
            } else {
                new MessageBox(DcResources.getText("msgSelectItemToDel"), MessageBox._WARNING);
            }
        }
    }

    public void saveSelected() {
        if (!isTaskRunning()) {
            int[] rows = table.getSelectedRows();
            task = new SavingThread(rows);
            task.start();
        }
    }

    public void save() {
        save(true);
    }

    public void save(boolean threaded) {
        if (!isTaskRunning()) {
            int[] rows = table.getChangedObjectsIndices();

            boolean childrenSaved = true;
            if (hasChild()) {
                childView.cancelEdit();
            	childrenSaved = childView.changesSaved();
            }

            if (rows.length > 0 || !childrenSaved) {
                task = new SavingThread(rows);
                task.startRunning();
                
                if (threaded) {
                    task.start();
                } else {
                	task.run();
                }
            } else {
                new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
            }
        }
    }

    public DcPopupMenu getPopupMenu() {
        DcObject dco = getSelectedObject();
        DcPopupMenu popupMenu = new DcPopupMenu(dco, DcPopupMenu._SEARCH);
        addCopyPaste(popupMenu);
        return popupMenu;
    }

    public DcObject getObjectForRow(int row, boolean allChildren) {
        DcObject dco = table.getDCObject(row);
        dco.removeChildren();
        if (hasChild()) {
            DcObject[] children = null;
            if (allChildren) {
                children = childView.getObjects();
            } else {
            	children = ((TableSearchView) childView).getChangedObjects();
            }

            for (int j = 0; j < children.length; j++) {
                DcObject child = children[j];
                child.addRequest(new RemoveFromCacheRequest(childView.getModule().getIndex(), child.getID()));
                dco.addChild(child);
            }
        }

        dco.setSilence(true);
        dco.setBatch(true);
        return dco;
    }

    public void removeRows(String[] ids) {
        super.removeRows(ids);

        if (hasChild()) {
            childView.clear();
            childView.setParentID(null);
        }

        if (quickView != null)
            quickView.clear();
    }

    public DcObject[] getChangedObjects() {
        return table.getChangedObjects();
    }

    public boolean changesSaved() {
        return table.noChanges();
    }

    public void saveChildItems() {
        if (hasChild() && table.getSelectedRow() > -1) {
            childView.cancelEdit();
            if (!childView.changesSaved()) {
            	new QuestionBox(DcResources.getText("msgNotSaved"));
                if (QuestionBox.result.equals(Boolean.TRUE)) {
                    save(false);
                }
            }
        }
    }

    public void loadChildValues() {}

    private void buildPanel() {
        //**********************************************************
        //Search result panel
        //**********************************************************
        panelInput.setLayout(Layout.getGBL());
        
        table = ComponentFactory.getDCTable(getModule(), false, true);
        table.addMouseListener(tableMouseListener);

        scrollResults1 = new JScrollPane(table);
        scrollResults1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollResults1.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

        panelInput.add(scrollResults1,  Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));

        int length = 1;

        table.getSelectionModel().addListSelectionListener(rowSelectionListener);
        table.addKeyListener(changeRowByKey);

        if (hasChild()) {
        	DcModule module  = getChildModule();
            childView = (TableSearchView) module.getSearchView().get(MasterView._TABLE_VIEW);

            scrollResults2 = new JScrollPane(childView.table);
            scrollResults2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            scrollResults2.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

            panelInput.add(    scrollResults2,  Layout.getGBC( 0, 1, 1, 1, 80.0, 80.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
            length = 2;
        }

        quickView = getModule().getQuickView();
        if (quickView != null) {
            boolean showQuickView = DcSettings.getValueAsBoolean(Repository.settings.stShowQuickView);
            quickView.setVisible(showQuickView);
            panelInput.add(quickView,  Layout.getGBC( 1, 0, 1, length, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.VERTICAL,
                            new Insets(5, 5, 5, 0), 0, 0));
        }

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        add(    panelInput,       Layout.getGBC( 0, 0, 1, 1, 100.0, 100.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 0, 5), 0, 0));
        add(    panelTableActions,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));
        add(    panelStatus,      Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    protected void addAfterDeleteRequest(DcObject dco) {}

    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************
    private class DeletingThread extends DataTask {

        public DeletingThread(int[] rows) {
        	super(rows);
        }

        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(rows.length);

                setStatus("Deleting " + rows.length + " items from to the database");

                startRunning();
                cancelEdit();
                denyActions();

                new QuestionBox(DcResources.getText("msgDeleteQuestion"));
                if (!QuestionBox.result.booleanValue()) {
                    stopRunning();
                }

                int counter = 1;
                for (int i = rows.length -1; i > -1 && keepOnRunning(); i--) {
                    DcObject dco = getObjectForRow(rows[i], false);

                    dco.markAsUnchanged();
                    dco.setSilence(true);
                    dco.setEndOfBatch(i == 0);

                    dco.addRequest(new UpdateStatusProgressBarRequest(
                            dco.getModule().getIndex(), MainFrame._SEARCHTAB, counter++));                        
                    
                    if (i == 0) {
                        dco.addRequest(new StatusUpdateRequest(getModuleForRow(rows[i]).getIndex(),
                                MainFrame._SEARCHTAB, "msgDeleteSuccessfull"));
                        
                        addAfterDeleteRequest(dco);
                        dco.setSilence(false);
                    }

                    dco.delete();

                    try { sleep(300); } catch (Exception ignore) {}
                }

                table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            } finally {
                if (getChildModule() != null)
                    getChildModule().getCurrentSearchView().clear();
                
            	stopRunning();
            	allowActions();
            }
        }
    }

    private class SavingThread extends DataTask {
        public SavingThread(int[] rows) {
            super(rows);
        }

        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(rows.length);
                setStatus("Saving " + rows.length + " items to the database");
                table.setCursor(new Cursor(Cursor.WAIT_CURSOR));

                startRunning();
                cancelEdit();
                denyActions();

                if (hasChild() && !childView.changesSaved() && ((rows == null || rows.length == 0))) {
                	rows = table.getSelectedRows();
                    childView.cancelEdit();
                }

                if (rows == null || rows.length == 0)
                    stopRunning();

                boolean ignoreErrors = false;
                int counter = 1;
                for (int i = rows.length - 1; i > -1 && keepOnRunning(); i--) {
                	try {
	                    updateProgressBar(counter++);
	                    DcObject dco = getObjectForRow(rows[i], false);
	                    dco.setEndOfBatch(i == 0);
	                    dco.addRequest(new RemoveFromCacheRequest(getModule().getIndex(), dco.getID()));
                        dco.setSilence(true);

                        if (dco.isEndOfBatch()) {
                            dco.setSilence(false);
                            dco.addRequest(
                              new StatusUpdateRequest(dco.getModule().getIndex(), MainFrame._SEARCHTAB, "msgDataSaved"));
                        }

                        dco.saveUpdate(true);
                    } catch (ValidationException vExp) {
                        if (!ignoreErrors) {
                            SaveQuestionBox question = new SaveQuestionBox(vExp);
                            if (question.getResult() == SaveQuestionBox._CANCEL)
                                stopRunning();
                            else if (question.getResult() == SaveQuestionBox._IGNORE)
                                ignoreErrors = true;
                        }
                    }

                    try { sleep(300); } catch (Exception ignore) {}
                }
            } finally {
                table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                allowActions();
            	stopRunning();
            }
        }
    }
    
    public void setDataForSelectedRow() {
        if (quickView != null && quickView.isVisible() && updateQuickView) {
            int row = table.getSelectedRow() > -1 ? table.getSelectedRow() : 0;
            
            try {
                quickView.setObject(getObjectForRow(row, true));
            } catch (Exception ignore) {}
        }

        if (hasChild())
           loadChildValues();
    }

    protected class RowSelectionListener implements ListSelectionListener {
    	public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting()) {
                setDataForSelectedRow();
            }
    	}
    }

    protected class ChangeRowByKey implements KeyListener {
        private int rowFrom = -1;
        private int rowTo = -1;

        public void keyPressed(KeyEvent e) {
           if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN)
               rowFrom = table.getSelectedRow();
        }

        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN) {
                rowTo = table.getSelectedRow();
                if (rowFrom != rowTo) {

                    if (hasChild()) {
                        saveChildItems();
                        loadChildValues();
                    }
                    
                    DcObject dco = getObjectForRow(rowTo, true);

                    if (quickView != null && quickView.isVisible() && updateQuickView)
                        quickView.setObject(dco);
                }
            }
        }

        public void keyTyped(KeyEvent arg0) {}
    }

    private class SaveUpdatedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }
}
