/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core;

import java.awt.Toolkit;
import java.io.File;
import java.net.URL;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.dialogs.TipOfTheDayDialog;
import net.sf.dc.console.forms.SplashScreen;
import net.sf.dc.console.menu.DcEditPopupEventQueue;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.LoanModule;
import net.sf.dc.core.modules.PictureModule;
import net.sf.dc.core.modules.audiocd.AudioCDModule;
import net.sf.dc.core.modules.audiocd.AudioCDStateModule;
import net.sf.dc.core.modules.audiocd.AudioTrackModule;
import net.sf.dc.core.modules.book.BookCategoryModule;
import net.sf.dc.core.modules.book.BookModule;
import net.sf.dc.core.modules.book.BookStateModule;
import net.sf.dc.core.modules.contactperson.ContactPersonCategoryModule;
import net.sf.dc.core.modules.contactperson.ContactPersonModule;
import net.sf.dc.core.modules.generic.GenericModule;
import net.sf.dc.core.modules.image.ImageCategoryModule;
import net.sf.dc.core.modules.image.ImageModule;
import net.sf.dc.core.modules.image.ImageStateModule;
import net.sf.dc.core.modules.image.ImageStorageMediaModule;
import net.sf.dc.core.modules.movie.MovieModule;
import net.sf.dc.core.modules.movie.MovieStateModule;
import net.sf.dc.core.modules.movie.MovieStorageMediaModule;
import net.sf.dc.core.modules.musicfile.MusicAlbumModule;
import net.sf.dc.core.modules.musicfile.MusicAlbumStateModule;
import net.sf.dc.core.modules.musicfile.MusicAlbumStorageMediaModule;
import net.sf.dc.core.modules.musicfile.MusicTrackModule;
import net.sf.dc.core.modules.musicfile.MusicTrackStateModule;
import net.sf.dc.core.modules.software.SoftwareCategoryModule;
import net.sf.dc.core.modules.software.SoftwareModule;
import net.sf.dc.core.modules.software.SoftwarePlatformModule;
import net.sf.dc.core.modules.software.SoftwareStateModule;
import net.sf.dc.core.modules.software.SoftwareStorageMediaModule;
import net.sf.dc.core.modules.template.TemplateAudioCdModule;
import net.sf.dc.core.modules.template.TemplateBookModule;
import net.sf.dc.core.modules.template.TemplateImageModule;
import net.sf.dc.core.modules.template.TemplateMovieModule;
import net.sf.dc.core.modules.template.TemplateMusicAlbumModule;
import net.sf.dc.core.modules.template.TemplateMusicTrackModule;
import net.sf.dc.core.modules.template.TemplateSoftwareModule;
import net.sf.dc.core.objects.enhancement.ValueEnhancers;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.DcSettingsConverter;
import net.sf.ext.util.GarbageControl;

public class DataCrow {

    public static String baseDir;
    public static String imageDir;
    public static MainFrame mainFrame;
    private static SplashScreen splashScreen;

    public static void main(String[] args) {
        // set the base directory
        setBaseDir();

        // check if the data directory exists. If not, create it
        File file = new File(baseDir + "data");
        if (!file.exists())
            file.mkdir();

        
        file = new File(imageDir);
        if (!file.exists())
            file.mkdir();
        
        DataCrow application = new DataCrow();

        // Initialize the Resources
        new DcResources();

        showSplashScreen();

        // Initialize all settings
        splashScreen.setStatusMsg("Initializing settings");
        DcSettingsConverter settingsConverter = new DcSettingsConverter();
        settingsConverter.convert();

        new DcSettings();
        
        // Initialize the Component factory
        splashScreen.setStatusMsg("Loading components");
        new ComponentFactory();

        // Start the logging thread
        new Messages();

		Messages.add(DcResources.getText("msgApplicationStarts"), Messages._STANDARD);

        // Initialize all modules
        application.registerModules();

        Messages.add(DcResources.getText("msgSettingsLoaded"), Messages._STANDARD);

        ValueEnhancers.initialize();
        
        // set the database name
        DcSettings.setValue(Repository.settings.stConnectionString, "dc");
        if (args.length == 1 && !args[0].toUpperCase().startsWith("-X"))
            DcSettings.setValue(Repository.settings.stConnectionString, args[0]);

        // Establish a connection to the database / server
        splashScreen.setStatusMsg("Initializing database");
        
        DatabaseManager.initialize();
        
        try {
            // Start the UI
            if (splashScreen == null)
                showSplashScreen();

            splashScreen.setStatusMsg("Loading items");
            DcModules.loadData();
            
            splashScreen.setStatusMsg("Loading UI");
            Toolkit.getDefaultToolkit().getSystemEventQueue().push( new DcEditPopupEventQueue());

            // load the filters
            new DataFilters().load();
            
            mainFrame = new MainFrame();
            mainFrame.setVisible(true);
            
            Thread splashCloser = new Thread(new SplashScreenCloser());
            splashCloser.start();

        } catch (Exception exp) {
            Messages.add(exp);
            exp.printStackTrace();
            System.exit(0);
        }
        
        try {
            new GarbageControl().scheduleRegularGC(20000);
        } catch (Exception exp) {
            Messages.add("Could not start scheduled task: Garbage Collection : " + 
                         exp.getMessage(), Messages._ERROR);
        }
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stShowTipsOnStartup)) {
            TipOfTheDayDialog dlg = new TipOfTheDayDialog();
            dlg.setVisible(true);
        }        
    }

    private void registerModules() {

        splashScreen.setStatusMsg("Loading picture module");
        DcModules.register(new PictureModule());

        
        splashScreen.setStatusMsg("Loading contact person module");
        DcModules.register(new ContactPersonCategoryModule());

        DcModules.register(new ContactPersonModule());


        splashScreen.setStatusMsg("Loading loan module");
        DcModules.register(new LoanModule());

        
        splashScreen.setStatusMsg("Loading software module");
        DcModules.register(new TemplateSoftwareModule());
        DcModules.register(new SoftwareCategoryModule());
        DcModules.register(new SoftwareStateModule());
        DcModules.register(new SoftwarePlatformModule());
        DcModules.register(new SoftwareStorageMediaModule());

        DcModules.register(new SoftwareModule());
        
        splashScreen.setStatusMsg("Loading music module");
        DcModules.register(new TemplateMusicAlbumModule());
        DcModules.register(new TemplateMusicTrackModule());
        DcModules.register(new MusicAlbumStateModule());
        DcModules.register(new MusicTrackStateModule());
        DcModules.register(new MusicAlbumStorageMediaModule());
        
        MusicAlbumModule musicAlbumModule = new MusicAlbumModule();
        MusicTrackModule musicTrackModule = new MusicTrackModule();
        musicAlbumModule.setChildModule(musicTrackModule);
        DcModules.register(musicAlbumModule);
        DcModules.register(musicTrackModule);
        
        splashScreen.setStatusMsg("Loading movie module");
        DcModules.register(new MovieStateModule());
        DcModules.register(new MovieStorageMediaModule());
        DcModules.register(new TemplateMovieModule());
        
        DcModules.register(new MovieModule());
        
        splashScreen.setStatusMsg("Loading audio cd module");
        AudioTrackModule audioTrackModule = new AudioTrackModule();
        AudioCDModule audioCDModule = new AudioCDModule();
        audioCDModule.setChildModule(audioTrackModule);
        DcModules.register(new AudioCDStateModule());
        DcModules.register(new TemplateAudioCdModule());

        DcModules.register(audioCDModule);
        DcModules.register(audioTrackModule);
        
        splashScreen.setStatusMsg("Loading book module");
        DcModules.register(new BookCategoryModule());
        DcModules.register(new BookStateModule());
        DcModules.register(new TemplateBookModule());
        
        DcModules.register(new BookModule());
        
        splashScreen.setStatusMsg("Loading image module");
        DcModules.register(new TemplateImageModule());
        DcModules.register(new ImageCategoryModule());
        DcModules.register(new ImageStorageMediaModule());
        DcModules.register(new ImageStateModule());
        DcModules.register(new ImageModule());
        
        splashScreen.setStatusMsg("Loading generic module");
        DcModules.register(new GenericModule());
    }

    private static void showSplashScreen(){
    	splashScreen = new SplashScreen();
    	splashScreen.splash();
    }

    public static void closeSplashScreen() {
        if (splashScreen != null) {
            splashScreen.dispose();
            splashScreen = null;
        }
    }

    public static void moveSplashScreenToBack() {
        if (splashScreen != null) splashScreen.toBack();
    }

    public static void moveSplashScreenToFront() {
        if (splashScreen != null) splashScreen.toFront();
    }

    private static final class SplashScreenCloser implements Runnable {
    	public void run(){
            if (splashScreen != null) 
                splashScreen.dispose();
                
            splashScreen = null;
    	}
    }

    private static void setBaseDir() {
        final URL location;
        final String classLocation = DataCrow.class.getName().replace('.', '/') + ".class";
        final ClassLoader loader = DataCrow.class.getClassLoader();
        if (loader == null) {
          location = ClassLoader.getSystemResource(classLocation);
        } else {
          location = loader.getResource(classLocation);
        }

        String dir = location.getFile();
        int index = dir.indexOf("_classes/net/sf/dc");
        if (index > -1)
            dir = dir.substring(0, index);

        index = dir.indexOf("_build/net/sf/dc");
        if (index > -1)
            dir = dir.substring(0, index);

        index = dir.indexOf("classes/net/sf/dc");
        if (index > -1)
            dir = dir.substring(0, index);

        index = dir.indexOf("datacrow.jar");
        if (index > -1)
            dir = dir.substring(0, index);

        if (dir.startsWith("file:"))
            dir = dir.substring(5, dir.length());

        dir = dir.replaceAll("%20", " ");
        baseDir = dir;
        imageDir = dir + "data" + '/' + "images" + '/';
    }
}
