/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.actions;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;

public abstract class DcAction implements Action {

    protected static final int _SEARCHTAB = 0;
    protected static final int _INSERTTAB = 1;
    protected static final int _NOTETAB = 2;

    private int module = -1;

    public DcAction() {
    }

    public DcAction(int module) {
        this.module = module;
    }

    public int getCurrentTab() {
    	return DataCrow.mainFrame.getSelectedTab();
    }

    public DcModule getModule() {
        return module == -1 ? DcModules.getCurrentModule() : DcModules.getModule(module);
    }

    public void changeModule(int moduleIndex) {
        int activeModule = getModule().getIndex();
        if (moduleIndex != activeModule) {
            DataCrow.mainFrame.changeModule(moduleIndex, true);
        }
    }

    public abstract void actionPerformed(ActionEvent ae);

    public boolean isEnabled() {
        return true;
    }

    public void setEnabled(boolean arg0) {}

    public void addPropertyChangeListener(PropertyChangeListener arg0) {
    }

    public void removePropertyChangeListener(PropertyChangeListener arg0) {
    }

    public Object getValue(String arg0) {
        return null;
    }

    public void putValue(String arg0, Object arg1) {
    }
}
