/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.audiocd;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.AudioCDQuickViewPane;
import net.sf.dc.console.audiocd.OnlineAudioCdSearchForm;
import net.sf.dc.console.components.panels.ParentInsertTablePanel;
import net.sf.dc.console.components.panels.ParentSearchTablePanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

public class AudioCDModule extends DcMediaModule {

    public AudioCDModule() {
        super(DcModules._AUDIOCD,
              true,
              DcResources.getText(Repository.systemlabels.sysAudioCD),
              "audiocds",
              "aucd",
              " audiocds aucd LEFT JOIN audiotracks autr ON (aucd.ID = autr.Album)",
              true,
              false,
              false,
              true);
    }
    
    public AudioCDModule( int index, boolean topModule, String name,
                          String tableName, String tableShortName, String tableJoin,
                          boolean canBeLended, boolean allowImdbUpdate, boolean allowAmazonUpdate, 
                          boolean allowFreeDbUpdate) {

        super(index, topModule, name, tableName, tableShortName,
             tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }     

    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoAudioCDModule16;
    }

    public String getObjectName() {
    	return DcResources.getText(Repository.systemlabels.sysAudioCD);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysAudioCDPlural);
    }

    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_4, InputEvent.CTRL_MASK);
    }

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));
        JMenuItem menuImport = ComponentFactory.getMenuItem(IconLibrary._icoImport, DcResources.getText("lblAudioCDImport"));
        menuImport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
        
        menuImport.addActionListener(new ImportCDAction());
        JMenuItem menuState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));  
        menuState.addActionListener(new ManageStatesAction(DcModules._AUDIOCD));
        
        JMenuItem menuTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblManageTemplates"));  
        menuTemplate.addActionListener(new ManageTemplatesAction(DcModules._AUDIOCD));
        
        menu.add(menuState);
        menu.addSeparator();
        menu.add(menuTemplate);
        menu.addSeparator();
        menu.add(menuImport);

    	return menu;
    }
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stAudioCDModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stAudioCDModuleEnabled, new Boolean(b));
    }

    public DcObject getDcObject() {
        return new AudioCD();
    }

    public DcCategoryModule getCategoryModule() {
        return null;
    }

    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.getModule(DcModules._AUDIOCDSTATE);
    }

    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.getModule(DcModules._TEMPLATEAUDIOCD);
    }
    
    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new ParentInsertTablePanel(DcModules._AUDIOCD));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this,
                DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new ParentSearchTablePanel(DcModules._AUDIOCD, AudioTrack._F_TRACKNUMBER));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioCD;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stAudioCDSearchOrder;
    }

    public void startImport() {
        OnlineAudioCdSearchForm importAudioCDDialog = new OnlineAudioCdSearchForm(true);
        importAudioCDDialog.setVisible(true);
    }

    public QuickViewPanel getQuickView() {
        return new AudioCDQuickViewPane();
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir, boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    } 

    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_AVAILABLE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_LOANEDBY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_LOANDURATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._B_DESCRIPTION, false, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._D_LOCATION , false, 1, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._L_STATE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._E_RATING , true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._F_ARTIST, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._G_GENRE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._H_PICTUREFRONT, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._I_PICTUREBACK , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._J_PICTURECD , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._K_DISCID , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._M_WEBPAGE , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_CREATED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_MODIFIED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U5_USER_NUMERIC2, false, 0, 1));
        return definitions;
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put(new Integer(DcObject._SYS_AVAILABLE), getField(DcObject._SYS_AVAILABLE));
        fields.put(new Integer(DcObject._SYS_LOANEDBY), getField(DcObject._SYS_LOANEDBY));
        fields.put(new Integer(DcObject._SYS_LOANDURATION), getField(DcObject._SYS_LOANDURATION));

        fields.put( new Integer(AudioCD._F_ARTIST),
                    new DcField(AudioCD._F_ARTIST, getIndex(), "Artist",
                                false, true, false, true, false,
                                4000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Artist"));
        fields.put( new Integer(AudioCD._G_GENRE),
                    new DcField(AudioCD._G_GENRE, getIndex(), "Genre",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Genre"));
        fields.put( new Integer(AudioCD._H_PICTUREFRONT),
                    new DcField(AudioCD._H_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(AudioCD._I_PICTUREBACK),
                    new DcField(AudioCD._I_PICTUREBACK, getIndex(), "Picture Back",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureBack"));
        fields.put( new Integer(AudioCD._J_PICTURECD),
                    new DcField(AudioCD._J_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(AudioCD._K_DISCID),
                    new DcField(AudioCD._K_DISCID, getIndex(), "Disc ID",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "DiscID"));
        fields.put( new Integer(AudioCD._L_STATE),
                    new DcField(AudioCD._L_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._AUDIOCDSTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(AudioCD._M_WEBPAGE),
                    new DcField(AudioCD._M_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));   
        
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }

    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}
