/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.generic;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.KeyStroke;

import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.generic.GenericQuickViewPane;
import net.sf.dc.console.generic.SearchGenericObjectPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

public class GenericModule extends DcMediaModule {

    /**
     * Initializes this module
     */
    public GenericModule() {
        super(DcModules._GENERIC,
              true,
              DcResources.getText(Repository.systemlabels.sysGenericModule),
              "",
              "",
              "",
              true,
              false,
              false,
              false);
    }

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysGenericModule);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysGenericModule);
    }
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_8, InputEvent.CTRL_MASK);
    }    
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stGenericModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stGenericModuleEnabled, new Boolean(b));
    }     
    
    public ITemplateModule getTemplateModule() {
        return null;
    }       
    
    public DcStateModule getStateModule() {
        return null;
    } 

    public DcCategoryModule getCategoryModule() {
        return null;
    }
    
    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }    

    public ImageIcon getIcon() {
        return IconLibrary._icoGenericModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoGenericModule16;
    }
    
    public JMenu getToolsMenu() {
        return null;
    }

    public DcObject getDcObject() {
        return new GenericObject();
    }

    public void startImport() {
    }

    public QuickViewPanel getQuickView() {
        return new GenericQuickViewPane();
    }

    protected void initializeUI() {
        insertView = null;
        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new SearchGenericObjectPanel());
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getName(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsGeneric;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stGenericSearchOrder;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir, boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    }  

    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();

        definitions.addDefinition(new ReportFieldDefinition(GenericObject._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_AVAILABLE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_LOANEDBY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_LOANDURATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._B_DESCRIPTION, true, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._D_LOCATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._E_RATING, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._SYS_MODULE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_CREATED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_MODIFIED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U5_USER_NUMERIC2, false, 0, 1));
        return definitions;
    }

    protected void initializeFields() {
        super.initializeFields();
        fields.put(new Integer(DcObject._SYS_AVAILABLE), getField(DcObject._SYS_AVAILABLE));
        fields.put(new Integer(DcObject._SYS_LOANEDBY), getField(DcObject._SYS_LOANEDBY));
        fields.put(new Integer(DcObject._SYS_LOANDURATION), getField(DcObject._SYS_LOANDURATION));
        fields.put(new Integer(DcObject._SYS_MODULE), getField(DcObject._SYS_MODULE));
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
}
