package net.sf.dc.core.modules.musicfile;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.ParentInsertTablePanel;
import net.sf.dc.console.components.panels.ParentSearchTablePanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.importcd.ImportMusicFilesDialog;
import net.sf.dc.console.musicfile.MusicAlbumQuickViewPane;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageStorageMediaAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.actions.OpenMusicFileSynchronizer;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.musicfiles.MusicAlbum;
import net.sf.dc.core.objects.musicfiles.MusicTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMusicFilesCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

public class MusicAlbumModule extends DcMediaModule {

    public MusicAlbumModule() {
        super(DcModules._MUSICALBUM,
              true,
              DcResources.getText(Repository.systemlabels.sysMusicAlbum),
              "musicalbums",
              "musa",
              "musicalbums",
              false,
              false,
              false,
              false);
    }
    
    public MusicAlbumModule(int index, 
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean canBeLended,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        super(  index, topModule, name, tableName, tableShortName,
                tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }     

    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoMusicModule16;
    }    

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMusicAlbum);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysMusicAlbumPlural);
    }

    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_2, InputEvent.CTRL_MASK);
    }    
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stMusicAlbumModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stMusicAlbumModuleEnabled, new Boolean(b));
    }  
    
    public DcCategoryModule getCategoryModule() {
        return null;
    }
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.getModule(DcModules._MUSICALBUMSTATE);
    }     
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.getModule(DcModules._TEMPLATEMUSICALBUM);
    }     
    
    public DcStorageMediaModule getStorageMediaModule() {
        return (DcStorageMediaModule) DcModules.getModule(DcModules._MUSICALBUMSTORAGE);
    }    

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));

        JMenuItem menuImport = ComponentFactory.getMenuItem(IconLibrary._icoImport, DcResources.getText("lblMusicFileImport"));
        menuImport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
        menuImport.addActionListener(new ImportCDAction());

        JMenuItem menuAlbumState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, 
        							DcResources.getText("lblManageStatesX", getObjectName()));  
        menuAlbumState.addActionListener(new ManageStatesAction(DcModules._MUSICALBUM));

        JMenuItem menuTrackState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, 
        							DcResources.getText("lblManageStatesX", DcModules.getModule(DcModules._MUSICTRACK).getObjectName()));  
        menuTrackState.addActionListener(new ManageStatesAction(DcModules._MUSICTRACK));

        
        JMenuItem menuStorageMedia = ComponentFactory.getMenuItem(IconLibrary._icoSave, 
        			DcResources.getText("lblManageStorageMedia"));  
        menuStorageMedia.addActionListener(new ManageStorageMediaAction());
        
        JMenuItem menuAlbumTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, 
        			DcResources.getText("lblManageTemplatesX", getObjectName()));  
        menuAlbumTemplate.addActionListener(new ManageTemplatesAction(DcModules._MUSICALBUM));

        JMenuItem menuTrackTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate,
        			DcResources.getText("lblManageTemplatesX", DcModules.getModule(DcModules._MUSICTRACK).getObjectName()));  
        menuTrackTemplate.addActionListener(new ManageTemplatesAction(DcModules._MUSICTRACK));

        JMenuItem menuMusicSynchronizer = ComponentFactory.getMenuItem(
                    IconLibrary._icoMusicFileSynchronizer,
    				DcResources.getText("lblMusicSynchronizer"));  
        menuMusicSynchronizer.addActionListener(new OpenMusicFileSynchronizer());
        
        menu.add(menuAlbumState);
        menu.add(menuTrackState);
        menu.add(menuStorageMedia);
        menu.addSeparator();
        menu.add(menuAlbumTemplate);
        menu.add(menuTrackTemplate);
        menu.addSeparator();
        menu.add(menuImport);
        menu.add(menuMusicSynchronizer);

        return menu;
    }

    public QuickViewPanel getQuickView() {
        return new MusicAlbumQuickViewPane();
    }

    public DcObject getDcObject() {
        return new MusicAlbum();
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new ParentInsertTablePanel(DcModules._MUSICALBUM));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new ParentSearchTablePanel(DcModules._MUSICALBUM, MusicTrack._F_TRACKNUMBER));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMusicAlbum;
    }
    
    public String getSearchOrderSettingsKey() {
        return Repository.settings.stMusicAlbumSearchOrder;
    }

    public void startImport() {
        ImportMusicFilesDialog dlg = new ImportMusicFilesDialog(getIndex(), DataCrow.mainFrame, DcResources.getText("lblMusicFileImport"));
        dlg.setVisible(true);
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              DcStorageMedium storageMedium,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return new ImportMusicFilesCD(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, nameOfCD, 0);
    }

    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._B_DESCRIPTION, false, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._F_ARTIST, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._G_GENRE, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._H_STATE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._I_STORAGEMEDIUM, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._D_LOCATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._E_RATING, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._J_PICTUREFRONT, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._K_PICTUREBACK, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._L_PICTURECD, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._M_DISCID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._N_WEBPAGE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_CREATED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_MODIFIED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicAlbum._U5_USER_NUMERIC2, false, 0, 1));

        return definitions;
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(MusicAlbum._F_ARTIST),
                    new DcField(MusicAlbum._F_ARTIST, getIndex(), "Artist",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Artist"));
        fields.put( new Integer(MusicAlbum._G_GENRE),
                    new DcField(MusicAlbum._G_GENRE, getIndex(), "Genre",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Genre"));
        fields.put( new Integer(MusicAlbum._H_STATE),
                    new DcField(MusicAlbum._H_STATE, getIndex(), "State",
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._MUSICALBUMSTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(MusicAlbum._I_STORAGEMEDIUM),
                    new DcField(MusicAlbum._I_STORAGEMEDIUM, getIndex(), "Storage Medium",  
                                false, true, false, true, false, 
                                50, ComponentFactory._MEDIACOMBO, DcModules._MUSICALBUMSTORAGE, Repository.datatypes._DCOBJECTREFERENCE,
                                "StorageMedium"));
        fields.put( new Integer(MusicAlbum._J_PICTUREFRONT),
                    new DcField(MusicAlbum._J_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(MusicAlbum._K_PICTUREBACK),
                    new DcField(MusicAlbum._K_PICTUREBACK, getIndex(), "Picture Back",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureBack"));
        fields.put( new Integer(MusicAlbum._L_PICTURECD),
                    new DcField(MusicAlbum._L_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(MusicAlbum._M_DISCID),
                    new DcField(MusicAlbum._M_DISCID, getIndex(), "Disc ID",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "DiscID"));
        fields.put( new Integer(MusicAlbum._N_WEBPAGE),
                    new DcField(MusicAlbum._N_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));   
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }

    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}