package net.sf.dc.core.modules.musicfile;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.ChildInsertTablePanel;
import net.sf.dc.console.components.panels.ChildSearchTablePanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.musicfile.MusicTrackForm;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.IChildModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.musicfiles.MusicAlbum;
import net.sf.dc.core.objects.musicfiles.MusicTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

public class MusicTrackModule extends DcMediaModule implements IChildModule {

    public MusicTrackModule() {
        super(DcModules._MUSICTRACK,
              false,
              DcResources.getText(Repository.systemlabels.sysMusicTrack),
              "musictracks",
              "mutr",
              "musictracks",
              false,
              false,
              false,
              false);
    }
    
    public MusicTrackModule(int index, 
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean canBeLended,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        super(  index, topModule, name, tableName, tableShortName,
                tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }     

    public DcSimpleItemView getItemView(ItemForm form, DcObject parent, int module, boolean readonly) {
        return new MusicTrackForm(form, (MusicAlbum) parent, module, readonly);
    }    
    
    public int getSortIdx() {
        return MusicTrack._F_TRACKNUMBER;
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoMusicModule16;
    }    

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMusicTrack);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysMusicTrackPlural);
    }

    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_2, InputEvent.CTRL_MASK);
    }    
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stMusicAlbumModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stMusicAlbumModuleEnabled, new Boolean(b));
    }  
    
    public DcCategoryModule getCategoryModule() {
        return null;
    }
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.getModule(DcModules._MUSICTRACKSTATE);
    }     
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.getModule(DcModules._TEMPLATEMUSICTRACK);
    }     
    
    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }    

    public JMenu getToolsMenu() {
        return null;
    }

    public QuickViewPanel getQuickView() {
        return null;
    }

    public DcObject getDcObject() {
        return new MusicTrack();
    }

    public void startImport() {
        
    }
    
    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new ChildInsertTablePanel(DcModules._MUSICTRACK));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW,
                           new ChildSearchTablePanel(DcModules._MUSICTRACK, DcModules._MUSICALBUM));    
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMusicTrack;
    }
    
    public String getSearchOrderSettingsKey() {
        return null;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              DcStorageMedium storageMedium,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return null;
    }

    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._ID, false, 0, 1));
        
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._F_TRACKNUMBER, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._A_TITLE, true, 0, 5));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._J_PLAYLENGTH, true, 0, 1));
        
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._B_DESCRIPTION, false, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._G_ARTIST, false, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._P_ALBUM, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._C_YEAR, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._H_GENRE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._I_FILENAME, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._K_QUALITY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._L_ENCODING, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._M_LYRIC, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._N_FILESIZE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._D_LOCATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._O_STATE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._E_RATING, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicTrack._U5_USER_NUMERIC2, false, 0, 1));

        return definitions;
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(MusicTrack._F_TRACKNUMBER),
                	new DcField(MusicTrack._F_TRACKNUMBER, getIndex(), "Track",
                				false, true, false, false, false,
                				255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                            	"Track"));        
        fields.put( new Integer(MusicTrack._G_ARTIST),
                    new DcField(MusicTrack._G_ARTIST, getIndex(), "Artist",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Artist"));
        fields.put( new Integer(MusicTrack._H_GENRE),
                    new DcField(MusicTrack._H_GENRE, getIndex(), "Genre",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Genre"));
        fields.put( new Integer(MusicTrack._I_FILENAME),
                    new DcField(MusicTrack._I_FILENAME, getIndex(), "Filename",
                                false, true, false, true, false,
                                255, ComponentFactory._FILELAUNCHFIELD, getIndex(), Repository.datatypes._STRING,
                                "Filename"));
        fields.put( new Integer(MusicTrack._J_PLAYLENGTH),
                    new DcField(MusicTrack._J_PLAYLENGTH, getIndex(), "Playlength",
                                false, true, false, false, false,
                                20, ComponentFactory._TIMEFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Length"));
        fields.put( new Integer(MusicTrack._K_QUALITY),
                    new DcField(MusicTrack._K_QUALITY, getIndex(), "Quality",
                                false, true, false, true, true,
                                4, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Quality"));
        fields.put( new Integer(MusicTrack._L_ENCODING),
                    new DcField(MusicTrack._L_ENCODING, getIndex(), "Encoding",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Encoding"));
        fields.put( new Integer(MusicTrack._M_LYRIC),
                    new DcField(MusicTrack._M_LYRIC, getIndex(), "Lyric",
                                false, true, false, true, false,
                                8000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Lyric"));
        fields.put( new Integer(MusicTrack._N_FILESIZE),
                    new DcField(MusicTrack._N_FILESIZE, getIndex(), "File Size",
                                false, true, false, true, true,
                                15, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Filesize"));
        fields.put( new Integer(MusicTrack._O_STATE),
                    new DcField(MusicTrack._O_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._MUSICTRACKSTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(MusicTrack._P_ALBUM),
	                new DcField(MusicTrack._P_ALBUM, getIndex(), "Album",
	                            false, true, true, false, false,
	                            255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._DCPARENTREFERENCE,
	                            "Album"));
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
}