/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.software;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.CategoryForm;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.software.SoftwareQuickViewPane;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManagePlatformsAction;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageStorageMediaAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportSoftwareCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.util.Utilities;

public class SoftwareModule extends DcMediaModule {

    public SoftwareModule() {
        super(DcModules._SOFTWARE,
              true,
              DcResources.getText(Repository.systemlabels.sysSoftware),
              "software",
              "soft",
              "software",
              true,
              false,
              true,
              false);
    }

    public SoftwareModule( int index, 
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean canBeLended,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        super(index, topModule, name, tableName, tableShortName,
                tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }    
    
    public ImageIcon getIcon() {
        return IconLibrary._icoSoftwareModule;
    }

    public ImageIcon getIconSmall() {
        return IconLibrary._icoSoftwareModule16;
    }        
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_1, InputEvent.CTRL_MASK);
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysSoftware);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysSoftwarePlural);
    }

    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stSoftwareModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stSoftwareModuleEnabled, new Boolean(b));
    }      
    
    public DcCategoryModule getCategoryModule() {
        return (DcCategoryModule) DcModules.getModule(DcModules._SOFTWARECAT);
    }
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.getModule(DcModules._TEMPLATESOFTWARE);
    }       
    
    public DcStorageMediaModule getStorageMediaModule() {
        return (DcStorageMediaModule) DcModules.getModule(DcModules._SOFTWARESTORAGE);
    }    
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.getModule(DcModules._SOFTWARESTATE);
    } 

    public QuickViewPanel getQuickView() {
        return new SoftwareQuickViewPane();
    }

    public void startImport() {
        ImportCDDialog importCDDialog = new ImportCDDialog(getIndex(), DataCrow.mainFrame, DcResources.getText("lblSoftwareImport"));
        importCDDialog.setVisible(true);
    }

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));

        JMenuItem menuCat = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageCategories"));
        menuCat.addActionListener(new CatagoryFormMenuAction());

        JMenuItem menuState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));  
        menuState.addActionListener(new ManageStatesAction(DcModules._SOFTWARE));

        JMenuItem menuPlatform = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManagePlatforms"));  
        menuPlatform.addActionListener(new ManagePlatformsAction());

        JMenuItem menuStorageMedia = ComponentFactory.getMenuItem(IconLibrary._icoSave, DcResources.getText("lblManageStorageMedia"));  
        menuStorageMedia.addActionListener(new ManageStorageMediaAction());

        JMenuItem menuTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblManageTemplates"));  
        menuTemplate.addActionListener(new ManageTemplatesAction(DcModules._SOFTWARE));
        
        JMenuItem menuImport = ComponentFactory.getMenuItem(IconLibrary._icoImport, DcResources.getText("lblSoftwareImport"));
        menuImport.addActionListener(new ImportCDAction());
        menuImport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));

        menu.add(menuCat);
        menu.add(menuState);
        menu.add(menuPlatform);
        menu.add(menuStorageMedia);
        menu.addSeparator();
        menu.add(menuTemplate);
        menu.addSeparator();
        menu.add(menuImport);

        return menu;
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new TableInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new TableSearchView(this, getObjectNamePlural()));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsSoftware;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stSoftwareSearchOrder;
    }

    public DcObject getDcObject() {
        return new Software();
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              DcStorageMedium storageMedium,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return new ImportSoftwareCD(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }

    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(Software._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_AVAILABLE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_LOANEDBY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_LOANDURATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._A_TITLE, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._B_DESCRIPTION, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Software._F_DEVELOPER, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._G_PUBLISHER, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._H_PLATFORM, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._W_STORAGEMEDIUM, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._I_WEBPAGE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._J_FILENAME, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._K_TYPE, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._L_MULTI, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._M_PICTUREFRONT, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._N_PICTUREBACK, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._O_PICTURECD, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._P_SCREENSHOTONE, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._Q_SCREENSHOTTWO, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._R_SCREENSHOTTHREE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._S_SERIALKEY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._T_FILESIZE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._D_LOCATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._V_STATE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._E_RATING, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._U_ASIN, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_CREATED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(DcObject._SYS_MODIFIED, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Software._U5_USER_NUMERIC2, false, 0, 1));
        return definitions;
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put(new Integer(DcObject._SYS_AVAILABLE), getField(DcObject._SYS_AVAILABLE));
        fields.put(new Integer(DcObject._SYS_LOANEDBY), getField(DcObject._SYS_LOANEDBY));
        fields.put(new Integer(DcObject._SYS_LOANDURATION), getField(DcObject._SYS_LOANDURATION));

        fields.put( new Integer(Software._F_DEVELOPER),
                    new DcField(Software._F_DEVELOPER, getIndex(), "Developer",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Developer"));
        fields.put( new Integer(Software._G_PUBLISHER),
                    new DcField(Software._G_PUBLISHER, getIndex(), "Publisher",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Publisher"));
        fields.put( new Integer(Software._H_PLATFORM),
                    new DcField(Software._H_PLATFORM, getIndex(), "Platform",
                                false, true, false, true, false,
                                255, ComponentFactory._PLATFORMCOMBO, DcModules._SOFTWAREPLATFORM, Repository.datatypes._DCOBJECTREFERENCE,
                                "Platform"));
        fields.put( new Integer(Software._I_WEBPAGE),
                    new DcField(Software._I_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));
        fields.put( new Integer(Software._J_FILENAME),
                    new DcField(Software._J_FILENAME, getIndex(), "Filename",
                                false, true, false, true, false,
                                255, ComponentFactory._FILELAUNCHFIELD, getIndex(), Repository.datatypes._STRING,
                                "Filename"));
        fields.put( new Integer(Software._K_TYPE),
                    new DcField(Software._K_TYPE, getIndex(), "Category",
                                false, true, false, true, false,
                                255, ComponentFactory._SOFTWARECATEGORYCOMBO, DcModules._SOFTWARECAT, Repository.datatypes._DCOBJECTREFERENCE,
                                "Type"));
        fields.put( new Integer(Software._L_MULTI),
                    new DcField(Software._L_MULTI, getIndex(), "Multiplayer ?",
                                false, true, false, true, false,
                                4, ComponentFactory._YESNOCOMBO, getIndex(), Repository.datatypes._STRING,
                                "Multiplayer"));
        fields.put( new Integer(Software._M_PICTUREFRONT),
                    new DcField(Software._M_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(Software._N_PICTUREBACK),
                    new DcField(Software._N_PICTUREBACK, getIndex(), "Picture Back",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureBack"));
        fields.put( new Integer(Software._O_PICTURECD),
                    new DcField(Software._O_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(Software._P_SCREENSHOTONE),
                    new DcField(Software._P_SCREENSHOTONE, getIndex(), "Screenshot One",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "ScreenshotOne"));
        fields.put( new Integer(Software._Q_SCREENSHOTTWO),
                    new DcField(Software._Q_SCREENSHOTTWO, getIndex(), "Screenshot Two",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "ScreenshotTwo"));
        fields.put( new Integer(Software._R_SCREENSHOTTHREE),
                    new DcField(Software._R_SCREENSHOTTHREE, getIndex(), "Screenshot Three",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "ScreenshotThree"));
        fields.put( new Integer(Software._S_SERIALKEY),
                    new DcField(Software._S_SERIALKEY, getIndex(), "Serial Key",
                                false, true, false, false, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "SerialKey"));
        fields.put( new Integer(Software._T_FILESIZE),
                    new DcField(Software._T_FILESIZE, getIndex(), "Filesize",
                                false, true, false, true, true,
                                255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Filesize"));
        fields.put( new Integer(Software._U_ASIN),
                    new DcField(Software._U_ASIN, getIndex(), "ASIN",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "ASIN"));
        fields.put( new Integer(Software._V_STATE),
                    new DcField(Software._V_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._SOFTWARESTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(Software._W_STORAGEMEDIUM),
                    new DcField(Software._W_STORAGEMEDIUM, getIndex(), "Storage Medium",  
                                false, true, false, true, false, 
                                50, ComponentFactory._MEDIACOMBO, DcModules._SOFTWARESTORAGE, Repository.datatypes._DCOBJECTREFERENCE,
                                "StorageMedium"));      
    }

    public DcObject[] getDefaultData() throws Exception {
        DcObject[] objects = new DcObject[1];

        Software software = new Software();
        software.markAsUnchanged();

        software.setValue(Software._A_TITLE, Repository.system.version);
        software.setValue(Software._B_DESCRIPTION, "There are many programs that claim to have a nice database to put your music in, and there are other programs that allow you to store your audio CD information, but Data Crow has it all under one roof!\n\n" +
                                                   "Data Crow helps you organize your software, music, book, and movie collections by using online databases from Amazon, IMDb, and freedb, as well as the tag information stored within your files.\n\n" +
                                                   "We think you will find Data Crow easy to use and a great all-rounder!");
        software.setValue(Software._E_RATING, new Integer(4));
        software.setValue(Software._C_YEAR, new Integer(2006));
        software.setValue(Software._I_WEBPAGE, "http://www.datacrow.net");
        software.setValue(Software._J_FILENAME, "datacrow.zip");
        software.setValue(Software._S_SERIALKEY, "xxxx-xxxx-xxxx-xxxx");
        software.setValue(Software._D_LOCATION, "CD232");
        software.setValue(Software._T_FILESIZE, new Integer(4500));
        software.setValue(Software._U1_USER_LONGTEXT, "");
        software.setValue(Software._U2_USER_SHORTTEXT1, "");
        software.setValue(Software._U3_USER_SHORTTEXT2, "");
        software.setValue(Software._U4_USER_NUMERIC1, new Integer(0));
        software.setValue(Software._U5_USER_NUMERIC2, new Integer(0));

        try {
            byte[] b = Utilities.readFile(new File(DataCrow.baseDir + "icons_system/aboutinformation.jpg"));
            software.setValue(Software._M_PICTUREFRONT, b);
            
            Picture p = (Picture) software.getValue(Software._M_PICTUREFRONT);
            p.setValue(Picture._D_IMAGE, b);
        } catch (Exception ignore) {} 

        objects[0] = software;

        return objects;
    }

    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            SoftwareCategoryModule softCatMod =
                (SoftwareCategoryModule) DcModules.getModule(DcModules._SOFTWARECAT);
            CategoryForm categoryForm = softCatMod.getForm();
            categoryForm.setVisible(true);
        }
    }

    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}


