/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.io.File;

import javax.swing.ImageIcon;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.messages.Messages;
import net.sf.ext.util.Utilities;

public class Picture extends DcObject {

    public static final int _A_OBJECTID = 1;
    public static final int _B_FIELD = 2;
    public static final int _C_FILENAME = 3;
    public static final int _D_IMAGE = 4;
    public static final int _E_HEIGHT = 5;
    public static final int _F_WIDTH = 6;

    private boolean isNew = false;
    private boolean isUpdated = false;
    private boolean isDeleted = false;
    private boolean isLoaded = false;
    
    public Picture() {
        super();
    }
    
    public void loadImage() {
        if (!isLoaded) {
            String filename = (String) getValue(_C_FILENAME);

            try {
                byte[] image = Utilities.readFile(new File(DataCrow.imageDir + filename));
                setValue(Picture._D_IMAGE, image);
                isLoaded(true);
            } catch (Exception exp) {
                Messages.add(exp);
            }
            
            markAsUnchanged();
        }
    }
    
    public ImageIcon getScaledPicture() {
        ImageIcon scaledInstance = null;

        if (getScaledFilename() != null) {
            File file = new File(DataCrow.imageDir + getScaledFilename());
            
            if (file.exists() && file.canRead())
                scaledInstance = new ImageIcon(DataCrow.imageDir + getScaledFilename());
        }
        
        return scaledInstance;
    }
    
    public String getScaledFilename() {
        return getScaledFilename((String) getValue(Picture._C_FILENAME));
    }

    public String getScaledFilename(String filename) {
        if (filename != null) {
            try {
                int idx = filename.indexOf(".jpg");
                String plain = filename.substring(0, idx);
                return plain + "_small.jpg";
            } catch (Exception exp) {}
        }
        return null;
    }
    
    public void isLoaded(boolean b) {
        isLoaded = b;
        isNew(false);
        isUpdated(false);
        isDeleted(false);        
    }
    
    public void isUpdated(boolean b) {
        isUpdated = b;
        if (b) {
            isNew = false;
            isDeleted = false;
            isLoaded = true;

        }
    }
    
    public void isNew(boolean b) {
        isNew = b;

        if (b) {
            isUpdated = false;
            isDeleted = false;
            isLoaded = true;
        }
    }
    
    public void isDeleted(boolean b) {
        isDeleted = b;
        
        if (b) {
            isUpdated = false;
            isNew = false;
            isLoaded = true;
        }
    }
    
    public boolean isLoaded() {
        return isLoaded;
    }
    
    public boolean isUpdated() {
        return isUpdated;
    }
    
    public boolean isNew() {
        return isNew;
    }
    
    public boolean isDeleted() {
        return isDeleted;
    }
    
    protected void setModuleIndex() {
        module = DcModules._PICTURE;
    }     

    public ImageIcon getIcon() {
        return null;
    } 
    
    public String getFilename() {
        return null;
    }
    
    public boolean equals(Object o) {
        boolean equals = false;
        
        if (o instanceof Picture) {
            Picture picture = (Picture) o;
            
            String fieldname1 = (String) picture.getValue(Picture._B_FIELD);
            String fieldname2 = (String) getValue(Picture._B_FIELD);

            String objectID1 = (String) picture.getValue(Picture._A_OBJECTID);
            String objectID2 = (String) getValue(Picture._A_OBJECTID);
            
            equals = fieldname1.equals(fieldname2) && objectID1.equals(objectID2);
        } else {
            equals = super.equals(o);
        }
        
        return equals;
   }    
}