/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.book;

import javax.swing.ImageIcon;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.messages.Messages;

public class Book extends DcMediaObject {

    public static final int _F_PUBLISHER  = 6;
    public static final int _G_AUTHOR = 7;
    public static final int _H_WEBPAGE = 8;
    public static final int _I_CATEGORY = 9;
    public static final int _J_ISBN10 = 10;
    public static final int _K_PICTUREFRONT = 11;
    public static final int _L_STATE = 12;
    public static final int _M_FILENAME = 13;
    public static final int _N_ISBN13 = 14;

    public Book() {
       super();
    }

    protected void setModuleIndex() {
        module = DcModules._BOOK;
    }

    public String getFilename() {
    	return (String) getValue(_M_FILENAME);
    }

    public ImageIcon getIcon() {
        return null;
    }

    protected void beforeSave() {
        String s10 = (String) getValue(_J_ISBN10);
        String s13 = (String) getValue(_N_ISBN13);

        try {
            Isbn isbn = new Isbn();

            boolean isBarcode13 = isbn.isIsbn13(s13);
            boolean isBarcode10 = isbn.isIsbn10(s10);
            if (isBarcode10 && !isBarcode13) {
                String isbn13 = isbn.getIsbn13(s10);  
                setValue(_N_ISBN13, isbn13);
            } else if (isBarcode13 && !isBarcode10) {
                String isbn10 = isbn.getIsbn10(s13);  
                setValue(_J_ISBN10, isbn10);
            }
        } catch (InvalidBarCodeException ibce) {
            Messages.add(ibce);
        }
    }
    
    public String toString() {
        return getValue(Book._A_TITLE) != null ? getValue(Book._A_TITLE).toString() : "";
    }
}

