package net.sf.dc.core.objects.enhancement;

import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;

public class AutoIncrementer implements IValueEnhancer {

    private boolean enabled = false;
    private boolean fillGaps = false;
    private int step = 1;
    private int field;

    public AutoIncrementer(int field) {
        this.field = field;
    }

    public AutoIncrementer(int field, boolean enabled, boolean fillGaps, int step) {
        this.field = field;
        this.enabled = enabled;
        this.fillGaps = fillGaps;
        this.step = step;
    }
    
    public String toSaveString() {
        return enabled + "/&/" + fillGaps + "/&/" + step;
    }

    public boolean isEnabled() {
        return enabled;
    }
    
    public boolean isFillGaps() {
        return fillGaps;
    }
    
    public int getStep() {
        return step;
    }
    
    public boolean isRunOnUpdating() {
        return false;
    }
    
    public boolean isRunOnInsert() {
        return true;
    }
    
    public Object apply(DcField field, Object value) {
        Object result = value;
        
        DcModule module = DcModules.getModule(field.getModule());
        int counter = 0;
        
        try {
            if (!fillGaps) {
                String query = "SELECT MAX(" + field.getDatabaseFieldName() + ") AS COUNTMAXIMUM FROM " +
                               module.getTableName();
                ResultSet rs = DatabaseManager.runQueryDirectUnclosed(query, true);
                int maximum = 0;
                while (rs.next()) {
                    maximum = rs.getInt("COUNTMAXIMUM");
                }
                
                result = new Integer(maximum + getStep());
                rs.close();
            } else {
                counter = counter + getStep();
                
                String qryCurrent = "SELECT " + field.getDatabaseFieldName() + " FROM " + module.getTableName() + 
                " WHERE " + field.getDatabaseFieldName() + " IS NOT NULL AND " +
                field.getDatabaseFieldName() + " > 0 " +
                "ORDER BY 1";            
                
                Collection currentValues = new ArrayList();
                ResultSet rs = DatabaseManager.runQueryDirectUnclosed(qryCurrent, true);
                while (rs.next()) {
                    currentValues.add(new Integer(rs.getInt(field.getDatabaseFieldName())));
                }
                
                if (currentValues.contains(new Integer(counter))) {
                    boolean currentfound = false;
                    for (Iterator iter = currentValues.iterator(); iter.hasNext(); ) {
                        int x = ((Integer) iter.next()).intValue();
                        
                        while (!currentfound && x == counter) {
                            counter += getStep();
                        }
                    }
                }            
                result = new Integer(counter);
            }
        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox(DcResources.getText("msgAutoNumberError", 
                           new String[] {field.getLabel(), exp.getMessage()}), 
                           MessageBox._ERROR);
        }
            
        return result;
    }
    
    public int getField() {
        return field;
    }

    public void parse(String s) {
        String tmp = s;
        String s1 = tmp.substring(0, tmp.indexOf("/&/"));
        tmp = tmp.substring(tmp.indexOf("/&/") + 3, tmp.length());
        
        String s2 = tmp.substring(0, tmp.indexOf("/&/"));
        tmp = tmp.substring(tmp.indexOf("/&/") + 3, tmp.length());
        
        String s3 = tmp;
        
        enabled = new Boolean(s1).booleanValue();
        fillGaps = new Boolean(s2).booleanValue();
        step = new Integer(s3).intValue();
    }

    public int getIndex() {
        return ValueEnhancers._AUTOINCREMENT;
    }
}
