/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.io.File;
import java.io.FilenameFilter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.PictureModule;
import net.sf.dc.core.modules.generic.GenericModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcState;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.musicfiles.MusicAlbum;
import net.sf.dc.core.objects.musicfiles.MusicAlbumStorageMedium;
import net.sf.dc.core.objects.musicfiles.MusicTrack;
import net.sf.dc.core.objects.musicfiles.MusicTrackState;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.Utilities;

public class DatabaseConversion {
    
    public boolean start() {

    	boolean convertedMusicFiles = converMusicFiles();
        boolean convertedPictures = convertPictures();
        boolean platformsConverted = convertSoftwarePlatforms();
        boolean convertTableTypes = convertTableTypes();
        createThumbnails();
        
        if ((platformsConverted) && (!convertTableTypes && !convertedPictures && !convertedMusicFiles)) {
            new MessageBox("All Done! Starting Data Crow)", MessageBox._INFORMATION);
            MessagesWriter._windowShow = false;
            MessagesWriter._windowToFront = false;
        } else if (convertTableTypes || convertedPictures || convertedMusicFiles) {
            try {
                DataCrow.closeSplashScreen();
                new MessageBox("All Done! Compacting database and shutting down (this can take several minutes and will use a lot of resources)", MessageBox._INFORMATION);
                Messages.add("Rewriting database and shutting down", Messages._STANDARD);
                DatabaseManager.closeDatabases(true);
                System.exit(1);
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }
        return false;
    }    
    
    public void createThumbnails() {
        File imageDir = new File(DataCrow.imageDir);
        
        String[] imageFiles = imageDir.list();
        String[] thumbFiles = imageDir.list(new ThumbnailFileFilter());
        
        Picture picture = new Picture();
        if (thumbFiles.length == 0 && imageFiles.length > 0) {
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;

            new MessageBox("Data Crow will create smaller versions of the alongside the current " +
                           "available images. This can take several minutes and is only needed once", 
                           MessageBox._INFORMATION);
            
            DataCrow.closeSplashScreen();
            
            Messages.add("Creating thumbnails", Messages._STANDARD);
            
            for (int i = 0; i < imageFiles.length; i++) {
                try {
                    byte[] image = Utilities.readFile(new File(DataCrow.imageDir + imageFiles[i]));
                    String target = picture.getScaledFilename(DataCrow.imageDir + imageFiles[i]);
                    
                    Utilities.writeScaledImageToFile(image, target);
                    Messages.add("Created " + target, Messages._STANDARD);
                } catch (Exception exp) {
                    Messages.add(exp);
                }
            }
            Messages.add("Smaller versions of all images have been created successfully", Messages._STANDARD);
        } 
    }
    
    private class ThumbnailFileFilter implements FilenameFilter {

        public ThumbnailFileFilter() {
            super();
        }
        
        public boolean accept(File file, String filename) {
            return filename.toLowerCase().indexOf("small") > -1;
        }
    }    
    
    
    public boolean converMusicFiles () {
    	boolean hasBeenConverted = false;
        Connection connection = DatabaseManager.getConnection();
        
        try {
            Statement stmt = connection.createStatement();
            
            int counter = 0;
            try {
            	ResultSet rs = stmt.executeQuery("SELECT TOP 1 album from musicfiles;");
	            while (rs.next())
	                counter++;
	            
	            try {
	                rs.close();
	            } catch (Exception ignore) {}
	            
            } catch (Exception exp) {
            	return false;
            }
            
            if (counter > 1) {
                return false;
            }
        
            new QuestionBox("The Music Files needs to be converted. This version reflect albums and tracks. Do you want to continue ?");

            if (Boolean.FALSE.equals(QuestionBox.result))
                System.exit(0);
        
            new DataManager();
            
            DataCrow.closeSplashScreen();
            
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;

            // create the states and storage media first
            
            // remove auto-created states
            DcObject[] cachedStates =  DataManager.get(DcModules._MUSICTRACKSTATE, null);
            for (int i = 0; i < cachedStates.length; i++) {
                cachedStates[i].setSilence(true);
                cachedStates[i].delete();
            }

            DcObject[] cachedMedia =  DataManager.get(DcModules._MUSICALBUMSTORAGE, null);
            for (int i = 0; i < cachedMedia.length; i++) {
                cachedMedia[i].setSilence(true);
                cachedMedia[i].delete();
            }

            // wait for all the states and media to become deleted and synchronized with the datamanager
            while (DatabaseManager.getQueueSize() > 0) {}
            
            ResultSet rsMedia = stmt.executeQuery("SELECT * FROM musicfilestoragemedia;");
            Collection media = new ArrayList();
            while (rsMedia.next()) {
                String icon = rsMedia.getString("icon");
                String name = rsMedia.getString("name");
                String id = rsMedia.getString("id");
                
                MusicAlbumStorageMedium mtsm = new MusicAlbumStorageMedium();
                mtsm.setValue(DcStorageMedium._A_NAME, name);
                mtsm.setValue(DcStorageMedium._B_ICON, icon);
                mtsm.setValue(DcState._ID, id);
                mtsm.saveNew(false);

                media.add(mtsm);
            }
            
            ResultSet rsStates = stmt.executeQuery("SELECT * FROM musicfilestates;");
            Collection states = new ArrayList();
            while (rsStates.next()) {
                String icon = rsStates.getString("icon");
                String name = rsStates.getString("name");
                String id = rsStates.getString("id");
                
                MusicTrackState mts = new MusicTrackState();
                mts.setValue(DcState._A_NAME, name);
                mts.setValue(DcState._B_ICON, icon);
                mts.setValue(DcState._ID, id);
                mts.saveNew(false);
                
                states.add(mts);
            }
            
            
            ResultSet rsMf = stmt.executeQuery("SELECT * FROM musicfiles;");
            while (rsMf.next()) {
                String state = rsMf.getString("state");
                String location = rsMf.getString("location");
                String genre = rsMf.getString("genre");
                int quality = rsMf.getInt("quality");
                String storagemedium = rsMf.getString("storagemedium");
                int year = rsMf.getInt("year");
                String album = rsMf.getString("album");
                String encoding = rsMf.getString("encoding");
                String description = rsMf.getString("description");
                String lyric = rsMf.getString("lyric");
                String filename = rsMf.getString("filename");
                String artist = rsMf.getString("artist");
                String title = rsMf.getString("title");
                int filesize = rsMf.getInt("filesize");
                int length = rsMf.getInt("length");
                int rating = rsMf.getInt("rating");
                String userlongtext1 = rsMf.getString("userlongtext1");
                String usershorttext1 = rsMf.getString("usershorttext1");
                String usershorttext2 = rsMf.getString("usershorttext2");
                int userinteger1 = rsMf.getInt("userinteger1");
                int userinteger2 = rsMf.getInt("userinteger2");
                
                // check if albums exists
                ResultSet rsAlbum = stmt.executeQuery("Select TOP 1 * FROM musicalbums WHERE UPPER(title) = '" + (album != null ? album.toUpperCase() : "") + "'");
                counter = 0;
                while (rsAlbum.next())
                    counter++;
                
                MusicAlbum ma = null;
                if (counter == 0) {
                	ma = new MusicAlbum();
                	
                	if (album == null || album.trim().length() == 0)
                		album = "Unknown";

            		ResultSet rsAlbum2 = stmt.executeQuery("Select TOP 1 * FROM musicalbums WHERE title = '" + album + "'");
                	
                    counter = 0;
                    while (rsAlbum2.next())
                        counter++;
            		
                    if (counter > 0) {
                        try {
                            rsAlbum2.close();
                        } catch (Exception ignore) {}
                        
                        rsAlbum2 = stmt.executeQuery("Select TOP 1 * FROM musicalbums WHERE title = '" + album + "'");
                        
                    	ArrayList c = (ArrayList) new WorkFlow().convertToDCObjects(rsAlbum2);
                    	for (Iterator iter = c.iterator(); iter.hasNext(); ) {
                    		ma = (MusicAlbum) iter.next();
                            Messages.add("Reusing Album " + ma, Messages._STANDARD);
                    		break;
                    	}
                    	
                    } else {
	                	ma.setValue(MusicAlbum._A_TITLE, album);
	                	ma.setValue(MusicAlbum._C_YEAR, new Integer(year));
	                	ma.setValue(MusicAlbum._D_LOCATION, location);
	                	ma.setValue(MusicAlbum._F_ARTIST, artist);
	                	ma.setValue(MusicAlbum._G_GENRE, genre);
                        ma.setValue(MusicAlbum._I_STORAGEMEDIUM, storagemedium);
	                	
	                	ma.setIDs();
                        
                        Messages.add("Creating Album " + ma, Messages._STANDARD);
                        ma.keepInSync(false);
	                	ma.saveNew(false);
                        
                        // requery just to be sure!
                        ResultSet rsAlbum3 = stmt.executeQuery("Select TOP 1 * FROM musicalbums WHERE title = '" + album + "'");
                        ArrayList c = (ArrayList) new WorkFlow().convertToDCObjects(rsAlbum3);
                        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
                            ma = (MusicAlbum) iter.next();
                            //Messages.add("Reusing Album " + ma, Messages._STANDARD);
                            break;
                        }
                    }
                } else {
                	try {
                		rsAlbum.close();
                	} catch (Exception exp) {}
                	
                	rsAlbum = stmt.executeQuery("Select TOP 1 * FROM musicalbums WHERE UPPER(title) = '" + album.toUpperCase() + "'");
                	
                	ArrayList c = (ArrayList) new WorkFlow().convertToDCObjects(rsAlbum);
                	for (Iterator iter = c.iterator(); iter.hasNext(); ) {
                		ma = (MusicAlbum) iter.next();
                        Messages.add("Reusing Album " + ma, Messages._STANDARD);
                		break;
                	}
                }
                
                if (title == null || title.trim().length() == 0)
                	title = "Unknown";
                
                MusicTrack mt = new MusicTrack();
                mt.setValue(MusicTrack._A_TITLE, title);
                mt.setValue(MusicTrack._B_DESCRIPTION, description);
                mt.setValue(MusicTrack._C_YEAR, new Integer(year));
                mt.setValue(MusicTrack._E_RATING, new Integer(rating));
                mt.setValue(MusicTrack._G_ARTIST, artist);
                mt.setValue(MusicTrack._H_GENRE, genre);
                mt.setValue(MusicTrack._I_FILENAME, filename);
                mt.setValue(MusicTrack._J_PLAYLENGTH, new Integer(length));
                mt.setValue(MusicTrack._K_QUALITY, new Integer(quality));
                mt.setValue(MusicTrack._L_ENCODING, encoding);
                mt.setValue(MusicTrack._M_LYRIC, lyric);
                mt.setValue(MusicTrack._N_FILESIZE, new Integer(filesize));
                
                mt.setValue(MusicTrack._U1_USER_LONGTEXT, userlongtext1);
                mt.setValue(MusicTrack._U2_USER_SHORTTEXT1, usershorttext1);
                mt.setValue(MusicTrack._U3_USER_SHORTTEXT2, usershorttext2);
                mt.setValue(MusicTrack._U4_USER_NUMERIC1, new Integer(userinteger1));
                mt.setValue(MusicTrack._U5_USER_NUMERIC2, new Integer(userinteger2));
                mt.setValue(MusicTrack._O_STATE, state);
                
                
                mt.setValue(MusicTrack._P_ALBUM, ma.getID());
                
                mt.setIDs();

                Messages.add("Adding track " + mt + " to album " + ma, Messages._STANDARD);
                mt.keepInSync(false);
                mt.saveNew(false);
            }
            
            stmt.execute("drop table musicfiles");
            
            hasBeenConverted = true;
            stmt.close();
        } catch (Exception exp) {
        	exp.printStackTrace();
            Messages.add(exp);
            new MessageBox("Conversion was unsuccessful! " + exp.getMessage(), MessageBox._ERROR);
        }
        
        try {
            connection.close();
        } catch (Exception exp) {}
        
        return hasBeenConverted;    	
    }
    
    public boolean convertSoftwarePlatforms() {
        boolean hasBeenConverted = false;
        Connection connection = DatabaseManager.getConnection();
        
        try {
            Statement stmt = connection.createStatement();
            ResultSet rs = stmt.executeQuery("SELECT TOP 1 Platform from Software;");
            
            int counter = 0;
            while (rs.next()) {
                counter++;
            }
            
            boolean needed = counter > 1;
            
            if (needed) {
                int type = rs.getMetaData().getColumnType(1);
                needed = type != Types.BIGINT;
            }
            
            try {
                rs.close();
            } catch (Exception ignore) {}
            
            if (!needed) {
                return false;
            }
        
            new QuestionBox("The Software platforms needs to be converted. Do you want to continue ?");

            if (Boolean.FALSE.equals(QuestionBox.result))
                System.exit(0);
        
            DataCrow.closeSplashScreen();
            
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;
            
            DcObject[] platforms = DcModules.getModule(DcModules._SOFTWAREPLATFORM).getDefaultData();
            ResultSet softwareRs = stmt.executeQuery("SELECT ID, Platform FROM Software WHERE Platform IS NOT NULL");
            while (softwareRs.next()) {
                String platform = softwareRs.getString("Platform");
                String ID = softwareRs.getString("ID");
                if (platform != null && platform.trim().length() > 0) {
                    boolean found = false;
                    for (int i = 0; i < platforms.length && !found; i++) {
                        String dcPlatformName = platforms[i].getValue(SoftwarePlatform._A_NAME).toString().toLowerCase();
                        if (platform.toLowerCase().equals(dcPlatformName)) {
                            Messages.add("Converting " + platform + " to Object "  + platforms[i].getValue(SoftwarePlatform._A_NAME), Messages._STANDARD);
                            String platformID = getPlatformID(platforms[i].toString(), stmt); 
                            stmt.execute("UPDATE SOFTWARE SET Platform = '" + platformID  + "' WHERE ID = '" + ID + "'");
                            found = true;
                        }
                    }
                    
                    for (int i = 0; i < platforms.length && !found; i++) {
                        String dcPlatformName = platforms[i].getValue(SoftwarePlatform._A_NAME).toString().toLowerCase();
                        if (platform.toLowerCase().indexOf(dcPlatformName) > -1 || dcPlatformName.indexOf(platform.toLowerCase()) > -1) {
                            Messages.add("Converting " + platform + " to Object "  + platforms[i].getValue(SoftwarePlatform._A_NAME), Messages._STANDARD);
                            String platformID = getPlatformID(platforms[i].toString(), stmt);
                            stmt.execute("UPDATE SOFTWARE SET Platform = '" + platformID + "' WHERE ID = '" + ID + "'");
                            found = true;
                        }
                    }
                    
                    if (!found) {
                        SoftwarePlatform tmp = new SoftwarePlatform();
                        tmp.setIDs();
                        Messages.add("Adding platform object for " + platform, Messages._STANDARD);
                        stmt.execute("INSERT INTO softwareplatforms (ID, NAME) VALUES " +
                                     "('" + tmp.getID() + "', '" + platform + "')");
                        stmt.execute("UPDATE SOFTWARE SET Platform = '" + tmp.getID() + "' WHERE ID = '" + ID + "'");
                        
                        DcObject[] newPlatforms = new DcObject[platforms.length + 1];
                        for (int i = 0; i < platforms.length; i++) {
                            newPlatforms[i] = platforms[i];
                        }
                        
                        tmp.setValue(SoftwarePlatform._A_NAME, platform);
                        newPlatforms[newPlatforms.length - 1] = tmp;
                        platforms = newPlatforms;
                    }
                }
            }
            
            hasBeenConverted = true;
            stmt.close();
        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox("Conversion was unsuccessful! " + exp.getMessage(), MessageBox._ERROR);
        }
        
        try {
            connection.close();
        } catch (Exception exp) {}
        
        return hasBeenConverted;
    }

    public String getPlatformID(String name, Statement stmt) throws Exception {
        ResultSet rs = stmt.executeQuery("SELECT ID FROM softwareplatforms WHERE name ='" + name + "'");
        String ID = null;
        while (rs.next()) {
            ID = rs.getString("ID");
            break;
        }
        
        try {
            rs.close();
        } catch (Exception ignore) {}
        
        return ID;
    }
    
    public boolean convertTableTypes() {
        boolean hasBeenConverted = false;
        Connection connection = DatabaseManager.getConnection();
        try {
            Statement stmt = connection.createStatement();
            ResultSet rs = stmt.executeQuery("SELECT TOP 1 ID from Software;");
            
            boolean needsConversion = false;
            while (rs.next()) {
                needsConversion = rs.getMetaData().getColumnType(1) == Types.VARCHAR;
            }
            
            stmt.close();
            rs.close();
            
            if (needsConversion) {
                new QuestionBox("The database needs to be converted for optimal performance." +
                                "The whole process can take over 20 minutes. Continue ?");

                if (Boolean.FALSE.equals(QuestionBox.result))
                    System.exit(0);
                
                DataCrow.closeSplashScreen();
                
                MessagesWriter._windowShow = true;
                MessagesWriter._windowToFront = true;
                
                try {
                    stmt = connection.createStatement();
                    stmt.execute("ALTER TABLE audiotracks DROP CONSTRAINT FK_AUDIOCDS_AUDIOTRACKS");
                    
                    Messages.add("Converting picture table", Messages._STANDARD);
                    stmt.execute("ALTER TABLE " + DcModules.getModule(DcModules._PICTURE).getTableName() + " ALTER COLUMN OBJECTID BIGINT");
                    stmt.close();
                } catch (Exception ignore) {
                    Messages.add(ignore);
                }

                for (Iterator iter = DcModules.getAllModules(); iter.hasNext(); ) {
                    DcModule module = (DcModule) iter.next();
                    
                    if (!(module instanceof GenericModule) && !(module instanceof PictureModule)) {

                        stmt = connection.createStatement();
                        
                        DcObject dco = module.getDcObject();
                        Query q = new Query(Query._CREATE, dco, null, null);
                        String qs = q.getQueries()[0];
                        
                        String query = qs.replaceAll("CREATE MEMORY TABLE " + module.getTableName(), 
                                                     "CREATE MEMORY TABLE " + module.getTableName() + "_NEW");
                        
                        if (!query.equals(qs)) {
                            stmt.execute(query);
    
                            rs = stmt.executeQuery("SELECT ID FROM " + module.getTableName());
                            while (rs.next()) {
                                String ID = rs.getString("ID");
                                ResultSet rsObject = stmt.executeQuery("SELECT * FROM " + module.getTableName() + " WHERE ID = " + ID);
                                Collection objects = new WorkFlow().convertToDCObjects(rsObject);
                                for (Iterator iterObj = objects.iterator(); iterObj.hasNext(); ) {
                                    DcObject o = (DcObject) iterObj.next();
                                    
                                    Messages.add("Tranferring  [" + o.getID() + "] " + o.toString(), Messages._STANDARD);
                                    
                                    try {
                                        q = new Query(Query._INSERT, o, null, null);
                                        query = q.getQueries()[0];
                                        query = query.replaceAll("INSERT INTO " + module.getTableName(), "INSERT INTO " + module.getTableName() + "_NEW");
                                        stmt.execute(query);
                                    } catch (SQLException exp) {
                                        
                                        if (exp.getMessage().indexOf("Unique constraint violation") > -1) {
                                            o.setIDs();
                                            q = new Query(Query._INSERT, o, null, null);
                                            query = q.getQueries()[0];
                                            query = query.replaceAll("INSERT INTO " + module.getTableName(), "INSERT INTO " + module.getTableName() + "_NEW");
                                            stmt.execute(query);
                                            
                                        } else {
                                            throw new Exception(exp);
                                        }
                                    }
                                }
                                rsObject.close();
                            }
                            
                            rs.close();
                            
                            stmt.execute("DROP TABLE " + module.getTableName());
                            stmt.execute("ALTER TABLE " + module.getTableName() + "_NEW RENAME TO " + module.getTableName());
                            hasBeenConverted = true;
                            
                            MessagesWriter._useLogFile = true;
                            stmt.close();
                            System.gc();
                        }
                    }
                } 
            } 
        } catch (Exception exp) {
            Messages.add(exp);
            Messages.add("Could not upgrade the database ! Please restore a previous version !", Messages._ERROR);
            new MessageBox("Could not upgrade the database ! Please restore a previous version !", MessageBox._WARNING);
        }        

        try {
            connection.close();
        } catch (Exception exp) {}

        return hasBeenConverted;            
    }
    
  public boolean convertPictures() {
      boolean wasConverted = false;
      try {
          Connection connection = DatabaseManager.getConnection();
    
          Statement stmt = connection.createStatement();
          
          boolean needsConversion = false;
          try {
              ResultSet rs = stmt.executeQuery("SELECT TOP 1 image from Pictures;");
              rs.close();
              needsConversion = true;
          } catch (Exception exp) {
          }
          
          if (!needsConversion)
              return false;
    
          new QuestionBox("The database needs to be converted. Do you want to continue ?");
    
          if (Boolean.FALSE.equals(QuestionBox.result))
              System.exit(0);
    
          DataCrow.closeSplashScreen();
          MessagesWriter._windowShow = true;
          MessagesWriter._windowToFront = true;
    
          Messages.add("Scanning software table", Messages._STANDARD);
          String query = "Select Field, Objectid, Image from PICTURES";
          ResultSet rs = stmt.executeQuery(query);
          while (rs.next()) {
              String field = rs.getString("Field");
              String objectID = rs.getString("Objectid");
              String base64 = rs.getString("Image");
              
              if (objectID != null && field != null) {
                  String filename = objectID + "_" + field + ".jpg";
                  Messages.add("Creating image " + filename, Messages._STANDARD);
                  
                  Utilities.writeImageToFile(base64, DataCrow.imageDir + filename);
                  
                  ImageIcon icon = Utilities.base64ToImage(base64);
                  stmt.execute("UPDATE Pictures SET filename='" + filename + "', height=" + icon.getIconHeight() +
                               ", width=" + icon.getIconWidth() + " WHERE objectid = " + objectID + 
                               " AND field = '" + field + "'");
              }
          }
    
          Messages.add("Dropping picture columns", Messages._STANDARD);
          Messages.add("Dropping image column", Messages._STANDARD);
          stmt.execute("ALTER TABLE pictures  DROP COLUMN Image");
    
          wasConverted = true;
       } catch (Exception exp) {
          Messages.add(exp);
       }
       return wasConverted;
    }
}
