/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilterOptions;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.IChildModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.WorkFlow;
import net.sf.dc.wf.requests.SynchronizeWithManagerRequest;

public class DatabaseManager {

    public static DcDatabase db = null;

    public static void initialize() {
        db = new DcDatabase();

        Connection connection = null;
        while (connection == null) {
            connection = getConnection();
        }

        try {
            db.initiliaze(connection);
        } catch (Exception exp) {
            Messages.add("Could not find or connect to the database!", Messages._ERROR);
            Messages.add(exp);
            new MessageBox("Could not find or connect to the database!", MessageBox._ERROR);
        }
    }

    public static int getQueueSize() {
    	return db.getQueueSize();
    }
    
    public static void applySettings() {
        db.setDbProperies(getConnection());
    }
    
    public static void closeDatabases(boolean compact) {
        try {
            if (db != null) {
                Connection connection = getConnection();
                Statement stmt = connection.createStatement();
                
                if (compact)
                    stmt.executeUpdate("SHUTDOWN COMPACT");
                else 
                    stmt.executeUpdate("SHUTDOWN");
                connection.close();
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }

    public static Connection getConnection() {
        try {
            String name = db.getName();
            Class.forName(DcSettings.getValueAsString(Repository.settings.stDatabaseDriver));

            String address = "jdbc:hsqldb:" + DataCrow.baseDir +  "data/" + name;
            Connection connection = DriverManager.getConnection(address, "SA", "");
            connection.setAutoCommit(true);

            return connection;
        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox(exp.getMessage(), MessageBox._ERROR);
        }

        System.exit(1);
        return null;
    }

    public static ResultSet runQueryDirectUnclosed(String sQuery, boolean silent) throws Exception {
        if (!silent)
            Messages.add(sQuery, Messages._QUERY);

        Connection connection = getConnection();
        Statement stmt = connection.createStatement();
        return stmt.executeQuery(sQuery);
    }

    public static Collection runQueryDirect(String sQuery, boolean catchErrors, boolean logQuery) throws Exception {
        Collection data = null;
        try {
            Connection connection = getConnection();
            Statement stmt = connection.createStatement();
            ResultSet result = stmt.executeQuery(sQuery);
            data = new WorkFlow().convertToDCObjects(result);

            result.close();
            stmt.close();
            connection.close();
        } catch (SQLException sqlExp) {
            if (!sqlExp.getMessage().equals("No ResultSet was produced")) {
                if (catchErrors)
                    Messages.add(sqlExp);
                else
                    throw sqlExp;
            }
        }

        if (logQuery)
            Messages.add(sQuery, Messages._QUERY);

        return data;
    }

    public static Collection runQueryDirect(String sQuery, boolean logQuery) {
        Collection objects = null;

        try {
            objects = runQueryDirect(sQuery, true, logQuery);
        } catch (Exception exp) {
            Messages.add(exp);
        }

        return objects;
    }

    public static  Collection runQueryDirect(DcObject dco, boolean logQuery) {
        Query query = new Query(Query._SELECT, dco, null, null);
        Messages.add(query.getQueries()[0], Messages._QUERY);
        return runQueryDirect(query.getQueries()[0], logQuery);
    }

    public static  Collection runQueryDirect(Query query, boolean logQuery) {
        return runQueryDirect(query.getQueries()[0], logQuery);
    }

    public static  void updateValues(DcObject dco) {
        boolean isChanged = dco.isChanged();
        if (isChanged) {
            Query query = new Query( Query._UPDATE, dco, null, dco.getRequests());

            if (dco.isBatch())
                query.setBatch(dco.isEndOfBatch());

            query.setSilence(dco.getSilence());
            db.addQuery(query);
        }

        Collection c = dco.getChildren();
        Collection children = new ArrayList();
        children.addAll(c);
        int counter = 1;
        for (Iterator iter = children.iterator(); iter.hasNext(); counter++) {
            DcObject child = (DcObject) iter.next();
            if (child.isChanged()) {

                child.addRequest(new SynchronizeWithManagerRequest(
                        SynchronizeWithManagerRequest._UPDATE, child));
                
                boolean exists = false;
                if (child.getID() != null && child.getID().length() > 0) {
                    DcObject childTest = child.getModule().getDcObject();
                    childTest.setValue(DcObject._ID, child.getID());
                    Collection objects = runQueryDirect(childTest, true);
                    exists = objects.size() > 0;
                }

                Query query;
                if (!exists) {
                    query = new Query(Query._INSERT, child, null, child.getRequests());
                } else {
                    query = new Query(Query._UPDATE, child, null, child.getRequests());
                }

                if (!isChanged) {
                    query.setBatch(counter == children.size());
                    query.setSilence(counter != children.size());
                } else {
                    query.setSilence(true);
                }
                db.addQuery(query);
            }
        }
    }

    public static  void insertValues(DcObject dco) {
        if (dco.getID() == null || dco.getID().equals(""))
            dco.setIDs();

        Query query = new Query(Query._INSERT, dco, null, dco.getRequests());
        if (dco.isBatch())
            query.setBatch(dco.isEndOfBatch());

        query.setSilence(dco.getSilence());
        db.addQuery(query);

        Collection children = dco.getChildren();
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DcObject child = (DcObject) iter.next();
            
            child.addRequest(new SynchronizeWithManagerRequest(
                    SynchronizeWithManagerRequest._ADD, child));
            
            child.setValue(child.getParentReferenceFieldIndex(), dco.getID());
            query = new Query(Query._INSERT, child, null, child.getRequests());
            query.setSilence(true);
            db.addQuery(query);
        }
    }

    public static  void deleteValues(DcObject dco) {
        Query query = new Query(Query._DELETE, dco, null, dco.getRequests());
        if (dco.isBatch())
            query.setBatch(dco.isEndOfBatch());
        
        if (dco.getModule().getChildModule() != null) {
        	
        }
        
        query.setSilence(dco.getSilence());
        db.addQuery(query);
    }


    public static boolean uniqueValues(DcObject o, boolean isUpdate) {
        if (o.hasPrimaryKey() && !(o.getModule() instanceof IChildModule)) {
            boolean hasRequiredFields = false;
            DcObject dco = o.getModule().getDcObject();

            int[] fields = o.getFieldIndices();
            for (int i = 0 ; i < fields.length; i++) {
                int field = fields[i];
                if (o.isRequired(field)) {
                    hasRequiredFields = true;
                    dco.setValue(field, o.getValue(field));
                }
            }

            if (hasRequiredFields) {
                DataFilterOptions dfo = new DataFilterOptions(null, true, true);
                DataFilter df = new DataFilter(o.getModule().getIndex(), dco, dfo);
                DcObject[] objects = DataManager.get(o.getModule().getIndex(), df);
                
                int count = 0;
                for (int i = 0; i < objects.length; i++) {
                	count = !isUpdate || !objects[i].getID().equals(o.getID()) ? count + 1 : count;
                }

                if (count > 0)
                    return false;
            }
        }
        return true;
    }
}

