/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.http.covers;

import java.io.PrintWriter;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.resources.DcResources;

public class AmazonCoverSource {

    private final IOnlineUpdateListener listener;
    private final AudioCD audioCD;

    private boolean keepOnRunning = true;

	private static final String SOAP_ACTION = "SearchArtistRequest";
	private static final String SOAP_URL = "http://soap.amazon.com/onca/soap3";
	private static final Pattern SOAP_PATTERN = Pattern.compile(".+?ProductName.+?>(.+?)</ProductName.+?Artist .+?>(.+?)</Artist.+?ImageUrlSmall.+?>(.+?)</ImageUrlSmall.+?ImageUrlMedium.+?>(.+?)</ImageUrlMedium.+?ImageUrlLarge.+?>(.+?)</ImageUrlLarge.*");

    private static final Pattern[] stripPatterns = {
            Pattern.compile(" ?\\(dis[ck] \\d+?\\)"),
            Pattern.compile("^(the|a|an|el|le|les|la|los|die|der|das|den) "),
            Pattern.compile(" (the|a|an|el|le|les|la|los|die|der|das|den) "),
            Pattern.compile("('|~|\\!|@|#|\\$|%|\\^|\\*|_|\\[|\\{|\\]|\\}|\\||\\\\|;|:|`|\"|<|,|>|\\.|\\?|/|&)")};

	public AmazonCoverSource(IOnlineUpdateListener listener, AudioCD audioCD) {
        this.listener = listener;
        this.audioCD = audioCD;
    }

    public void stopSearch() {
        keepOnRunning = false;
    }

	private String getSoapRequest(String artist, String album) {
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<SOAP-ENV:Envelope SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:SOAP-ENC=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\" xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/1999/XMLSchema\">");
		sb.append("<SOAP-ENV:Body>");
		sb.append("<ns1:ArtistSearchRequest xmlns:ns1=\"http://soap.amazon.com\" SOAP-ENC:root=\"1\">");
		sb.append("<v1>" + "<sort xsi:type=\"xsd:string\">+pmrank</sort>");
		sb.append("<artist xsi:type=\"xsd:string\">").append(artist).append("</artist>");
		sb.append("<devtag xsi:type=\"xsd:string\">D1R5WXCELBX4NF</devtag>");
		sb.append("<tag xsi:type=\"xsd:string\">accessonline</tag>");
		sb.append("<mode xsi:type=\"xsd:string\">music</mode>");
		sb.append("<keywords xsi:type=\"xsd:string\">").append(album).append("</keywords>");
		sb.append("<type xsi:type=\"xsd:string\">lite</type>");
		sb.append("<page xsi:type=\"xsd:int\">1</page>" + "</v1>");
		sb.append("</ns1:ArtistSearchRequest>" + "</SOAP-ENV:Body>");
		sb.append("</SOAP-ENV:Envelope>");

		return sb.toString();
	}

	private String sendSoapRequest(String request) throws Exception {
		//Set up the request
        URL url = new URL(SOAP_URL);
		HttpConnection connection = new HttpConnection(url);
		connection.setContentLength(request.length());
		connection.setToSoapRequest(SOAP_ACTION);

		//Send our xml request
        //ui.addMessage(DcResources.getText("msgSendingSearchAmazon"));
		PrintWriter out = new PrintWriter(connection.getOutputStream());
		out.println(request);
		out.close();

		//Receive the answer
        String answer = connection.getString();
		return answer;
	}

	public void search() {
        String album = audioCD.getValue(AudioCD._A_TITLE).toString();
        String artist = audioCD.getValue(AudioCD._F_ARTIST).toString();

		//Transform the artist and album strings to better search
		artist = normalize(artist);
		album = normalize(album);

        String answer;
        try {
        	answer = sendSoapRequest(getSoapRequest(artist, album));
        	answer = answer.replaceAll("\n", "");
        	answer = answer.replaceAll("&amp;", "&");
        } catch (Exception exp) {
        	//ui.addMessage(DcResources.getText("msgCouldNotConnectAmazon", exp.getMessage()));
            Messages.add(exp);
            return;
        }

		int nextSubIndex = 0;
        Vector vURLs = new Vector(0);
		while (answer.length() > nextSubIndex && keepOnRunning) {
			Matcher matcher = SOAP_PATTERN.matcher(answer.substring(nextSubIndex));
			if (matcher.matches()) {
				//Retrieve the best url (non-empty image)
				String imageURL = "";
				for (int i = 5; i >= 3; i--) {
					imageURL = matcher.group(i);
					if (!imageURL.equals("")) {
                        vURLs.add(imageURL);
                        break;
                    }
				}
				nextSubIndex += matcher.end(matcher.groupCount());
			} else {
				break;
            }
		}
        retrieveImages(vURLs);
	}

    private void retrieveImages(Collection urls) {
        int results = urls.size();
        if (results == 0)
            return;

        int counter = 0;
        for (Iterator iter = urls.iterator(); iter.hasNext(); counter++) {
            try {
                URL url = new URL(iter.next().toString());
                HttpConnection connection = new HttpConnection(url);
                // check if this image is empty or not
                if (connection.getContentLength() != 807) {
                    // In case more then one image is found, add the item
                    // again with the new image
                    audioCD.setValue(AudioCD._H_PICTUREFRONT, connection.getBytes());
                    break;
                } 
            } catch (Exception exp) {
                listener.addMessage(DcResources.getText("msgErrorWithImage", exp.getMessage()));
                Messages.add(exp);
            }
        }
    }

    public static String normalize(String s) {
        s = s.toLowerCase();

        for (int i = 0; i < stripPatterns.length; i++) {
            Matcher ma = stripPatterns[i].matcher(s);
            s = ma.replaceAll("");
        }

        return s;
    }
}
