/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.messages;

import java.util.Date;
import java.util.LinkedList;

import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;

public class Messages {

	public static final int _STANDARD = 0;
	public static final int _LEAVEDATELESS = 1;
	public static final int _SIMPLE = 2;
	public static final int _QUERY = 3;
	public static final int _TABLESS = 4;
	public static final int _ERROR = 5;

	private static LinkedList lMessagePool = new LinkedList();
	public static boolean isNew = false;

	private static MessagesWriter writeToLog;
    private static String sError =  DcResources.getText("msgError");

	public Messages() {
		writeToLog = new MessagesWriter();
		writeToLog.setDaemon(true);
		writeToLog.start();
	}

    public synchronized static void add(Throwable throwable) {
        StringBuffer sb = new StringBuffer(" ");
        sb.append(throwable.getMessage());
        StackTraceElement[] elements = throwable.getStackTrace();
        for (int i = 0; i < elements.length; i++) {
            sb.append(Repository.operators.opTab);
            sb.append(elements[i].toString());
            sb.append("\n");
        }
        add(sb.toString(), _ERROR);
    }

	public synchronized static void add(String message, int type) {
        StringBuffer sb = new StringBuffer();
		switch (type) {
			case _STANDARD :
                sb.append(Repository.operators.opTab);
                sb.append(getCurrentTime());
                sb.append(Repository.operators.opSeparator);
                sb.append(message);
                sb.append('\n');
				break;
            case _SIMPLE :
			case _LEAVEDATELESS :
                sb.append(Repository.operators.opTab);
                sb.append(message);
                sb.append('\n');
				break;
			case _QUERY :
                sb.append('\n');
                sb.append(message);
                sb.append('\n');
                sb.append('\n');
				break;
			case _TABLESS :
                sb.append(message);
				break;
			case _ERROR :
                sb.append('\n');
                sb.append(Repository.operators.opTab);
                sb.append(Repository.operators.opTab);
                sb.append(Repository.operators.opTab);
                sb.append(sError);
                sb.append(message);
                sb.append('\n');
                sb.append('\n');
				break;
		}
        String s = sb.toString();
		lMessagePool.add(s);
        sb = null;
		isNew = true;
	}

	public synchronized static String getNewMessages() {
		String message = null;
		if (lMessagePool.size() <= 0)
			isNew = false;
		else
			message = lMessagePool.removeFirst().toString();

		return message;
	}

	private static String getCurrentTime() {
        Date date = new Date(System.currentTimeMillis());
		String sCurrentTime = date.toString();
		return sCurrentTime;
	}
}

