/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.messages;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import net.sf.dc.console.forms.LogForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.resources.DcResources;

public final class MessagesWriter extends Thread {

	private static RandomAccessFile fileAccess;
	private static final String accessMode = "rw";
	public static LogForm logForm = new LogForm();

	public static boolean _windowToFront = false;
	public static boolean _windowShow = false;
	public static boolean _useLogFile = true;

    static {
        new Messages();
		final File logFile = new File(DataCrow.baseDir + "data/data_crow.log");
		setWriter(logFile);
	}

	private static final void setWriter(File logFile) {
		try {
			long fileLenght = logFile.length();
			if (fileLenght > 1014220) {
				logFile.delete();
				logFile = new File(DataCrow.baseDir + "data/data_crow.log");
				fileAccess = new RandomAccessFile(logFile, accessMode);
				Messages.add(DcResources.getText("msgNewLogUsed"), Messages._STANDARD);
			} else {
				fileAccess = new RandomAccessFile(logFile, accessMode);
			}
		} catch (IOException exp) {
			Messages.add(exp);
		}
	}

	public MessagesWriter() {
		logForm.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				_windowShow = false;
				logForm.logToFront(false);
			}
		});
	}

	public void run() {
		while (true) {
			if (_windowToFront) {
				logForm.setVisible(true);
				logForm.toFront();
				_windowToFront = false;
				_windowShow = false;
			} else if (_windowShow) {
				logForm.setVisible(true);
				_windowToFront = false;
				_windowShow = false;
			}
			if (Messages.isNew) {
				String newMessage = Messages.getNewMessages();
				if (newMessage != null) {
					write(newMessage);
				}
			} else {
				try {
					// Give other programs time to get things done
					sleep(500);
				} catch (InterruptedException ignore) {}
			}
		}
	}

	public synchronized void write(String message) {
		try {
			// Error message are always written to the logfile,
			// but during export or import the logwriter will
			// only write info to the screen. Though the errors
			// are always shown to the user AND in the logfile.
			if (_useLogFile == true) {
				fileAccess.seek(fileAccess.length());
				fileAccess.writeBytes(message);
			}
            logForm.writeToScreen(message);
            message = null;
		} catch (IOException ioExp) {
		}
	}
}
