package net.sf.dc.processes.http;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.onlinesearch.DcRegion;
import net.sf.dc.core.objects.onlinesearch.DcSearchMode;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class AmazonAudioCdUpdate extends AmazonUpdate {

    public AmazonAudioCdUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener, searchString, mode, region);
    }

    protected Collection convertXmlToObjects(String xmlString, String locale) {
        Collection objects = new ArrayList();
        
        AudioCD base = new AudioCD();

        base.setValue(AudioCD._A_TITLE, getTitle(xmlString));
        base.setValue(AudioCD._B_DESCRIPTION, getDescription(xmlString));
        base.setValue(AudioCD._C_YEAR, getReleaseDate(xmlString));
        base.setValue(AudioCD._E_RATING, getRating(xmlString));
        base.setValue(AudioCD._F_ARTIST, getArtists(xmlString));
        base.setValue(AudioCD._E_RATING, getRating(xmlString));
        base.setValue(AudioCD._G_GENRE, getStyle(xmlString));
        base.setValue(AudioCD._M_WEBPAGE, getAmazonURL(xmlString));
        setImages(base, xmlString, AudioCD._H_PICTUREFRONT, null);
        
        Collection trackSets = getAllTagValues("<Disc Number=", "</Disc>", xmlString);
        try {
            int disc = 1;
            for (Iterator iter = trackSets.iterator(); iter.hasNext(); disc++) {
                String trackSet = (String) iter.next();
                
                Collection tracks = getAllTagValues("<Track Number=", "</Track>", trackSet);
                DcObject audioCD = (DcObject) base.clone();
                audioCD.setChildren(new ArrayList());
                if (trackSets.size() > 1) {
                    String title = (String) audioCD.getValue(AudioCD._A_TITLE);
                    String addition = DcResources.getText("lblDiscXofX", new String[] {"" + disc, "" + trackSets.size()});
                    title += " " + addition;
                    audioCD.setValue(AudioCD._A_TITLE, title);
                }
                
                int counter = 1;
                for (Iterator iterTracks = tracks.iterator(); iterTracks.hasNext(); counter++) {
                    String track = (String) iterTracks.next();
                    String title = track.substring(track.indexOf(">") + 1, track.length());
                    AudioTrack audioTrack = new AudioTrack();
                    audioTrack.setValue(AudioTrack._A_TITLE, title);
                    audioTrack.setValue(AudioTrack._F_TRACKNUMBER, new Integer(counter));
                    audioCD.addChild(audioTrack);
                }

                audioCD.setIDs();
                objects.add(audioCD);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        
        return objects;
    }
    
    private String getArtists(String xmlString) {
         String artists = StringUtils.getValueBetweenStrings("<Artist>", "</Artist>", xmlString);
         if (artists == null || artists.trim().equals("")) {
             Collection c = getAllTagValues("<Creator Role=", "</Creator>", xmlString);
             artists = "";
             int counter = 0;
             for (Iterator iter = c.iterator(); iter.hasNext(); counter++) {
                 String artistRaw = (String) iter.next(); 
                 String artist = artistRaw.substring(artistRaw.indexOf(">") + 1, artistRaw.length());

                 if (counter > 0)
                     artists += ", ";
                 
                 artists += artist;
             }
         }
         
         return StringUtils.convertHttpString(artists.trim());
    }
}
