/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.onlinesearch.DcRegion;
import net.sf.dc.core.objects.onlinesearch.DcSearchMode;
import net.sf.ext.util.StringUtils;

public class AmazonMovieUpdate extends AmazonUpdate {

    public AmazonMovieUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener, searchString, mode, region);

    }

    protected Collection convertXmlToObjects(String xmlString, String locale) {
        Movie movie = new Movie();

        movie.setValue(Movie._A_TITLE, getTitle(xmlString));
        movie.setValue(Movie._B_DESCRIPTION, getDescription(xmlString));

        movie.setValue(Movie._0_LANGUAGE, getLanguage(xmlString));
        
        movie.setValue(Movie._G_WEBPAGE, getAmazonURL(xmlString));
        movie.setValue(Movie._I_ACTORS, getActors(xmlString));
        movie.setValue(Movie._J_DIRECTOR, getDirectors(xmlString));
        
        movie.setValue(Movie._C_YEAR, getTheatricalReleaseDate(xmlString));
        movie.setValue(Movie._E_RATING, getRating(xmlString));
        movie.setValue(Movie._5_ASIN, getAsin(xmlString));
        
        DcObject storageMedium = getStorageMedium(xmlString, DataManager.get(DcModules._MOVIESTORAGE, null), "<ProductGroup>", "</ProductGroup>");
        movie.setValue(Movie._7_STORAGEMEDIUM, storageMedium);
        
        movie.setValue(Movie._L_PLAYLENGTH, getPlayLength(xmlString));
        
        movie.setValue(Movie._H_GENRE, getGenre(xmlString));        

        setImages(movie, xmlString, Movie._X_PICTUREFRONT, null);
        
        Collection objects = new ArrayList();
        objects.add(movie);
        return objects;

    }
    
    private String getLanguage(String xmlString) {
        Collection languages = getAllTagValues("<Language>", "</Language>", xmlString);
        for (Iterator iter = languages.iterator(); iter.hasNext(); ) {
            String language = (String) iter.next();
            
            if (language.toLowerCase().indexOf("original") > -1 || languages.size() == 1)
                return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<Name>", "</Name>", xmlString));
        }
        return null;
    }
    
    private Integer getPlayLength(String xmlString) {
        String runningTime = StringUtils.getValueBetweenStrings("<RunningTime Units=" + '"' + "minutes" + '"' + ">", "</RunningTime>", xmlString);
        try {
            int minutes = Integer.parseInt(runningTime);
            if (minutes > 0)
                return new Integer(minutes * 60);
        } catch (Exception ignore) {}
        
        return null;
    }
    
    private String getDirectors(String xmlString) {
        Collection c = getAllTagValues("<Director>", "</Director>", xmlString);
        int counter = 0;
        String actors = "";
        for (Iterator iter = c.iterator(); iter.hasNext(); counter++) {
            if (counter > 0)
                actors += ", ";
            
            actors += iter.next();
        }       
        return StringUtils.convertHttpString(actors);
    }    
    
    private String getActors(String xmlString) {
        Collection c = getAllTagValues("<Actor>", "</Actor>", xmlString);
        int counter = 0;
        String actors = "";
        for (Iterator iter = c.iterator(); iter.hasNext(); counter++) {
            if (counter > 0)
                actors += ", ";
            
            actors += iter.next();
        }       
        return StringUtils.convertHttpString(actors);
    }
}
