/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.awt.Color;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcReportTextPositionCombo;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

import com.lowagie.text.Cell;
import com.lowagie.text.Chapter;
import com.lowagie.text.Chunk;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;

public class HtmlCardReport extends HtmlReport {

    public HtmlCardReport(ReportingDialog dialog, Collection data, File target,
            int module, ReportFieldDefinitions definitions) {
        super();

        ReportSettings.setCurrentReport(ReportSettings._HTML_CARD_REPORT, module);

        this.dialog = dialog;
        this.data = data;
        this.target = target;
        this.definitions = definitions;
    }

    public void create() throws Exception {
        initialize();

        int textPosition = ReportSettings.getValueAsInt(Repository.reportsettings.stLabelPosition);
        int collabelspan = 1;
        int coltextspan = 3;
        if (textPosition == DcReportTextPositionCombo._BELOWLABELS) {
            collabelspan = 4;
            coltextspan = 4;
        }

        int counter = 0;
        for (Iterator iterData = data.iterator(); iterData.hasNext() && keepOnRunning; counter++) {
            DcObject o = (DcObject) iterData.next();

            Table datatable = getTable(4);
            datatable.setBorderWidth(0f);
            datatable.setBorder(0);

            dialog.addMessage(DcResources.getText("msgAddingToReport", o.toString()));

            Collection pictureFields = new ArrayList();
            for (Iterator iter = definitions.iterator(); iter.hasNext() && keepOnRunning;) {
                ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();

                if (definition.isEnabled()) {
                    int field = definition.getIndex();
                    String s = o.getDisplayString(field);
                    if (s.length() > 0 && !s.equals("0") && !s.equals("-1")) {
                        String label = o.getLabel(field);

                        Cell cellField = getLabelCell(label, definition);
                        cellField.setWidth("200");
                        cellField.setColspan(collabelspan);
                        datatable.addCell(cellField);

                        Cell cell;
                        if (field == GenericObject._SYS_AVAILABLE) {
                            cell = getAvailabilityCell(s, definition);
                            cell.setVerticalAlignment(Cell.ALIGN_CENTER);
                            cell.setColspan(coltextspan);
                        } else if (o.getFieldType(field) == ComponentFactory._RATINGCOMBOBOX) {
                            Object value = o.getValue(field);
                            cell = getRatingCell(value, definition);
                            cell.setVerticalAlignment(Cell.ALIGN_CENTER);
                            cell.setColspan(coltextspan);
                        } else {
                            cell = getTextCell(s, definition);
                            cell.setColspan(coltextspan);
                        }
                        datatable.addCell(cell);
                    } else if (o.getFieldType(field) == ComponentFactory._PICTUREFIELD
                            && o.getValue(field) != null) {

                        Image image = getImage(o, field);
                        if (image != null) {
                            Cell cell = new Cell(image);
                            pictureFields.add(cell);
                        }
                    }
                }
            }

            Table pictureTable = getImageTable(pictureFields);

            Color labelColor = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelColor);
            Font labelFont = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stLabelFont), labelColor);

            Paragraph cTitle = new Paragraph(o.toString(), labelFont);

            Table childrenTable = getChildTable(o);
            boolean createIndex = ReportSettings.getValueAsBoolean(Repository.reportsettings.stCreateIndex);

            datatable.complete();
            if (!createIndex) {
                document.add(datatable);
                document.add(new Chunk("\r\n\r\n"));
                document.add(pictureTable);
                document.add(new Chunk("\r\n\r\n"));
                if (childrenTable != null) {
                    document.add(new Chunk("\r\n\r\n"));
                    document.add(childrenTable);
                }
                
            } else {
                Chapter chapter = new Chapter(cTitle, counter + 1);
                chapter.add(datatable);
                chapter.add(pictureTable);
                if (childrenTable != null)
                    chapter.add(childrenTable);
                document.add(chapter);
            }

            document.add(new Chunk("\r\n\r\n"));
            document.newPage();
            dialog.updateProgressBar(counter + 1);
        }
        document.close();
        dialog.addMessage(DcResources.getText("lblExportHasFinished"));
    }

    private Table getImageTable(Collection cells) throws Exception {
        Table table = getTable(2);
        table.setBorderWidth(0f);
        table.setBorder(0);
        
        for (Iterator iter = cells.iterator(); iter.hasNext();) {
            table.addCell((Cell) iter.next());
        }
        return table;
    }

//    private Table getAudioTrackTable(DcObject parent) throws Exception {
//        Table childrenTable = null;
//        if (DcModules.getCurrentModule().getIndex() == DcModules._AUDIOCD) {
//            parent.loadChildren();
//            Collection children = parent.getChildren();
//
//            if (children != null && children.size() > 0) {
//                AudioTrack audioTrack = new AudioTrack();
//                int[] indices = new int[] { AudioTrack._F_TRACKNUMBER, AudioTrack._A_TITLE, AudioTrack._H_PLAYLENGTH };
//                childrenTable = getTable(3);
//                childrenTable.setBorderWidth(0f);
//                childrenTable.setBorder(0);                
//                childrenTable.setWidths(new float[] { 1, 5, 1 });
//
//                for (int i = 0; i < indices.length; i++) {
//                    String label = audioTrack.getField(indices[i]).getLabel();
//                    childrenTable.addCell(getLabelCell(label, null));
//                }
//
//                for (Iterator iter = children.iterator(); iter.hasNext();) {
//                    DcObject o = (DcObject) iter.next();
//                    for (int j = 0; j < indices.length; j++) {
//                        String s = o.getDisplayString(indices[j]);
//                        childrenTable.addCell(getTextCell(s, null));
//                    }
//                }
//            }
//        }
//        return childrenTable;
//    }

}
