/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.awt.Color;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcReportTextPositionCombo;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

import com.lowagie.text.Cell;
import com.lowagie.text.Chapter;
import com.lowagie.text.Font;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;

public class PdfCardReport extends PdfReport {

    public PdfCardReport(ReportingDialog dialog,
                         Collection data,
                         File target,
                         int module,
                         ReportFieldDefinitions definitions) {
        super();

        ReportSettings.setCurrentReport(ReportSettings._PDF_CARD_REPORT, module);

        this.dialog = dialog;
        this.data = data;
        this.target = target;
        this.definitions = definitions;
    }

    public void create() throws Exception {
        initialize();

        int textPosition = ReportSettings.getValueAsInt(Repository.reportsettings.stLabelPosition);
        int collabelspan = 1;
        int coltextspan = 3;
        if (textPosition == DcReportTextPositionCombo._BELOWLABELS) {
            collabelspan = 4;
            coltextspan = 4;
        }

        int counter = 0;
        for (Iterator iterData = data.iterator(); iterData.hasNext() && keepOnRunning; counter++) {
            Table datatable = getTable(4);

            DcObject o = (DcObject) iterData.next();
            dialog.addMessage(DcResources.getText("msgAddingToReport", o.toString()));

            Collection pictureFields = new ArrayList();
            for (Iterator iter = definitions.iterator(); iter.hasNext() && keepOnRunning; ) {
                ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();

                if (definition.isEnabled()) {
                    int field = definition.getIndex();
                	String s = o.getDisplayString(field);
                	if (s.length() > 0 && !s.equals("0") && !s.equals("-1")) {
                        String label = o.getLabel(field);

                        Cell cellField = getLabelCell(label, definition);
                        cellField.setColspan(collabelspan);
                		datatable.addCell(cellField);

                        Cell cell;
                        if (field == GenericObject._SYS_AVAILABLE) {
                            cell = getAvailabilityCell(s, definition);
                            cell.setVerticalAlignment(Cell.ALIGN_BOTTOM);
                            cell.setColspan(coltextspan);
                        } else if (field == GenericObject._SYS_MODULE) {
                            DcModule value = (DcModule) o.getValue(field);
                            cell = getModuleCell(value, definition);
                            cell.setVerticalAlignment(Cell.ALIGN_BOTTOM);
                            cell.setColspan(coltextspan);
                        } else if (o.getFieldType(field) == ComponentFactory._RATINGCOMBOBOX) {
                            Object value = o.getValue(field);
                            cell = getRatingCell(value, definition);
                            cell.setVerticalAlignment(Cell.ALIGN_BOTTOM);
                            cell.setColspan(coltextspan);
                        } else {
                    		cell = getTextCell(s, definition);
                            cell.setColspan(coltextspan);
                        }

                        cell.setWidth("" + definition.getWeight());
                        datatable.addCell(cell);

                	} else if (    o.getFieldType(field) == ComponentFactory._PICTUREFIELD &&
                                   o.getValue(field) != null) {

                        Picture picture = (Picture) o.getValue(field);
                        try {
                            ImageIcon image = picture.getScaledPicture();
                            if (image != null)
                                pictureFields.add(getImageCell(image, definition, 200, 150, false));
                        } catch (Exception ignore) {}
                    }
                } // enabled
            }

            Table pictureTable = getImageTable(pictureFields);

            Color labelColor = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelColor);
            Font labelFont = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stLabelFont), labelColor);
            Paragraph cTitle = new Paragraph(o.toString(), labelFont);

            Table childTable = getChildTable(o);
            boolean createIndex = ReportSettings.getValueAsBoolean(Repository.reportsettings.stCreateIndex);

            datatable.complete();
            if (!createIndex) {
                document.add(datatable);
                document.add(pictureTable);
                if (childTable != null) document.add(childTable);
            } else {
                Chapter chapter = new Chapter(cTitle, counter + 1);
                chapter.add(datatable);
                chapter.add(pictureTable);
                if (childTable != null) chapter.add(childTable);
                document.add(chapter);
            }

            document.newPage();
            dialog.updateProgressBar(counter + 1);
        }
        document.close();
        dialog.addMessage(DcResources.getText("lblExportHasFinished"));
    }

    private Table getImageTable(Collection cells) throws Exception {
        Table table = getTable(2);
        for (Iterator iter = cells.iterator(); iter.hasNext(); ) {
            table.addCell((Cell) iter.next());
        }
        return table;
    }
}
