/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

import com.lowagie.text.Cell;
import com.lowagie.text.Table;

public class PdfListReport extends PdfReport {

    public PdfListReport(   ReportingDialog dialog,
                            Collection data,
                            File target,
                            int module,
                            ReportFieldDefinitions definitions) {

        ReportSettings.setCurrentReport(ReportSettings._PDF_LIST_REPORT, module);

        this.dialog = dialog;
        this.data = data;
        this.target = target;
        this.definitions = definitions;
    }

    public void create() throws Exception {
        initialize();

        if (data == null || data.size() == 0) return;

        // create the table and the header
        Table datatable = getTable(definitions.getEnabledCount());
        datatable.setCellsFitPage(true);
        DcObject dco = (DcObject) data.toArray()[0];
        setWidths(datatable, dco, definitions);

        for (Iterator iter = definitions.iterator(); iter.hasNext() && keepOnRunning; ) {
            ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
            int field = definition.getIndex();

            if (definition.isEnabled()) {
                String label = dco.getLabel(field);
                datatable.addCell(getLabelCell(label, definition));
            }
        }

        datatable.endHeaders();

        int counter = 0;
        for (Iterator iterData = data.iterator(); iterData.hasNext() && keepOnRunning; counter++) {
            DcObject o = (DcObject) iterData.next();

            dialog.addMessage(DcResources.getText("msgAddingToReport", o.toString()));
            for (Iterator iter = definitions.iterator(); iter.hasNext() && keepOnRunning; ) {
                ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
                int field = definition.getIndex();

                if (definition.isEnabled()) {
                    String s = o.getDisplayString(field);

                    if (field == GenericObject._SYS_AVAILABLE) {
                        Cell availabilityCell = getAvailabilityCell(s, definition);
                        datatable.addCell(availabilityCell);
                    } else if (field == GenericObject._SYS_MODULE) {
                        DcModule module = o.getModule();
                        Cell moduleCell = getModuleCell(module, definition);
                        datatable.addCell(moduleCell);
                    } else if (o.getFieldType(field) == ComponentFactory._RATINGCOMBOBOX) {
                        Object value = o.getValue(field);
                        datatable.addCell(getRatingCell(value, definition));
                    } else if (o.getFieldType(field) == ComponentFactory._PICTUREFIELD) {
                        Picture picture = (Picture) o.getValue(field);
                        Cell cell = null;
                        try {
                            ImageIcon image = picture.getScaledPicture();
                            if (image != null)
                                cell = getImageCell(image, definition, 200, 150, true);
                        } catch (Exception ignore) {}
                        
                        if (cell == null)
                            cell = getTextCell(" ", definition);

                        datatable.addCell(cell);
                    } else {
                        Cell textCell = getTextCell(s, definition);
                        textCell.setMaxLines(5);
                        datatable.addCell(textCell);
                    }
                }
            }
            dialog.updateProgressBar(counter + 1);
        }

        datatable.complete();
        document.add(datatable);
        document.close();

        dialog.addMessage(DcResources.getText("lblExportHasFinished"));
    }
}
