/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.Document;

public class TextReport extends Report {
    
    private BufferedOutputStream bos;
    
    public TextReport(ReportingDialog dialog, Collection data, File target,
                     int module, ReportFieldDefinitions definitions) {
        
        super();
        
        ReportSettings.setCurrentReport(ReportSettings._TEXT_REPORT, module);

        this.dialog = dialog;
        this.data = data;
        this.target = target;
        this.definitions = definitions;
    }    
    
    public void initialize() throws Exception {
        super.initialize();
        
        dialog.initProgressBar(data.size());
        dialog.addMessage(DcResources.getText("msgCreatingReportForXObjects", String.valueOf(data.size())));
    }
    
    public void initializeWriter(Document document, FileOutputStream out) throws Exception {
        bos = new BufferedOutputStream(out);
    }     
    
    private void writeBytes(String value, boolean addTab) throws IOException {
        value = value.replaceAll("\t", "");
        
        if (value.length() > 4 && value.indexOf("\n") > 0) {
            value = value.replaceAll("\r", " ");
            value = value.replaceAll("\n", " ");
        }
        
        if (addTab)
            value = "\t" + value;
        
        bos.write(value.getBytes("UTF8"));
    }
    
    public void create() throws Exception {
        initialize();
        
        if (data == null || data.size() == 0) return;
        
        // create the table and the header
        DcObject dco = (DcObject) data.toArray()[0];
        
        int counter = 0;
        for (Iterator iter = definitions.iterator(); iter.hasNext() && keepOnRunning; ) {
            ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
            int field = definition.getIndex();
            
            if (definition.isEnabled()) {
                String label = dco.getLabel(field);
                writeBytes(label, counter != 0);
                counter++;
            }                
        }        
        
        writeBytes("\r\n", false);
        
        counter = 0;
        for (Iterator iterData = data.iterator(); iterData.hasNext() && keepOnRunning; counter++) {
            DcObject o = (DcObject) iterData.next();
            
            dialog.addMessage(DcResources.getText("msgAddingToReport", o.toString()));
            counter = 0;
            for (Iterator iter = definitions.iterator(); iter.hasNext() && keepOnRunning; ) {
                ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
                int field = definition.getIndex();

                if (definition.isEnabled()) { 
                    String s = "";
                    if (o.getFieldType(field) == ComponentFactory._PICTUREFIELD) {
                        if (o.getValue(field) == null || o.getValue(field).toString().length() < 10)
                            s = DcResources.getText("lblNo");
                        else 
                            s = DcResources.getText("lblYes");
                    } else {
                        s = o.getDisplayString(field);
                    }
                    
                    writeBytes(s, counter != 0);
                    counter++;
                }
            }
            
            writeBytes("\r\n", false);
            dialog.updateProgressBar(counter + 1);
            bos.flush();
        }
        
        bos.close();
        
        dialog.addMessage(DcResources.getText("lblExportHasFinished"));
    }
    
    protected Cell getLabelCell(String label, ReportFieldDefinition definition) throws BadElementException {
        return null;
    }
    
    protected void addFrontPage(Document document) throws Exception {}  

    protected void setTextCellDefaults(Cell cell, ReportFieldDefinition definition) {}
}
