/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.resources;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.messages.Messages;

public class DcResources {

    private final static String resourcesFile = DataCrow.baseDir + "data/resources.properties";
    private static final Properties resources = new Properties();

    public DcResources() {
        try {
            resources.load(getClass().getResourceAsStream("/net/sf/dc/resources/DcLabels.properties"));
            resources.load(getClass().getResourceAsStream("/net/sf/dc/resources/DcMessages.properties"));
            resources.load(getClass().getResourceAsStream("/net/sf/dc/resources/DcTooltips.properties"));
            resources.load(getClass().getResourceAsStream("/net/sf/dc/resources/DcAudioCodecs.properties"));
            resources.load(getClass().getResourceAsStream("/net/sf/dc/resources/DcSystem.properties"));
            resources.load(getClass().getResourceAsStream("/net/sf/dc/resources/DcTips.properties"));
        } catch (Exception ignore) {
            try {
                resources.load(getClass().getResourceAsStream("DcLabels.properties"));
                resources.load(getClass().getResourceAsStream("DcMessages.properties"));
                resources.load(getClass().getResourceAsStream("DcTooltips.properties"));
                resources.load(getClass().getResourceAsStream("DcAudioCodecs.properties"));
                resources.load(getClass().getResourceAsStream("DcSystem.properties"));
                resources.load(getClass().getResourceAsStream("DcTips.properties"));
            } catch (Exception exp) {
                Messages.add(exp);
                new MessageBox("SEVERE ERROR! unable to load resources! " + exp.getMessage(), MessageBox._ERROR);
            }
        }

        try {
            File file = new File(resourcesFile);
            if (file.exists())
                resources.load(new FileInputStream(file));
        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox("Unable to load custom resources! " + exp.getMessage(), MessageBox._ERROR);
        }
    }

    public static Properties getResources() {
        return resources;
    }

    public static void setText(String key, String text) {
        resources.setProperty(key, text);
    }

    public static void save() {
        try {
            File file = new File(resourcesFile);
            FileOutputStream fos = new FileOutputStream(file);
            resources.store(fos, "Custom Resources");
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }

    public static String getText(String id) {
    	return getText(id, (String[]) null);
    }

    public static String getText(String id, String param1) {
    	return getText(id, new String[] {param1});
    }

    public static String getText(String id, String[] params) {
        String value = (String) resources.get(id);
        return params == null ? value : insertParams(value, params);
    }

    private static String insertParams(String s, String[] params) {
    	String result = s;
    	for (int i = 1; i - 1 < params.length; i++) {
    	    String searchPat = "%" + i;
    	    int index = 0;
    	    boolean escapedPatFound = false;
    	    do {
        		index = result.indexOf(searchPat, index);
        		if (index < 0) 
                    Messages.add("Could not insert the parameter for label " + s, Messages._ERROR);
        		else
        		    escapedPatFound = (index > 0 && result.charAt(index - 1) == '\\');
    	    } while (escapedPatFound);

            StringBuffer sb = new StringBuffer(result);
    	    sb.replace(index, index+searchPat.length(), params[i -1]);
    	    result = sb.toString();
    	}
    	return result;
    }
}
