/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class DcSettingsConverter {

    public DcSettingsConverter() {}
    
    public void convert() {
        File settingsFile = new File(DataCrow.baseDir + "data/data_crow.properties");
        if (settingsFile.exists()) {
            
            Properties properties = new Properties();
            try {
                FileInputStream fis = new FileInputStream(settingsFile);
                properties.load(fis);
                fis.close();
            } catch (IOException exp) {
                Messages.add(exp);
            }

            int versionMajor = -1;
            int versionMinor = -1;
            int versionBuild = -1;
            try {
                Object major = properties.get(Repository.settings.stVersionMajor);
                Object minor = properties.get(Repository.settings.stVersionMinor);
                Object build = properties.get(Repository.settings.stVersionBuild);
                
                if (major != null && minor != null && build != null) {
                    versionMajor = Integer.valueOf(major.toString()).intValue();
                    versionMinor = Integer.valueOf(minor.toString()).intValue();
                    versionBuild = Integer.valueOf(build.toString()).intValue();
                }
            } catch (Exception ignore) {}
            
            if (versionMajor < 2 || (versionMinor < 3 || versionBuild < 0)) {
                new DcSettings();
                convertAudioCdFieldIndices();
                convertAudioTrackFieldIndices();
                convertBookFieldIndices();
                convertMovieFieldIndices();
                convertSoftwareFieldIndices();
                
                DcSettings.save();
            }
        }
    }
    
    private boolean convertAudioCdFieldIndices() {
        boolean success = false;
        
        DcFieldDefinitions definitions = (DcFieldDefinitions) 
            DcSettings.getValueAsDefinitionGroup(Repository.settings.stFieldDefinitionsAudioCD);
        success = definitions == null ? true : false;
        
        if (!success) {
            Map conversion = new HashMap(); // key = old index, value = new index
            conversion.put(new Integer(3), new Integer(AudioCD._B_DESCRIPTION));
            conversion.put(new Integer(4), new Integer(AudioCD._C_YEAR));
            conversion.put(new Integer(10), new Integer(AudioCD._D_LOCATION));
            conversion.put(new Integer(11), new Integer(AudioCD._E_RATING));
            conversion.put(new Integer(12), new Integer(AudioCD._U1_USER_LONGTEXT));
            conversion.put(new Integer(13), new Integer(AudioCD._U2_USER_SHORTTEXT1));
            conversion.put(new Integer(14), new Integer(AudioCD._U3_USER_SHORTTEXT2));
            conversion.put(new Integer(15), new Integer(AudioCD._U4_USER_NUMERIC1));
            conversion.put(new Integer(16), new Integer(AudioCD._U5_USER_NUMERIC2));
            conversion.put(new Integer(2), new Integer(AudioCD._F_ARTIST));
            conversion.put(new Integer(5), new Integer(AudioCD._G_GENRE));
            conversion.put(new Integer(6), new Integer(AudioCD._H_PICTUREFRONT));
            conversion.put(new Integer(7), new Integer(AudioCD._I_PICTUREBACK));
            conversion.put(new Integer(8), new Integer(AudioCD._J_PICTURECD));
            conversion.put(new Integer(9), new Integer(AudioCD._K_DISCID));
            
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                Integer value = (Integer) conversion.get(new Integer(definition.getIndex()));
                if (value != null) 
                    definition.setIndex(value.intValue());
            }
            success = true;
        }
        return success;
    }
    
    private boolean convertAudioTrackFieldIndices() {
        boolean success = false;
        
        DcFieldDefinitions definitions = (DcFieldDefinitions) 
            DcSettings.getValueAsDefinitionGroup(Repository.settings.stFieldDefinitionsAudioTrack);
        success = definitions == null ? true : false;

        if (!success) {
            Map conversion = new HashMap(); // key = new index, value = old index
            conversion.put(new Integer(1), new Integer(AudioTrack._F_TRACKNUMBER));
            conversion.put(new Integer(3), new Integer(AudioTrack._G_LYRICS));
            conversion.put(new Integer(4), new Integer(AudioTrack._H_PLAYLENGTH));
            conversion.put(new Integer(5), new Integer(AudioTrack._I_ALBUM));
            conversion.put(new Integer(6), new Integer(AudioTrack._U1_USER_LONGTEXT));
            conversion.put(new Integer(7), new Integer(AudioTrack._U2_USER_SHORTTEXT1));
            conversion.put(new Integer(8), new Integer(AudioTrack._U3_USER_SHORTTEXT2));
            conversion.put(new Integer(9), new Integer(AudioTrack._U4_USER_NUMERIC1));
            conversion.put(new Integer(10), new Integer(AudioTrack._U5_USER_NUMERIC2));
            
            
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                Integer value = (Integer) conversion.get(new Integer(definition.getIndex()));
                if (value != null) 
                    definition.setIndex(value.intValue());
            }
            success = true;
        }
        return success;
    }   
    
    private boolean convertBookFieldIndices() {
        boolean success = false;
        
        DcFieldDefinitions definitions = (DcFieldDefinitions) 
            DcSettings.getValueAsDefinitionGroup(Repository.settings.stFieldDefinitionsBook);
        success = definitions == null ? true : false;

        if (!success) {
            Map conversion = new HashMap(); // key = new index, value = old index
            conversion.put(new Integer(3), new Integer(Book._F_PUBLISHER));
            conversion.put(new Integer(4), new Integer(Book._G_AUTHOR));
            conversion.put(new Integer(6), new Integer(Book._H_WEBPAGE));
            conversion.put(new Integer(7), new Integer(Book._I_CATEGORY));
            conversion.put(new Integer(8), new Integer(Book._J_ISBN10));
            conversion.put(new Integer(9), new Integer(Book._K_PICTUREFRONT));
            conversion.put(new Integer(5), new Integer(Book._C_YEAR));
            conversion.put(new Integer(10), new Integer(Book._D_LOCATION));
            conversion.put(new Integer(11), new Integer(Book._E_RATING));
            conversion.put(new Integer(12), new Integer(Book._U1_USER_LONGTEXT));
            conversion.put(new Integer(13), new Integer(Book._U2_USER_SHORTTEXT1));
            conversion.put(new Integer(14), new Integer(Book._U3_USER_SHORTTEXT2));
            conversion.put(new Integer(15), new Integer(Book._U4_USER_NUMERIC1));
            conversion.put(new Integer(16), new Integer(Book._U5_USER_NUMERIC2));
            
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                Integer value = (Integer) conversion.get(new Integer(definition.getIndex()));
                if (value != null) 
                    definition.setIndex(value.intValue());
            }
            success = true;
        }
        return success;
    }     
    
    private boolean convertMovieFieldIndices() {
        boolean success = false;
        
        DcFieldDefinitions definitions = (DcFieldDefinitions) 
            DcSettings.getValueAsDefinitionGroup(Repository.settings.stFieldDefinitionsMovie);
        success = definitions == null ? true : false;
        
        if (!success) {
            Map conversion = new HashMap(); // key = new index, value = old index
            conversion.put(new Integer(2), new Integer(Movie._F_TITLE_LOCAL));
            conversion.put(new Integer(4), new Integer(Movie._G_WEBPAGE));
            conversion.put(new Integer(5), new Integer(Movie._H_GENRE));
            conversion.put(new Integer(6), new Integer(Movie._I_ACTORS));
            conversion.put(new Integer(7), new Integer(Movie._J_DIRECTOR));
            conversion.put(new Integer(8), new Integer(Movie._K_COUNTRY));
            conversion.put(new Integer(10), new Integer(Movie._L_PLAYLENGTH));
            conversion.put(new Integer(11), new Integer(Movie._M_FILENAME));
            conversion.put(new Integer(12), new Integer(Movie._N_VIDEOCODEC));
            conversion.put(new Integer(13), new Integer(Movie._O_AUDIOCODEC));
            conversion.put(new Integer(14), new Integer(Movie._P_WIDTH));
            conversion.put(new Integer(15), new Integer(Movie._Q_HEIGHT));
            conversion.put(new Integer(16), new Integer(Movie._R_FPS));
            conversion.put(new Integer(17), new Integer(Movie._S_FRAMES));
            conversion.put(new Integer(18), new Integer(Movie._T_AUDIOBITRATE));
            conversion.put(new Integer(19), new Integer(Movie._U_AUDIOSAMPLINGRATE));
            conversion.put(new Integer(20), new Integer(Movie._V_AUDIOCHANNEL));
            conversion.put(new Integer(21), new Integer(Movie._W_VIDEOBITRATE));
            conversion.put(new Integer(22), new Integer(Movie._X_PICTUREFRONT));
            conversion.put(new Integer(23), new Integer(Movie._Y_PICTUREBACK));
            conversion.put(new Integer(24), new Integer(Movie._Z_PICTURECD));
            conversion.put(new Integer(25), new Integer(Movie._0_LANGUAGE));
            conversion.put(new Integer(26), new Integer(Movie._1_AUDIOLANGUAGE));
            conversion.put(new Integer(27), new Integer(Movie._2_SUBTITLELANGUAGE));
            conversion.put(new Integer(28), new Integer(Movie._3_CERTIFICATION));
            conversion.put(new Integer(29), new Integer(Movie._4_FILESIZE));
            conversion.put(new Integer(9), new Integer(Movie._C_YEAR));
            conversion.put(new Integer(30), new Integer(Movie._D_LOCATION));
            conversion.put(new Integer(31), new Integer(Movie._E_RATING));
            conversion.put(new Integer(32), new Integer(Movie._U1_USER_LONGTEXT));
            conversion.put(new Integer(33), new Integer(Movie._U2_USER_SHORTTEXT1));
            conversion.put(new Integer(34), new Integer(Movie._U3_USER_SHORTTEXT2));
            conversion.put(new Integer(35), new Integer(Movie._U4_USER_NUMERIC1));
            conversion.put(new Integer(36), new Integer(Movie._U5_USER_NUMERIC2));
            
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                Integer value = (Integer) conversion.get(new Integer(definition.getIndex()));
                if (value != null) 
                    definition.setIndex(value.intValue());
            }
            success = true;
        }
        return success;
    }
    
//    private boolean convertMusicFieldIndices() {
//        boolean success = false;
//        
//        DcFieldDefinitions definitions = (DcFieldDefinitions) 
//            DcSettings.getValueAsDefinitionGroup(Repository.settings.stFieldDefinitionsMusic);
//        success = definitions == null ? true : false;
//        
//        if (!success) {
//            Map conversion = new HashMap(); // key = new index, value = old index
//            conversion.put(new Integer(4), new Integer(MusicFile._C_YEAR));
//            conversion.put(new Integer(12), new Integer(MusicFile._D_LOCATION));
//            conversion.put(new Integer(13), new Integer(MusicFile._E_RATING));
//            conversion.put(new Integer(14), new Integer(MusicFile._U1_USER_LONGTEXT));
//            conversion.put(new Integer(15), new Integer(MusicFile._U2_USER_SHORTTEXT1));
//            conversion.put(new Integer(16), new Integer(MusicFile._U3_USER_SHORTTEXT2));
//            conversion.put(new Integer(17), new Integer(MusicFile._U4_USER_NUMERIC1));
//            conversion.put(new Integer(18), new Integer(MusicFile._U5_USER_NUMERIC2));
//            conversion.put(new Integer(2), new Integer(MusicFile._F_ARTIST));
//            conversion.put(new Integer(3), new Integer(MusicFile._G_ALBUM));
//            conversion.put(new Integer(5), new Integer(MusicFile._H_GENRE));
//            conversion.put(new Integer(6), new Integer(MusicFile._I_FILENAME));
//            conversion.put(new Integer(7), new Integer(MusicFile._J_PLAYLENGTH));
//            conversion.put(new Integer(8), new Integer(MusicFile._K_QUALITY));
//            conversion.put(new Integer(9), new Integer(MusicFile._L_ENCODING));            
//            conversion.put(new Integer(10), new Integer(MusicFile._M_LYRIC));
//            conversion.put(new Integer(11), new Integer(MusicFile._N_FILESIZE));
//            
//            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
//                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
//                Integer value = (Integer) conversion.get(new Integer(definition.getIndex()));
//                if (value != null) 
//                    definition.setIndex(value.intValue());
//            }
//            success = true;
//        }
//        return success;
//    }    
    
    private boolean convertSoftwareFieldIndices() {
        boolean success = false;
        
        DcFieldDefinitions definitions = (DcFieldDefinitions) 
            DcSettings.getValueAsDefinitionGroup(Repository.settings.stFieldDefinitionsSoftware);
        success = definitions == null ? true : false;
        
        if (!success) {
            Map conversion = new HashMap(); // key = new index, value = old index
            conversion.put(new Integer(3), new Integer(Software._F_DEVELOPER));
            conversion.put(new Integer(4), new Integer(Software._G_PUBLISHER));
            conversion.put(new Integer(5), new Integer(Software._H_PLATFORM));
            conversion.put(new Integer(7), new Integer(Software._I_WEBPAGE));
            conversion.put(new Integer(8), new Integer(Software._J_FILENAME));
            conversion.put(new Integer(9), new Integer(Software._K_TYPE));
            conversion.put(new Integer(10), new Integer(Software._L_MULTI));
            conversion.put(new Integer(11), new Integer(Software._M_PICTUREFRONT));
            conversion.put(new Integer(12), new Integer(Software._N_PICTUREBACK));
            conversion.put(new Integer(13), new Integer(Software._O_PICTURECD));
            conversion.put(new Integer(14), new Integer(Software._P_SCREENSHOTONE));
            conversion.put(new Integer(15), new Integer(Software._Q_SCREENSHOTTWO));
            conversion.put(new Integer(16), new Integer(Software._R_SCREENSHOTTHREE));
            conversion.put(new Integer(17), new Integer(Software._S_SERIALKEY));
            conversion.put(new Integer(18), new Integer(Software._T_FILESIZE));
            conversion.put(new Integer(6), new Integer(Software._C_YEAR));
            conversion.put(new Integer(19), new Integer(Software._D_LOCATION));
            conversion.put(new Integer(20), new Integer(Software._E_RATING));
            conversion.put(new Integer(21), new Integer(Software._U1_USER_LONGTEXT));
            conversion.put(new Integer(22), new Integer(Software._U2_USER_SHORTTEXT1));
            conversion.put(new Integer(23), new Integer(Software._U3_USER_SHORTTEXT2));
            conversion.put(new Integer(24), new Integer(Software._U4_USER_NUMERIC1));
            conversion.put(new Integer(25), new Integer(Software._U5_USER_NUMERIC2));
            
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                Integer value = (Integer) conversion.get(new Integer(definition.getIndex()));
                if (value != null) 
                    definition.setIndex(value.intValue());
            }
            success = true;
        }
        return success;
    }


}

