/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JFrame;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.Definitions;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.jflexsett.settings.ui.SettingsView;

public class ReportSettings {

    public static final int _PDF_CARD_REPORT = 0;
    public static final int _HTML_CARD_REPORT = 1;
    public static final int _PDF_LIST_REPORT = 2;
    public static final int _HTML_LIST_REPORT = 3;
    public static final int _TEXT_REPORT = 4;

    public static final String _ReportPage =  DcResources.getText("lblReportPageSetting");
    public static final String _ReportText =  DcResources.getText("lblReportTextSetting");
    public static final String _ReportLabel = DcResources.getText("lblReportLabelSetting");
    public static final String _ReportTextBorderColor =  DcResources.getText("lblReportTextBorderColor");
    public static final String _ReportLabelBorderColor =  DcResources.getText("lblReportLabelBorderColor");
    public static final String _ReportHeader = DcResources.getText("lblReportHeaderSetting");
    public static final String _ReportHeaderFont = DcResources.getText("lblReportHeaderFont");
    public static final String _ReportHeaderColor = DcResources.getText("lblReportHeaderColor");
    public static final String _ReportHeaderBackgroundColor = DcResources.getText("lblReportHeaderBackgroundColor");
    public static final String _ReportFooter = DcResources.getText("lblReportFooterSetting");
    public static final String _ReportFooterFont = DcResources.getText("lblReportFooterFont");
    public static final String _ReportFooterColor = DcResources.getText("lblReportFooterColor");
    public static final String _ReportFooterBackgroundColor = DcResources.getText("lblReportFooterBackgroundColor");
    public static final String _ReportTextFontSetting = DcResources.getText("lblReportTextFontSetting");
    public static final String _ReportTextColorSetting = DcResources.getText("lblReportTextColorSetting");
    public static final String _ReportTextBackgroundColorSetting = DcResources.getText("lblReportTextBackgroundColorSetting");
    public static final String _ReportLabelFontSetting = DcResources.getText("lblReportLabelFontSetting");
    public static final String _ReportLabelColorSetting = DcResources.getText("lblReportLabelColorSetting");
    public static final String _ReportLabelBackgroundColorSetting = DcResources.getText("lblReportLabelBackgroundColorSetting");

    private static Map htmlCardSettingsMap = new HashMap();
    private static Map htmlListSettingsMap = new HashMap();
    private static Map pdfCardSettingsMap = new HashMap();
    private static Map pdfListSettingsMap = new HashMap();
    private static Map textSettingsMap = new HashMap();

    private static int currentModule = -1;
    private static int currentReportType = -1;

    public ReportSettings() {
    }

    protected void initializeSettings(DcModule module) {
        Settings textSettings = new Settings();
        createGroups(textSettings);
        
        
        createSettings(textSettings, _TEXT_REPORT);
        File textSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_csv_report.properties");
        textSettings.setSettingsFile(textSettingsFile);
        textSettingsMap.put(module.getName(), textSettings);

        Settings pdfListSettings = new Settings();
        createGroups(pdfListSettings);
        createSettings(pdfListSettings, _PDF_LIST_REPORT);
        File pdfListSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_pdf_list_report.properties");
        pdfListSettings.setSettingsFile(pdfListSettingsFile);
        pdfListSettingsMap.put(module.getName(), pdfListSettings);

        Settings htmlListSettings = new Settings();
        createGroups(htmlListSettings);
        createSettings(htmlListSettings, _HTML_LIST_REPORT);
        File htmlListSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_html_list_report.properties");
        htmlListSettings.setSettingsFile(htmlListSettingsFile);
        htmlListSettingsMap.put(module.getName(), htmlListSettings);

        Settings htmlCardSettings = new Settings();
        createGroups(htmlCardSettings);
        createSettings(htmlCardSettings, _HTML_CARD_REPORT);
        File htmlCardSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_html_card_report.properties");
        htmlCardSettings.setSettingsFile(htmlCardSettingsFile);
        htmlCardSettingsMap.put(module.getName(), htmlCardSettings);

        Settings pdfCardSettings = new Settings();
        createGroups(pdfCardSettings);
        createSettings(pdfCardSettings, _PDF_CARD_REPORT);
        File pdfCardSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_pdf_card_report.properties");
        pdfCardSettings.setSettingsFile(pdfCardSettingsFile);
        pdfCardSettingsMap.put(module.getName(), pdfCardSettings);

        setCurrentReport(_TEXT_REPORT, module.getIndex());
        loadSetting(textSettings);

        setCurrentReport(_HTML_LIST_REPORT, module.getIndex());
        loadSetting(htmlListSettings);
        
        setCurrentReport(_HTML_CARD_REPORT, module.getIndex());
        loadSetting(htmlCardSettings);

        setCurrentReport(_PDF_LIST_REPORT, module.getIndex());
        loadSetting(pdfListSettings);

        setCurrentReport(_PDF_CARD_REPORT, module.getIndex());
        loadSetting(pdfCardSettings);
    }
    
    public static void loadSetting(Settings settings) {
        try {
            SettingsFile.load(settings);
        } catch (Exception exp) {
            Messages.add(DcResources.getText("msgFailedToLoadUserSettings", exp.getMessage()), Messages._ERROR);
        }
    }

    public static void setCurrentReport(int index, int module) {
        currentReportType = index;
        currentModule = module;
    }

    private static Settings getCurrentSettings() {
        DcModule module = DcModules.getModule(currentModule);
        String moduleName = DcModules.getModule(currentModule).getName();

        if (!pdfCardSettingsMap.keySet().contains(moduleName) && !pdfListSettingsMap.keySet().contains(moduleName))
            new ReportSettings().initializeSettings(module);

        Settings settings;
        if (currentReportType == _HTML_CARD_REPORT)
            settings = (Settings) htmlCardSettingsMap.get(moduleName);
        else if (currentReportType == _PDF_CARD_REPORT)
            settings = (Settings) pdfCardSettingsMap.get(moduleName);
        else if (currentReportType == _HTML_LIST_REPORT)
            settings = (Settings) htmlListSettingsMap.get(moduleName);
        else if (currentReportType == _PDF_LIST_REPORT)
            settings = (Settings) pdfListSettingsMap.get(moduleName);
        else {
            settings = (Settings) textSettingsMap.get(moduleName);
        }

        return settings;
    }

    public static void save() {
        for (Iterator iter = htmlListSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }

        for (Iterator iter = htmlCardSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }

        for (Iterator iter = pdfListSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }

        for (Iterator iter = pdfCardSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }

        for (Iterator iter = textSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }
    }

    public static SettingsView getView(JFrame parent) {
        Settings settings = getCurrentSettings();
        SettingsView view = new SettingsView(parent, "Report Settings Panel", settings);
        view.setDisclaimer(DataCrow.baseDir + "icons_system/logo.jpg");
        view.setSize(new Dimension(875, 450));
        return view;
    }

    public static Definitions getValueAsDefinitionGroup(String key) {
        return getCurrentSettings().getValueAsDefinitionGroup(key);
    }

    public static String[] getValueAsStringArray(String key) {
        return getCurrentSettings().getValueAsStringArray(key);
    }

    public static int[] getValueAsIntegerArray(String key) {
        return getCurrentSettings().getValueAsIntegerArray(key);
    }

    public static Font getValueAsFont(String key) {
        return getCurrentSettings().getValueAsFont(key);
    }

    public static int getValueAsInt(String key) {
        return getCurrentSettings().getValueAsInt(key);
    }

    public static boolean getValueAsBoolean(String key) {
        return getCurrentSettings().getValueAsBoolean(key);
    }


    public static Color getValueAsColor(String key) {
        return getCurrentSettings().getValueAsColor(key);
    }

    public static String getValueAsString(String key) {
        return getCurrentSettings().getValueAsString(key);
    }

    public static void setValue(String key, Object value) {
        getCurrentSettings().setValue(key, value);
    }

    public static void setStringAsValue(String key, String value) {
        getCurrentSettings().setStringAsValue(key, value);
    }

    protected void createGroups(Settings settings) {
        SettingsGroup reportPageGroup = new SettingsGroup(_ReportPage, "dc.Reporting");
        SettingsGroup reportTextGroup = new SettingsGroup(_ReportText, "dc.Reporting.TextSetting");
        SettingsGroup reportLabelGroup = new SettingsGroup(_ReportLabel, "dc.Reporting.LabelSetting");
        SettingsGroup reportHeaderGroup = new SettingsGroup(_ReportHeader, "dc.Reporting.HeaderSetting");
        SettingsGroup reportFooterGroup = new SettingsGroup(_ReportFooter, "dc.Reporting.FooterSetting");

        reportTextGroup.addChild(new SettingsGroup(_ReportTextFontSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextColorSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextBackgroundColorSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextBorderColor, "dc.Reporting.TextSetting"));

        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelFontSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelColorSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelBackgroundColorSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelBorderColor, "dc.Reporting.LabelSetting"));

        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterFont, "dc.Reporting.FooterSetting"));
        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterColor, "dc.Reporting.FooterSetting"));
        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterBackgroundColor, "dc.Reporting.FooterSetting"));

        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderFont, "dc.Reporting.HeaderSetting"));
        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderColor, "dc.Reporting.HeaderSetting"));
        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderBackgroundColor, "dc.Reporting.HeaderSetting"));

        settings.addGroup(_ReportPage, reportPageGroup);
        settings.addGroup(_ReportText, reportTextGroup);
        settings.addGroup(_ReportLabel, reportLabelGroup);
        settings.addGroup(_ReportHeader, reportHeaderGroup);
        settings.addGroup(_ReportFooter, reportFooterGroup);
    }

    private void createSettings(Settings settings, int reportType) {
        String extension = reportType == _PDF_CARD_REPORT || reportType == _PDF_LIST_REPORT ? "pdf" :
                           reportType == _TEXT_REPORT ? "txt" : "html";
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stFileExtension,
                            extension, -1, "", "", false, false));

        if (reportType != _TEXT_REPORT) {
            if (reportType == _PDF_CARD_REPORT || reportType == _HTML_CARD_REPORT) {
                settings.addSetting(_ReportText,
                        new Setting(Repository.datatypes._BOOLEAN,
                                    Repository.reportsettings.stCreateIndex,
                                    Boolean.TRUE,
                                    ComponentFactory._CHECKBOX,
                                    DcResources.getText("tpCreateIndex"),
                                    DcResources.getText("lblCreateIndex"),
                                    false,
                                    true));
                settings.addSetting(_ReportText,
                        new Setting(Repository.datatypes._INTEGER,
                                    Repository.reportsettings.stLabelPosition,
                                    new Integer(1),
                                    ComponentFactory._REPORTTEXTPOSCOMBO,
                                    DcResources.getText("tpTextPosition"),
                                    DcResources.getText("lblTextPosition"),
                                    true,
                                    true));
                settings.addSetting(_ReportPage,
                        new Setting(Repository.datatypes._BOOLEAN,
                                    Repository.reportsettings.stPageLayoutLandscape,
                                    Boolean.FALSE,
                                    ComponentFactory._CHECKBOX,
                                    "",
                                    DcResources.getText("lblLandscape"),
                                    false,
                                    true));                
            } else {
                settings.addSetting(_ReportPage,
                        new Setting(Repository.datatypes._BOOLEAN,
                                    Repository.reportsettings.stPageLayoutLandscape,
                                    Boolean.TRUE,
                                    ComponentFactory._CHECKBOX,
                                    "",
                                    DcResources.getText("lblLandscape"),
                                    false,
                                    true));                
            }

            settings.addSetting(_ReportTextFontSetting,                           // Group key
                    new Setting(Repository.datatypes._FONT,                       // Data type
                                Repository.reportsettings.stTextFont,             // Key
                                new Font("Arial", Font.PLAIN, 11),                // Default value
                                ComponentFactory._FONTSELECTOR,                   // UI Component
                                DcResources.getText("tpReportTextFont"),          // Help text
                                "",                                               // Label text
                                false,                                            // show label ?
                                true));                                           // Add to panel ?
            settings.addSetting(_ReportTextBackgroundColorSetting,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stTextBackgroundcolor,
                                new Color(233, 247, 221),
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportTextBackground"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportTextColorSetting,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stTextColor,
                                Color.BLACK,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportTextColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportTextBorderColor,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stTextBorderColor,
                                Color.BLACK,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportTextBorderColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportLabelFontSetting,
                    new Setting(Repository.datatypes._FONT,
                                Repository.reportsettings.stLabelFont,
                                new Font("Arial", Font.BOLD, 11),
                                ComponentFactory._FONTSELECTOR,
                                DcResources.getText("tpReportLabelFont"),
                                "",
                                false,
                                true));
            settings.addSetting(_ReportLabelBackgroundColorSetting,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stLabelBackgroundColor,
                                new Color(199, 240, 191),
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportLabelBackground"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportLabelColorSetting,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stLabelColor,
                                Color.BLACK,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportLabelColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportLabelBorderColor,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stLabelBorderColor,
                                Color.BLACK,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportLabelBorderColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportHeader,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stShowHeader,
                                Boolean.TRUE,
                                ComponentFactory._CHECKBOX,
                                DcResources.getText("tpShowHeader"),
                                DcResources.getText("lblShowHeader"),
                                false,
                                true));
            settings.addSetting(_ReportHeader,
                    new Setting(Repository.datatypes._STRING,
                                Repository.reportsettings.stHeaderValue,
                                "Report Generated by Data Crow",
                                ComponentFactory._SHORTTEXTFIELD,
                                DcResources.getText("tpHeaderValue"),
                                DcResources.getText("lblHeaderValue"),
                                true,
                                true));
            settings.addSetting(_ReportFooter,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stShowFooter,
                                Boolean.TRUE,
                                ComponentFactory._CHECKBOX,
                                DcResources.getText("tpShowFooter"),
                                DcResources.getText("lblShowFooter"),
                                false,
                                true));
            settings.addSetting(_ReportFooter,
                    new Setting(Repository.datatypes._STRING,
                                Repository.reportsettings.stFooterValue,
                                "page",
                                ComponentFactory._SHORTTEXTFIELD,
                                DcResources.getText("tpFooterValue"),
                                DcResources.getText("lblFooterValue"),
                                true,
                                true));
            settings.addSetting(_ReportFooterFont,
                    new Setting(Repository.datatypes._FONT,
                                Repository.reportsettings.stFooterFont,
                                new Font("dialog.plain", Font.BOLD, 11),
                                ComponentFactory._FONTSELECTOR,
                                DcResources.getText("tpReportFooterFont"),
                                "",
                                false,
                                true));
            settings.addSetting(_ReportFooterColor,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stFooterColor,
                                Color.BLACK,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportFooterColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportFooterBackgroundColor,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stFooterBackgroundColor,
                                Color.WHITE,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportFooterBackgroundColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportHeaderFont,
                    new Setting(Repository.datatypes._FONT,
                                Repository.reportsettings.stHeaderFont,
                                new Font("dialog.plain", Font.BOLD, 11),
                                ComponentFactory._FONTSELECTOR,
                                DcResources.getText("tpReportHeaderFont"),
                                "",
                                false,
                                true));
            settings.addSetting(_ReportHeaderColor,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stHeaderColor,
                                Color.BLACK,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportHeaderColor"),
                                "",
                                true,
                                true));
            settings.addSetting(_ReportHeaderBackgroundColor,
                    new Setting(Repository.datatypes._COLOR,
                                Repository.reportsettings.stHeaderBackgroundColor,
                                Color.WHITE,
                                ComponentFactory._COLORSELECTOR,
                                DcResources.getText("tpReportHeaderBackgroundColor"),
                                "",
                                true,
                                true));

            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stTextBorderBottom,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblTextBorderBottom"),
                                false,
                                true));
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stTextBorderTop,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblTextBorderTop"),
                                false,
                                true));
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stTextBorderLeft,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblTextBorderLeft"),
                                false,
                                true));
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stTextBorderRight,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblTextBorderRight"),
                                false,
                                true));
            settings.addSetting(_ReportLabel,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stLabelBorderBottom,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblLabelBorderBottom"),
                                false,
                                true));
            settings.addSetting(_ReportLabel,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stLabelBorderTop,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblLabelBorderTop"),
                                false,
                                true));
            settings.addSetting(_ReportLabel,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stLabelBorderLeft,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblLabelBorderLeft"),
                                false,
                                true));
            settings.addSetting(_ReportLabel,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stLabelBorderRight,
                                Boolean.FALSE,
                                ComponentFactory._CHECKBOX,
                                "",
                                DcResources.getText("lblLabelBorderRight"),
                                false,
                                true));
        }

        // INVISIBLE SETTINGS
        if (currentModule != -1) {
            DcModule module = DcModules.getModule(currentModule);
            ReportFieldDefinitions definitions = module.getReportFieldDefinitions();

            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._DEFINITIONGROUP,
                                Repository.reportsettings.stFieldDefinitions,
                                definitions,
                                -1,
                                "",
                                "",
                                false,
                                false));
        }

        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stTarget,
                            "",
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.reportsettings.stReportType,
                            new Integer(0),
                            -1,
                            "",
                            "",
                            false,
                            false));
    }
}