/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings.definitions;

import net.sf.dc.core.modules.DcModule;

public class DcFieldDefinition extends Definition {
    
    private int index;
    private final boolean required;
    private boolean visible;
    private boolean enabled;
    private final String label;
    
    public DcFieldDefinition(int index, String label, boolean enabled, boolean required, boolean visible) {
        this.index = index;
        this.required = required;
        this.visible = visible;
        this.enabled = enabled;
        this.label = label;
    }
    
    public void setIndex(int index) {
        this.index = index;
    }
    
    public int getIndex() {
        return index;
    }
    
    public boolean isEnabled() {
        return enabled;
    }    

    public void isEnabled(boolean b) {
        this.enabled = b;
    }  
    
    public boolean isVisible() {
        return visible;
    }

    public void isVisible(boolean b) {
        this.visible = b;
    } 
    
    public boolean isRequired() {
        return required;
    }
    
    public String getLabel() {
        return label;
    }
    
    public Object[] getDisplayValues(DcModule module) {
        return new Object[] {module.getField(index).getSystemName(),
                             label,
                             new Boolean(enabled),
                             new Boolean(visible),
                             new Boolean(required),
                             module.getField(index)};
    }    
    
    public String toSettingValue() {
        return index + "/&/" + label + "/&/" + enabled + "/&/" + required + "/&/" + visible;
    }
}
