/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings.definitions;

import net.sf.dc.core.modules.DcModule;
import net.sf.dc.resources.DcResources;

public class ReportFieldDefinition extends Definition {

    public final static int _LEFT = 0;
    public final static int _RIGHT = 1;
    public final static int _CENTER = 2;

    private int index = 0;
    private boolean enabled = true;
    private int alignment = 1;
    private int weight = 1;

    public ReportFieldDefinition(int index, boolean enabled, int alignment, int weight) {
        this.index = index;
        this.enabled = enabled;
        this.alignment = alignment;
        this.weight = weight;
    }

    public ReportFieldDefinition(int index, boolean enabled, String alignment, int widthPerc) {
        this.index = index;
        this.enabled = enabled;
        this.alignment = getAlignmentIndex(alignment);
        this.weight = widthPerc;
    }

    public Object[] getDisplayValues(DcModule module) {
        return new Object[] {module.getField(index),
                             new Boolean(enabled),
                             getAlignmentLabel(),
                             new Integer(weight)};
    }

    private int getAlignmentIndex(String label) {
        if (label.equals(DcResources.getText("lblLeft"))) {
            return ReportFieldDefinition._LEFT;
        } else if (label.equals(DcResources.getText("lblRight"))) {
            return ReportFieldDefinition._RIGHT;
        } else {
            return ReportFieldDefinition._CENTER;
        }
    }

    private String getAlignmentLabel() {
        if (alignment == ReportFieldDefinition._LEFT) {
            return DcResources.getText("lblLeft");
        } else if (alignment == ReportFieldDefinition._RIGHT) {
            return DcResources.getText("lblRight");
        } else {
            return DcResources.getText("lblCenter");
        }
    }

    public int getIndex() {
        return index;
    }

    public int getAlignment() {
        return alignment;
    }

    public int getWeight() {
        return weight;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public String toSettingValue() {
        return getIndex() + "/&/" + isEnabled() + "/&/" + getAlignment() + "/&/" + weight;
    }
}
