/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.wf.requests;

import java.util.Collection;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.forms.CategoryForm;
import net.sf.dc.console.forms.StateForm;
import net.sf.dc.console.forms.StorageMediaForm;
import net.sf.dc.console.forms.TemplateForm;
import net.sf.dc.console.software.SoftwarePlatformForm;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.software.SoftwarePlatformModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.resources.DcResources;

public class StatusUpdateRequest implements IRequest {

    private final int module;
    private final int tab;
    private final String msgID;
    private boolean executeOnFail = false;

    public StatusUpdateRequest(int module, int tab, String msgID) {
        this.module = module;
        this.tab = tab;
        this.msgID = msgID;
    }

    public void execute(Collection objects) {
        DcModule mod = DcModules.getModule(module);

        String msg = DcResources.getText(msgID);
        if (mod instanceof DcCategoryModule) {
            CategoryForm form = ((DcCategoryModule) mod).getForm();
            form.setStatus(msg);
        } else if (mod instanceof DcStateModule) {
            StateForm form = ((DcStateModule) mod).getForm();
            form.setStatus(msg);
        } else if (mod instanceof SoftwarePlatformModule) {
            SoftwarePlatformForm form = ((SoftwarePlatformModule) mod).getForm();
            form.setStatus(msg);
        } else if (mod instanceof DcStorageMediaModule) {
            StorageMediaForm form = ((DcStorageMediaModule) mod).getForm();
            form.setStatus(msg); 
        } else if (mod instanceof ITemplateModule) {
            TemplateForm form = ((ITemplateModule) mod).getForm();
            form.setStatus(msg);             
        } else {
            if (tab == MainFrame._INSERTTAB) {
            	mod.getCurrentInsertView().setStatus(msg);
            } else if (tab == MainFrame._SEARCHTAB) {
            	mod.getCurrentSearchView().setStatus(msg);
            }
        }
    }

    public boolean getExecuteOnFail() {
        return executeOnFail;
    }

    public void setExecuteOnFail(boolean b) {
        executeOnFail = b;
    }
}
