/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.jflexsett.settings.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcFontSelector;
import net.sf.dc.console.components.DcReportTextPositionCombo;
import net.sf.dc.core.Repository;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.SettingsGroup;

/**
 * An autimatically filled panel for the settings view.
 * Each panel represents a settingsgroup
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class SettingsPanel extends JPanel {
    
    private SettingsGroup group;
    private Hashtable htComponents = new Hashtable();
    
    public SettingsPanel(SettingsGroup group) {
        this.group = group;
        buildPanel(group.getSettings());
    }
    
    public String getHelpIndex() {
        return group.getHelpIndex();
    }
    
    /**
     * Creates the panel with its components to allow changing the
     * values of the settings. The components are stored in each Setting
     */
    private void buildPanel(Map htSettings) {
        int y = 0;
        this.setLayout(Layout.getGBL());
        
        for (Iterator iter = htSettings.values().iterator(); iter.hasNext(); ) {
            Setting setting = (Setting) iter.next();
            
            if (setting.showToUser()) {
                JLabel label = ComponentFactory.getLabel(setting.getLabelText());
                JComponent comp = setting.getUIComponent();
                
                boolean simplecomp = false;
                if (comp instanceof JComboBox || comp instanceof JTextField || comp instanceof DcCheckBox) {
                    comp.setMinimumSize(new Dimension(200, 21));
                    comp.setPreferredSize(new Dimension(200,21));
                    comp.setMaximumSize(new Dimension(200,21));
                    simplecomp = true;
                }
                
                if (comp instanceof DcFontSelector) {
                    comp.setBorder(ComponentFactory.getTitleBorder(setting.getLabelText()));
                }

                htComponents.put(setting, comp);
                
                if (setting.displayLabel()) {
                    label.setToolTipText(setting.getHelpText());
                    add(label, Layout.getGBC( 0, y, 1, 1, 1.0, 1.0
                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                               new Insets( 5, 5, 5, 5), 0, 0));
                    add(comp,  Layout.getGBC( 1, y, 1, 1, 1.0, 1.0
                              ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                               new Insets( 5, 5, 5, 5), 0, 0));
                } else {
                    int stretch = simplecomp ? GridBagConstraints.HORIZONTAL : GridBagConstraints.BOTH;
                    double weigth = simplecomp ? 1.0 : 10.0;
                    
                    add(comp,  Layout.getGBC( 0, y, 1, 1, weigth, weigth,
                        GridBagConstraints.NORTHWEST, stretch, new Insets( 5, 5, 5, 5), 0, 0));
                }
                y++;
            }
        }
        this.setBorder(new EtchedBorder());
        this.setVisible(false);
    }    

    public String toString() {
        return group.toString();
    }

    public void initializeSettings() {
        for (Enumeration enumerator = htComponents.keys(); enumerator.hasMoreElements(); ) {
            Setting setting = (Setting) enumerator.nextElement();
            Component comp = (Component) htComponents.get(setting);
            Object oValue = setting.getValue();
            if (comp instanceof JPasswordField) {
                ((JPasswordField) comp).setText(oValue.toString());
            } else if (comp instanceof JTextField) {
                ((JTextField) comp).setText(oValue.toString());
            } else if (comp instanceof DcReportTextPositionCombo) {
                ((DcReportTextPositionCombo) comp).setValue(oValue);                
            } else if (comp instanceof JComboBox) {
                ((JComboBox) comp).setSelectedItem(oValue);
            } else if (comp instanceof JEditorPane) {
                ((JEditorPane) comp).setText(oValue.toString());
            } else if (comp instanceof JCheckBox) {
                boolean bValue = ((Boolean) oValue).booleanValue();
                ((JCheckBox) comp).setSelected(bValue);
            } else if (comp instanceof JColorChooser) {
                Color color = (Color) oValue;
                ((JColorChooser) comp).setColor(color);
            } else if (comp instanceof JTextArea) {
                ((JTextArea) comp).setText(oValue.toString());
            } else if (comp instanceof ISettingsComponent) {
                ((ISettingsComponent) comp).setValue(oValue);
            } 
        }
    }
    
    public void saveSettings() {
        for (Enumeration enumerator = htComponents.keys(); enumerator.hasMoreElements(); ) {
            Object oValue = null;
            Setting setting = (Setting) enumerator.nextElement();
            Component comp = (Component) htComponents.get(setting);

            if (comp instanceof JPasswordField) {
                oValue = new String(((JPasswordField) comp).getPassword());
            } else if (comp instanceof JTextField) {
                oValue = ((JTextField) comp).getText();
            } else if (comp instanceof DcReportTextPositionCombo) {
                oValue = ((DcReportTextPositionCombo) comp).getValue().toString();
            } else if (comp instanceof JComboBox) {
                oValue = ((JComboBox) comp).getSelectedItem();
            } else if (comp instanceof JEditorPane) {
                oValue = ((JEditorPane) comp).getText();
            } else if (comp instanceof JCheckBox) {
                oValue = new Boolean(((JCheckBox) comp).isSelected());
            } else if (comp instanceof JColorChooser) {
                oValue = ((JColorChooser) comp).getColor();
            } else if (comp instanceof JTextArea) {
                oValue = ((JTextArea) comp).getText();
            } else if (comp instanceof ISettingsComponent) {
                oValue = ((ISettingsComponent) comp).getValue();   
            }
            
            if (setting.getDataType() == Repository.datatypes._INTEGER) {
                oValue = new Integer(Integer.parseInt((String) oValue));
            }
            
            if (oValue != null) {
                setting.setValue(oValue);
            }
        }
    }
}
